{
Copyright  1998 by Delphi 4 Developer's Guide - Xavier Pacheco and Steve Teixeira
}

unit MainFrm;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics,
  Controls, Forms, Dialogs, Menus;

type
  TMainForm = class(TForm)
    mmMain: TMainMenu;
    mmiShapes: TMenuItem;
    mmiArc: TMenuItem;
    mmiChord: TMenuItem;
    mmiEllipse: TMenuItem;
    mmiPie: TMenuItem;
    mmiPolygon: TMenuItem;
    mmiPolyline: TMenuItem;
    mmiRectangle: TMenuItem;
    mmiRoundRect: TMenuItem;
    N1: TMenuItem;
    mmiFill: TMenuItem;
    mmiUseBitmapPattern: TMenuItem;
    procedure mmiFillClick(Sender: TObject);
    procedure mmiArcClick(Sender: TObject);
    procedure mmiChordClick(Sender: TObject);
    procedure mmiEllipseClick(Sender: TObject);
    procedure mmiUseBitmapPatternClick(Sender: TObject);
    procedure mmiPieClick(Sender: TObject);
    procedure mmiPolygonClick(Sender: TObject);
    procedure mmiPolylineClick(Sender: TObject);
    procedure mmiRectangleClick(Sender: TObject);
    procedure mmiRoundRectClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FBitmap: TBitmap;
  public
    procedure ClearCanvas;
    procedure SetFillPattern;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}


procedure TMainForm.ClearCanvas;
begin
 // Clear the contents of the canvas
  with Canvas do
  begin
    Brush.Style := bsSolid;
    Brush.Color := clWhite;
    FillRect(ClientRect);
  end;
end;

procedure TMainForm.SetFillPattern;
begin
 { Determine if shape is to be draw with a bitmap pattern in which
   case load a bitmap. Otherwise, use the brush pattern. }
  if mmiUseBitmapPattern.Checked then
    Canvas.Brush.Bitmap := FBitmap
  else
    with Canvas.Brush do
    begin
      Bitmap := nil;
      Color := clBlue;
      Style := bsCross;
    end;
end;

procedure TMainForm.mmiFillClick(Sender: TObject);
begin
  mmiFill.Checked := not mmiFill.Checked;
  { If mmiUseBitmapPattern was checked, uncheck it set the
    brush's bitmap to nil. }
  if mmiUseBitmapPattern.Checked then
  begin
    mmiUseBitmapPattern.Checked := not mmiUseBitmapPattern.Checked;
    Canvas.Brush.Bitmap := nil;
  end;
end;

procedure TMainForm.mmiUseBitmapPatternClick(Sender: TObject);
begin
  { Set mmiFil1.Checked mmiUseBitmapPattern.Checked. This will cause
    the SetFillPattern procedure to be called. However, if mmiUseBitmapPattern
    is being set, set Canvas.Brush.Bitmap to nil. }
  mmiUseBitmapPattern.Checked := not mmiUseBitmapPattern.Checked;
  mmiFill.Checked := mmiUseBitmapPattern.Checked;
  if not mmiUseBitmapPattern.Checked then
    Canvas.Brush.Bitmap := nil;
end;

procedure TMainForm.mmiArcClick(Sender: TObject);
begin
  ClearCanvas;
  with ClientRect do
    Canvas.Arc(Left, Top, Right, Bottom, Right, Top, Left, Top);
end;

procedure TMainForm.mmiChordClick(Sender: TObject);
begin
  ClearCanvas;
  with ClientRect do
  begin
    if mmiFill.Checked then
      SetFillPattern;
    Canvas.Chord(Left, Top, Right, Bottom, Right, Top, Left, Top);
  end;
end;

procedure TMainForm.mmiEllipseClick(Sender: TObject);
begin
  ClearCanvas;
  if mmiFill.Checked then
    SetFillPattern;
  Canvas.Ellipse(0, 0, ClientWidth, ClientHeight);
end;

procedure TMainForm.mmiPieClick(Sender: TObject);
begin
  ClearCanvas;
  if mmiFill.Checked then
    SetFillPattern;
  Canvas.Pie(0, 0, ClientWidth, ClientHeight, 50, 5, 300, 50);
end;

procedure TMainForm.mmiPolygonClick(Sender: TObject);
begin
  ClearCanvas;
  if mmiFill.Checked then
    SetFillPattern;
  Canvas.Polygon([Point(0, 0), Point(150, 20), Point(230, 130),
                 Point(40, 120)]);
end;

procedure TMainForm.mmiPolylineClick(Sender: TObject);
begin
  ClearCanvas;
  Canvas.PolyLine([Point(0, 0), Point(120, 30), Point(250, 120),
    Point(140, 200), Point(80, 100), Point(30, 30)]);
end;

procedure TMainForm.mmiRectangleClick(Sender: TObject);
begin
  ClearCanvas;
  if mmiFill.Checked then
    SetFillPattern;
  Canvas.Rectangle(10 , 10, 125, 240);
end;

procedure TMainForm.mmiRoundRectClick(Sender: TObject);
begin
  ClearCanvas;
  if mmiFill.Checked then
    SetFillPattern;
  Canvas.RoundRect(15, 15, 150, 200, 50, 50);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FBitmap := TBitmap.Create;
  FBitMap.LoadFromFile('Pattern.bmp');
  Canvas.Brush.Bitmap := nil;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  FBitmap.Free;
end;

end.
