{
Copyright  1998 by Delphi 4 Developer's Guide - Xavier Pacheco and Steve Teixeira
}

unit MainFrm;

interface

uses
  WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Menus,
  StdCtrls, Messages, Dialogs, SysUtils, ComCtrls,
  ToolWin,  ExtCtrls, Buttons, ImgList;

type
  TMainForm = class(TForm)
    mmMain: TMainMenu;
    OpenDialog: TOpenDialog;
    mmiFile: TMenuItem;
    mmiExit: TMenuItem;
    N3: TMenuItem;
    mmiOpen: TMenuItem;
    mmiNew: TMenuItem;
    mmiWindow: TMenuItem;
    mmiArrangeIcons: TMenuItem;
    mmiCascade: TMenuItem;
    mmiTile: TMenuItem;
    mmiCloseAll: TMenuItem;
    tlbMain: TToolBar;
    ilMain: TImageList;
    tbFileOpen: TToolButton;

    { File Event Handlers }
    procedure mmiNewClick(Sender: TObject);
    procedure mmiOpenClick(Sender: TObject);
    procedure mmiExitClick(Sender: TObject);

    { Window Event Handlers }
    procedure mmiTileClick(Sender: TObject);
    procedure mmiArrangeIconsClick(Sender: TObject);
    procedure mmiCascadeClick(Sender: TObject);
    procedure mmiCloseAllClick(Sender: TObject);
  public
    { User defined methods }
    procedure OpenTextFile(EditForm: TForm; Filename: string);
    procedure OpenBMPFile(FileName: String);
    procedure OpenRTFFile(RTFForm: TForm; FileName: string);
    procedure SetToolBar(AToolBar: TToolBar);
  end;

var
  MainForm: TMainForm;

implementation
uses MDIBmpFrm, MdiEditFrm, MdiRtfFrm, FTypForm;

const
  { Define constants to represent file name extensions }
  BMPExt      = '.BMP';  // Bitmapped file
  TextExt     = '.TXT';  // Text file
  RTFExt      = '.RTF';  // Rich Text Format file

{$R *.DFM}

procedure TMainForm.mmiNewClick(Sender: TObject);
begin
  { Determine the file type the user wishes to open by calling the
    GetFileType function. Call the appropriate method based on the
    retrieved file type. }
  case GetFileType of
    mrTXT: OpenTextFile(nil, ''); // Open a text file.
    mrRTF: OpenRTFFile(nil, '');  // Open an RTF file.
    mrBMP:
     begin
        { Set the defalt filter for OpenDialog1 for BMP files. }
        OpenDialog.FilterIndex := 2;
        mmiOpenClick(nil);
     end;
  end;
end;

procedure TMainForm.mmiOpenClick(Sender: TObject);
var
  Ext: string[4];
begin
  { Call the appropriate method based on the file type of the file
    selected from OpenDialog1 }
  if OpenDialog.Execute then
  begin
    { Get the file's extension and compare it to determine the
      file type the user is opening. Call the appropriate method and
      pass in the file name. }
    Ext := ExtractFileExt(OpenDialog.FileName);
    if CompareStr(UpperCase(Ext), TextExt) = 0 then
      OpenTextFile(ActiveMDIChild, OpenDialog.FileName)
    else if CompareStr(UpperCase(Ext), BMPExt) = 0 then
      OpenBMPFile(OpenDialog.FileName)
    else if CompareStr(UpperCase(Ext), RTFExt) = 0 then
      OpenRTFFile(ActiveMDIChild, OpenDialog.FileName);
  end;
end;

procedure TMainForm.mmiExitClick(Sender: TObject);
begin
  Close;
end;

{ Window Event Handlers }

procedure TMainForm.mmiTileClick(Sender: TObject);
begin
  Tile;
end;

procedure TMainForm.mmiArrangeIconsClick(Sender: TObject);
begin
   ArrangeIcons;
end;

procedure TMainForm.mmiCascadeClick(Sender: TObject);
begin
  Cascade;
end;


procedure TMainForm.mmiCloseAllClick(Sender: TObject);
var
  i: integer;
begin
  { Close all forms in revers order as they appear in the
    MDIChildren property. }
  for i := MdiChildCount - 1 downto 0  do
    MDIChildren[i].Close;
end;

{ User Defined Methods }
procedure TMainForm.OpenTextFile(EditForm: TForm; FileName: string);
begin
  { If EditForm is of a TEditForm type, then give the user the option
   of loading the file contents into this form. Otherwise, create a
   new TEditForm instance and load the file into that instance }
  if (EditForm <> nil) and (EditForm is TMdiEditForm) then
    if MessageDlg('Load file into current form?', mtConfirmation,
      [mbYes, mbNo], 0) = mrYes then
    begin
      TMdiEditForm(EditForm).OpenFile(FileName);
       Exit;
    end;
  { Create a new TEditForm and call its OpenFile() method }
  with TMdiEditForm.Create(self) do
    if FileName <> '' then
      OpenFile(FileName)
end;

procedure TMainForm.OpenRTFFile(RTFForm: TForm; FileName: string);
begin
  { If RTFForm is of a TRTFForm type, then give the user the option
   of loading the file contents into this form. Otherwise, create a
   new TRTFForm instance and load the file into that instance }
  if (RTFForm <> nil) and (RTFForm is TMdiRTFForm) then
    if MessageDlg('Load file into current form?', mtConfirmation,
      [mbYes, mbNo], 0) = mrYes then begin
        (RTFForm as TMdiRTFForm).OpenFile(FileName);
      Exit;
    end;
    { Create a new TRTFForm and call its OpenFile() method }
    with TMdiRTFForm.Create(self) do
      if FileName <> '' then
        OpenFile(FileName);
end;

procedure TMainForm.OpenBMPFile(FileName: String);
begin
  { Create a new TBMPForm instances and load a BMP file into it. }
  with TMdiBmpForm.Create(self) do
    OpenFile(FileName);
end;

procedure TMainForm.SetToolBar(AToolBar: TToolBar);
begin
  if (MDIChildCount = 1) and (AToolBar = nil) then
    tlbMain.Parent := self
  else begin
    tlbMain.Parent := nil;
    if not (AToolBar = nil) then
      AToolBar.Parent := self;
  end;
end;

end.
