{
Copyright  1998 by Delphi 4 Developer's Guide - Xavier Pacheco and Steve Teixeira
}

unit MdiEditFrm;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Menus, ExtCtrls, Buttons, ComCtrls,
  ToolWin, MdiChildFrm, ImgList;

type

  TMdiEditForm = class(TMDIChildForm)
    memMainMemo: TMemo;
    SaveDialog: TSaveDialog;
    FontDialog: TFontDialog;
    mmiEdit: TMenuItem;
    mmiSelectAll: TMenuItem;
    N7: TMenuItem;
    mmiDelete: TMenuItem;
    mmiPaste: TMenuItem;
    mmiCopy: TMenuItem;
    mmiCut: TMenuItem;
    mmiCharacter: TMenuItem;
    mmiFont: TMenuItem;
    N8: TMenuItem;
    mmiWordWrap: TMenuItem;
    N9: TMenuItem;
    mmiCenter: TMenuItem;
    mmiRight: TMenuItem;
    mmiLeft: TMenuItem;
    mmiUndo: TMenuItem;
    N4: TMenuItem;
    mmiBold: TMenuItem;
    mmiItalic: TMenuItem;
    mmiUnderline: TMenuItem;
    PrintDialog: TPrintDialog;

    { File Event Handlers }
    procedure mmiSaveClick(Sender: TObject);
    procedure mmiSaveAsClick(Sender: TObject);

    { Edit Event Handlers }
    procedure mmiCutClick(Sender: TObject);
    procedure mmiCopyClick(Sender: TObject);
    procedure mmiPasteClick(Sender: TObject);
    procedure mmiDeleteClick(Sender: TObject);
    procedure mmiUndoClick(Sender: TObject);
    procedure mmiSelectAllClick(Sender: TObject);

    { Character Event Handlers }
    procedure CharAlignClick(Sender: TObject);
    procedure mmiBoldClick(Sender: TObject);
    procedure mmiItalicClick(Sender: TObject);
    procedure mmiUnderlineClick(Sender: TObject);
    procedure mmiWordWrapClick(Sender: TObject);
    procedure mmiFontClick(Sender: TObject);

    { Form Event Handlers }          
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure mmiPrintClick(Sender: TObject);
  public
    { User Defined Methods }
    procedure OpenFile(FileName: String);
    procedure SetButtons;
  end;

var
  MdiEditForm: TMdiEditForm;

implementation

uses Printers;

{$R *.DFM}
{ File Event Handlers }

procedure TMdiEditForm.mmiSaveClick(Sender: TObject);
begin
  inherited;
  { If there isn't a caption, then there isn't already a filename.
    Therefore, call mmiSaveAsClick since it gets a filename. }
  if Caption = '' then
    mmiSaveAsClick(nil)
  else begin
    { Save to the file specified by the form's Caption. }
    memMainMemo.Lines.SaveToFile(Caption);
    memMainMemo.Modified := false; // Set to false since the text is saved.
  end;
end;

procedure TMdiEditForm.mmiSaveAsClick(Sender: TObject);
begin
  inherited;
  SaveDialog.FileName := Caption;
  if SaveDialog.Execute then
  begin
    { Set caption to filename specified by SaveDialog1 since this
      may have changed. }
    Caption := SaveDialog.FileName;
    mmiSaveClick(nil); // Save the file.
  end;
end;

{ Edit Event Handlers }

procedure TMdiEditForm.mmiCutClick(Sender: TObject);
begin
  inherited;
  memMainMemo.CutToClipBoard;
end;

procedure TMdiEditForm.mmiCopyClick(Sender: TObject);
begin
  inherited;
  memMainMemo.CopyToClipBoard;
end;

procedure TMdiEditForm.mmiPasteClick(Sender: TObject);
begin
  inherited;
  memMainMemo.PasteFromClipBoard;
end;

procedure TMdiEditForm.mmiDeleteClick(Sender: TObject);
begin
  inherited;
  memMainMemo.ClearSelection;
end;

procedure TMdiEditForm.mmiUndoClick(Sender: TObject);
begin
  inherited;
  memMainMemo.Perform(EM_UNDO, 0, 0);
end;

procedure TMdiEditForm.mmiSelectAllClick(Sender: TObject);
begin
  inherited;
  memMainMemo.SelectAll;
end;

{ Character Event Handlers }
procedure TMdiEditForm.CharAlignClick(Sender: TObject);
begin
  inherited;
  mmiLeft.Checked := false;
  mmiRight.Checked := false;
  mmiCenter.Checked := false;

{ TAlignment is defined by VCL as:

  TAlignment = (taLeftJustify, taRightJustify, taCenter);

  Therefore each of the menu items contains the appropriate Tag property
  whose value represents one of the TAlignment values: 0, 1, 2 }

  { If the menu invoked this event handler, set it to checked and
    set the alignment for the memo }
  if Sender is TMenuItem then
  begin
    TMenuItem(Sender).Checked := true;
    memMainMemo.Alignment := TAlignment(TMenuItem(Sender).Tag);
  end
  { If a TToolButton from the main form invoked this event handler,
    set the memo's alignment and then check the appropriate TMenuItem. }
  else if Sender is TToolButton then
  begin
    memMainMemo.Alignment := TAlignment(TToolButton(Sender).Tag);
    case memMainMemo.Alignment of
      taLeftJustify:   mmiLeft.Checked := True;
      taRightJustify:  mmiRight.Checked := True;
      taCenter:        mmiCenter.Checked := True;
    end;
  end;
  SetButtons;
end;

procedure TMdiEditForm.mmiBoldClick(Sender: TObject);
begin
  inherited;
  if not mmiBold.Checked then
    memMainMemo.Font.Style := memMainMemo.Font.Style + [fsBold]
  else
    memMainMemo.Font.Style := memMainMemo.Font.Style - [fsBold];
  SetButtons;
end;

procedure TMdiEditForm.mmiItalicClick(Sender: TObject);
begin
  inherited;
  if not mmiItalic.Checked then
    memMainMemo.Font.Style := memMainMemo.Font.Style + [fsItalic]
  else
    memMainMemo.Font.Style := memMainMemo.Font.Style - [fsItalic];
  SetButtons;
end;

procedure TMdiEditForm.mmiUnderlineClick(Sender: TObject);
begin
  inherited;
  if not mmiUnderline.Checked then
    memMainMemo.Font.Style := memMainMemo.Font.Style + [fsUnderline]
  else
    memMainMemo.Font.Style := memMainMemo.Font.Style - [fsUnderline];
  SetButtons;
end;

procedure TMdiEditForm.mmiWordWrapClick(Sender: TObject);
begin
  inherited;
  with memMainMemo do
  begin
    WordWrap := not WordWrap;
    { Remove scrollbars if Memo1 is wordwrapped since they're not
      required. Otherwise, make sure scrollbars are present. }
    if WordWrap then
      ScrollBars := ssVertical
   else
      ScrollBars := ssBoth;
    mmiWordWrap.Checked := WordWrap;
  end;
end;

procedure TMdiEditForm.mmiFontClick(Sender: TObject);
begin
  inherited;
  FontDialog.Font := memMainMemo.Font;
  if FontDialog.Execute then
    memMainMemo.Font := FontDialog.Font;
end;

{ Form Event Handlers }
procedure TMdiEditForm.FormCloseQuery(Sender: TObject;
     var CanClose: Boolean);
{ This procedure ensures that the user has saved the contents of the
  memo if it was modified since the last time the file was saved. }
const
  CloseMsg = '''%s'' has been modified, Save?';
var
  MsgVal: integer;
  FileName: string;
begin
  inherited;
  FileName := Caption;
  if memMainMemo.Modified then
  begin
    MsgVal := MessageDlg(Format(CloseMsg, [FileName]), mtConfirmation, mbYesNoCancel, 0);
    case MsgVal of
      mrYes:    mmiSaveClick(Self);
      mrCancel: CanClose := false;
    end;
  end;
end;

procedure TMdiEditForm.OpenFile(FileName: string);
begin
  memMainMemo.Lines.LoadFromFile(FileName);
  Caption := FileName;
end;

procedure TMdiEditForm.SetButtons;
{ This procedure ensures that menu items and buttons on the main form
  accurately reflect various settings for the memo. }
begin
  mmiBold.Checked := fsBold in memMainMemo.Font.Style;
  mmiItalic.Checked := fsItalic in memMainMemo.Font.Style;
  mmiUnderLine.Checked := fsUnderline in memMainMemo.Font.Style;

  tbBold.Down      := mmiBold.Checked;
  tbItalic.Down    := mmiItalic.Checked;
  tbUnderline.Down := mmiUnderLine.Checked;
  tbLAlign.Down    := mmiLeft.Checked;
  tbRAlign.Down    := mmiRight.Checked;
  tbCAlign.Down    := mmiCenter.Checked;
end;

procedure TMdiEditForm.mmiPrintClick(Sender: TObject);
var
  i: integer;
  PText: TextFile;
begin
  inherited;
  if PrintDialog.Execute then
  begin
    AssignPrn(PText);
    Rewrite(PText);
    try
      Printer.Canvas.Font := memMainMemo.Font;
      for i := 0 to memMainMemo.Lines.Count -1 do
        writeln(PText, memMainMemo.Lines[i]);
    finally
      CloseFile(PText);
    end;
  end;
  end;

end.

