unit InitExpt;

interface

uses
  SysUtils, Windows, Dialogs, VirtIntf, ExptIntf, ToolIntf;

procedure HandleException;

type
  TSearchExpert = class(TIExpert)
    function GetName: string; override;
    function GetStyle: TExpertStyle; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;
    function GetMenuText: string; override;
    procedure Execute; override;
  end;

function InitExpert(AToolServices: TIToolServices;
  RegisterProc: TExpertRegisterProc;
  var Terminate: TExpertTerminateProc): Boolean; stdcall;

implementation

uses Forms, Controls, Main, PriU;

procedure HandleException;
{ Raise exception within the context of the IDE }
begin
  ToolServices.RaiseException(ReleaseException);
end;

function TSearchExpert.GetName: string;
{ Return name of expert }
begin
  Result := 'DDG Search';
end;

function TSearchExpert.GetStyle: TExpertStyle;
{ Return the style of the expert.  This one is standard. }
begin
  Result := esStandard;
end;

function TSearchExpert.GetState: TExpertState;
{ This expert is always enabled on the menu }
begin
  Result := [esEnabled];
end;

function TSearchExpert.GetIDString: String;
{ Return the unique Vendor.Product name of expert }
begin
  Result := 'DDG.DDGSearch';
end;

function TSearchExpert.GetMenuText: string;
{ Return text for Help menu }
begin
  Result := 'DDG Search Expert';
end;

procedure TSearchExpert.Execute;
{ Called when expert name is selected from Help menu of IDE. }
{ This function invokes the expert }
begin
  try
    // if not created, created it and show it
    if MainForm = nil then
    begin
      MainForm := TMainForm.Create(Application);
      ThreadPriWin := TThreadPriWin.Create(Application);
      MainForm.Show;
    end
    else
    // if created then restore window and show it
      with MainForm do
      begin
        if not Visible then Show;
        if WindowState = wsMinimized then WindowState := wsNormal;
        SetFocus;
      end;
  except
    HandleException;
  end;
end;

function InitExpert(AToolServices: TIToolServices;
  RegisterProc: TExpertRegisterProc;
  var Terminate: TExpertTerminateProc): Boolean; stdcall;
begin
  Result := AToolServices <> nil;
  if Result then
    try
      ToolServices := AToolServices;
      Application.Handle := ToolServices.GetParentHandle;
      RegisterProc(TSearchExpert.Create);
    except
      HandleException;
    end;
end;

end.
