unit InitExpt;

interface

uses
  SysUtils, Windows, VirtIntf, ExptIntf, ToolIntf;

procedure HandleException;

type
  TExpertExpert = class(TIExpert)
    function GetName: string; override;
    function GetComment: string; override;
    function GetGlyph: HICON; override;
    function GetStyle: TExpertStyle; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;
    function GetMenuText: string; override;
    procedure Execute; override;
  end;

function InitExpert(AToolServices: TIToolServices;
  RegisterProc: TExpertRegisterProc;
  var Terminate: TExpertTerminateProc): Boolean; stdcall;

var
  { Registry key where Delphi 4.0 experts are kept }
  SDelphiKey: string;

implementation

uses Forms, Controls, Main;


procedure HandleException;
{ Causes IDE to raise exception }
begin
  ToolServices.RaiseException(ReleaseException);
end;

function TExpertExpert.GetName: string;
{ Return name of expert }
begin
  Result := 'ExpertExpert';
end;

function TExpertExpert.GetComment: string;
{ Return comment used in repository.  Not used for esStandard experts. }
begin
  Result := '';
end;

function TExpertExpert.GetGlyph: HICON;
{ Returns bitmap associated with expert.  Not needed for esStandard. }
begin
  Result := 0;
end;

function TExpertExpert.GetStyle: TExpertStyle;
{ Returns expert style.  This is a standard exepert. }
begin
  Result := esStandard;
end;

function TExpertExpert.GetState: TExpertState;
{ This expert is always enabled }
begin
  Result := [esEnabled];
end;

function TExpertExpert.GetIDString: String;
{ "Vendor.AppName" ID string for expert }
begin
  Result := 'DDG.ExpertExpert';
end;

function TExpertExpert.GetMenuText: string;
{ Menu text for expert }
begin
  Result := 'Expert Expert';
end;

procedure TExpertExpert.Execute;
{ Called when expert is chosen from the main menu. }
{ This procedure creates, shows, and frees the main form. }
begin
  try
    MainForm := TMainForm.Create(Application);
    try
      MainForm.ShowModal;
    finally
      MainForm.Free;
    end;
  except
    HandleException;
  end;
end;

function InitExpert(AToolServices: TIToolServices;
  RegisterProc: TExpertRegisterProc;
  var Terminate: TExpertTerminateProc): Boolean; stdcall;
begin
  Result := AToolServices <> nil;
  if Result then
    try
      ToolServices := AToolServices;
      Application.Handle := ToolServices.GetParentHandle;
      SDelphiKey := ToolServices.GetBaseRegistryKey + '\Experts';
      RegisterProc(TExpertExpert.Create);
    except
      HandleException;
    end;
end;

end.
