unit IBUtils;
//  IBUtils unit -- Contains the CreateDatabase wraper to create
// an InterBase Database.

interface

procedure CreateDatabase(DBName, User, Pswd: string);

implementation

uses DB, SysUtils, Windows, Dialogs, IBase;

function FormulateError(Vector: PInteger): string;
// Create an Interbase error string from a Vector
const
  SMsgStr = 'SQL Code: %d, Message: "%s"';
var
  SqlMsg: array[0..126] of char;
  SqlRslt: Integer;
begin
  SqlRslt := isc_sqlcode(Vector);
  isc_sql_interprete(SqlRslt, SqlMsg, SizeOf(SqlMsg));
  Result := Format(SMsgStr, [SqlRslt, SqlMsg]);
end;

procedure IBCheck(IBResult: Integer; Vector: PInteger);
// Raise an exception if an error occurs
begin
  if IBResult <> 0 then
    raise EDatabaseError.CreateFmt('InterBase Error! ISC# %d'#13#10'%s',
      [IBResult, FormulateError(Vector)]);
end;

procedure CreateDatabase(DBName, User, Pswd: string);
// Creates an InterBase Database.  DBName is the path and database name,
// User is username for the specified server, and Pswd is the password
// for the specified user and server.
const
  SBaseSQL = 'CREATE DATABASE "%s" USER "%s" PASSWORD "%s"';
var
  Status: array[1..19] of Integer;
  db: isc_db_handle;
  tran: isc_tr_handle;
  Rslt: Integer;
  BaseSQL: string;
begin
  // Both Database and Transaction handles must be nil for
  // CREATE DATABASE call
  db := nil;
  tran := nil;
  BaseSQL := Format(SBaseSQL, [DBName, User, Pswd]);
  // Create the specified database, raise exception on error
  Rslt := isc_dsql_execute_immediate(@status, db, tran, 0, PChar(BaseSQL), 1, nil);
  IBCheck(Rslt, @Status);
  // Drop the connection to the newly created Database
  Rslt := isc_detach_database(@status, db);
  IBCheck(Rslt, @Status);
end;

end.
