unit IBase;
//            IBase.pas
//  Interbase OSRI entrypoints and defines
//     Copyright (c) 1997,98 Steve Teixeira and Xavier Pacheco

interface

uses Windows;

const
  ISC_TRUE            = True;
  ISC_FALSE           = False;
  DSQL_close          = 1;
  DSQL_drop           = 2;


{ ***************************************************************** }
{ Blob id structure                                               }
{ ***************************************************************** }

type
  PGdsQuad = ^TGdsQuad;
  TGdsQuad = packed record
    gds_quad_high: Integer;
    gds_quad_low: Integer;
  end;

  PIscQuad = ^TIscQuad;
  TIscQuad = TGdsQuad;

  PIscArrayBound = ^TIscArrayBound;
  TIscArrayBound = packed record
    array_bound_lower: Shortint;
    array_bound_upper: Shortint;
  end;

  PIscArrayDesc = ^TIscArrayDesc;
  TIscArrayDesc = packed record
    array_desc_dtype: Byte;
    array_desc_scale: Char;
    array_desc_length: Word;
    array_desc_field_name: packed array[0..31] of Char;
    array_desc_relation_name: packed array[0..31] of Char;
    array_desc_dimensions: Shortint;
    array_desc_flags: Shortint;
    array_desc_bounds: packed array[0..15] of TIscArrayBound;
  end;

  PIscBlobDesc = ^TIscBlobDesc;
  TIscBlobDesc = packed record
    blob_desc_subtype: Shortint;
    blob_desc_charset: Shortint;
    blob_desc_segment_size: Shortint;
    blob_desc_field_name: packed array[0..31] of Byte;
    blob_desc_relation_name: packed array[0..31] of Byte;
  end;

{ ************************* }
{ Blob control structure  }
{ ************************* }

  PIscBlobCtl = ^TIscBlobCtl;
  TIscBlobCtl = packed record
    ctl_source: function: Integer; stdcall;  { Source filter }
    ctl_source_handle: PIscBlobCtl;          { Argument to pass to source filter }
    ctl_to_sub_type: Shortint;               { Target type }
    ctl_from_sub_type: Shortint;             { Source type }
    ctl_buffer_length: Word;                 { Length of buffer }
    ctl_segment_length: Word;                { Length of current segment }
    ctl_bpb_length: Word;                    { Length of blob parameter block }
    ctl_bpb: PChar;                          { Address of blob parameter block }
    ctl_buffer: PByte;                       { Address of segment buffer }
    ctl_max_segment: Integer;                { Length of longest segment }
    ctl_number_segments: Integer;            { Total number of segments }
    ctl_total_length: Integer;               { Total length of blob }
    ctl_status: PInteger;                    { Address of status vector }
    ctl_data: packed array[0..7] of Longint; { Application specific data }
  end;

{ ************************* }
{ Blob stream definitions }
{ ************************* }

  PBStream = ^TBStream;
  TBStream = packed record
    bstr_blob: Pointer;                 { Blob handle }
    bstr_buffer: PChar;                 { Address of buffer }
    bstr_ptr: PChar;                    { Next character }
    bstr_length: Shortint;              { Length of buffer }
    bstr_cnt: Shortint;                 { Characters in buffer }
    bstr_mode: Char;                    { (mode) ? OUTPUT : INPUT }
  end;

function GetB(var S: TBStream): Integer;
function PutB(C: Char; var S: TBStream): Integer;
function PutBX(C: Char; var  S: TBStream): Integer;

{ ************************* }
{ Dynamic SQL definitions }
{ ************************* }

type
  PShortInt = ^ShortInt;

  PSQLVar = ^TSQLVar;
  TSQLVar = packed record
    sqltype: Shortint;
    sqllen: Shortint;
    sqldata: PChar;
    sqlind: PShortint;
    sqlname_length: Shortint;
    sqlname: packed array[0..29] of Char;
  end;

  PSQLDA = ^TSQLDA;
  TSQLDA = packed record
    sqldaid: packed array[0..7] of Char;
    sqldabc: Integer;
    sqln: Shortint;
    sqld: Shortint;
    sqlvar: packed array[0..0] of TSQLVar;
  end;

function SQLDA_Length(n: Integer): Integer;

{ **************************** }
{ Declare the extended SQLDA }
{ **************************** }

type
  PXSQLVar = ^TXSQLVar;
  TXSQLVar = packed record
    sqltype: Shortint;                  { datatype of field }
    sqlscale: Shortint;                 { scale factor }
    sqlsubtype: Shortint;               { datatype subtype - BLOBs & Text types only }
    sqllen: Shortint;                   { length of data area }
    sqldata: PChar;                     { address of data }
    sqlind: PShortint;                  { address of indicator variable }
    sqlname_length: Shortint;           { length of sqlname field }
    sqlname: packed array[0..31] of Char; { name of field, name length + space for NULL }
    relname_length: Shortint;           { length of relation name }
    relname: packed array[0..31] of Char; { field's relation name + space for NULL }
    ownname_length: Shortint;           { length of owner name }
    ownname: packed array[0..31] of Char; { relation's owner name + space for NULL }
    aliasname_length: Shortint;         { length of alias name }
    aliasname: packed array[0..31] of Char; { relation's alias name + space for NULL }
  end;

  PXSQLDA = ^TXSQLDA;
  TXSQLDA = packed record
    version: Shortint;                      { version of this XSQLDA }
    sqldaid: packed array[0..7] of Char;    { XSQLDA name field }
    sqldabc: Integer;                       { length in bytes of SQLDA }
    sqln: Shortint;                         { number of fields allocated }
    sqld: Shortint;                         { actual number of fields }
    sqlvar: packed array[0..0] of TXSQLVar; { first field address }
  end;

function XSQLDA_LENGTH(n: Integer): Integer;

{ ****************************** }
{ InterBase Handle Definitions }
{ ****************************** }

type
  isc_att_handle = Pointer;
  isc_blob_handle = Pointer;
  isc_db_handle = Pointer;
  isc_form_handle = Pointer;
  isc_req_handle = Pointer;
  isc_stmt_handle = Pointer;
  isc_svc_handle = Pointer;
  isc_tr_handle = Pointer;
  isc_win_handle = Pointer;

  TIscCallback = procedure; stdcall;

{ ************************* }
{ OSRI database functions }
{ ************************* }

function isc_attach_database(p1: PInteger; p2: Shortint; p3: PChar;
  var p4: isc_db_handle; p5: Shortint; p6: PChar): Integer; stdcall;

function isc_array_gen_sdl(p1: PInteger; p2: PIscArrayDesc; p3: PShortint;
  p4: PChar; p5: PShortint): Integer; stdcall;

function isc_array_get_slice(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: PIscQuad; p5: PIscArrayDesc; Pntr: Pointer;
  p7: PInteger): Integer; stdcall;

function isc_array_lookup_bounds(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: PChar; p5: PChar; p6: PIscArrayDesc): Integer; stdcall;

function isc_array_lookup_desc(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: PChar; p5: PChar; p6: PIscArrayDesc): Integer; stdcall;

function isc_array_set_desc(p1: PInteger; p2: PChar; p3: PChar;
  p4: PShortint; p5: PShortint; p6: PShortint;
  p7: PIscArrayDesc): Integer; stdcall;

function isc_array_put_slice(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: PIscQuad; p5: PIscArrayDesc; Pntr: Pointer;
  p7: PInteger): Integer; stdcall;

procedure isc_blob_default_desc(p1: PIscBlobDesc; p2: PByte; p3: PByte); stdcall;

function isc_blob_gen_bpb(p1: PInteger; p2: PIscBlobDesc;
  p3: PIscBlobDesc; p4: Word; p5: PByte; p6: PWord): Integer; stdcall;

function isc_blob_info(p1: PInteger; var p2: isc_blob_handle; p3: Shortint;
  p4: PChar; p5: Shortint; p6: PChar): Integer; stdcall;

function isc_blob_lookup_desc(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: PByte; p5: PByte; p6: PIscBlobDesc;
  p7: PByte): Integer; stdcall;

function isc_blob_set_desc(p1: PInteger; p2: PByte; p3: PByte; p4: Shortint;
  p5: Shortint; p6: Shortint; p7: PIscBlobDesc): Integer; stdcall;

function isc_cancel_blob(p1: PInteger; var p2: isc_blob_handle): Integer; stdcall;

function isc_cancel_events(p1: PInteger; var p2: isc_db_handle;
  p3: PInteger): Integer; stdcall;

function isc_close_blob(p1: PInteger; var p2: isc_blob_handle): Integer; stdcall;

function isc_commit_retaining(p1: PInteger; var p2: isc_tr_handle): Integer; stdcall;

function isc_commit_transaction(p1: PInteger;
  var p2: isc_tr_handle): Integer; stdcall;

function isc_create_blob(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; var p4: isc_blob_handle; p5: PIscQuad): Integer; stdcall;

function isc_create_blob2(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; var p4: isc_blob_handle; p5: PIscQuad; p6: Shortint;
  p7: PChar): Integer; stdcall;

function isc_create_database(p1: PInteger; p2: Shortint; p3: PChar;
  var p4: isc_db_handle; p5: Shortint; p6: PChar; p7: Shortint): Integer; stdcall;

function isc_database_info(p1: PInteger; var p2: isc_db_handle; p3: Shortint;
  p4: PChar; p5: Shortint; p6: PChar): Integer; stdcall;

procedure isc_decode_date(p1: PIscQuad; Pntr: Pointer); stdcall;

function isc_detach_database(p1: PInteger; var p2: isc_db_handle): Integer; stdcall;

function isc_drop_database(p1: PInteger; var p2: isc_db_handle): Integer; stdcall;

function isc_dsql_allocate_statement(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_stmt_handle): Integer; stdcall;

function isc_dsql_alloc_statement2(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_stmt_handle): Integer; stdcall;

function isc_dsql_describe(p1: PInteger; var p2: isc_stmt_handle; p3: Word;
  p4: PXSQLDA): Integer; stdcall;

function isc_dsql_describe_bind(p1: PInteger; var p2: isc_stmt_handle; p3: Word;
  p4: PXSQLDA): Integer; stdcall;

function isc_dsql_exec_immed2(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: Word; p5: PChar; p6: Word; p7: PXSQLDA;
  p8: PXSQLDA): Integer; stdcall;

function isc_dsql_execute(p1: PInteger; var p2: isc_tr_handle;
  var p3: isc_stmt_handle; p4: Word; p5: PXSQLDA): Integer; stdcall;

function isc_dsql_execute2(p1: PInteger; var p2: isc_tr_handle;
  var p3: isc_stmt_handle; p4: Word; p5, P6: PXSQLDA): Integer; stdcall;

function isc_dsql_execute_immediate(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: Word; p5: PChar; p6: Word; p7: PXSQLDA): Integer; stdcall;

function isc_dsql_fetch(p1: PInteger; var p2: isc_stmt_handle; p3: Word;
  p4: PXSQLDA): Integer; stdcall;

function isc_dsql_fetch2(p1: PInteger; var p2: isc_stmt_handle; p3: Word;
  p5: PXSQLDA; p6: Word; p7: Integer): Integer; stdcall;

function isc_dsql_finish(var p1: isc_db_handle): Integer; stdcall;

function isc_dsql_free_statement(p1: PInteger; var p2: isc_stmt_handle;
  p3: Word): Integer; stdcall;

function isc_dsql_insert(p1: PInteger; var p2: isc_stmt_handle; p3: Word;
  p4: PXSQLDA): Integer; stdcall;

function isc_dsql_prepare(p1: PInteger; var p2: isc_tr_handle;
  var p3: isc_stmt_handle; p4: Word; p5: PChar; p6: Word;
  p7: PXSQLDA): Integer; stdcall;

function isc_dsql_set_cursor_name(p1: PInteger; var p2: isc_stmt_handle;
  p3: PChar; p4: Word): Integer; stdcall;

function isc_dsql_sql_info(p1: PInteger; var p2: isc_stmt_handle; p3: Shortint;
  p4: PChar; p5: Shortint; p6: PChar): Integer; stdcall;

procedure isc_encode_date(Pntr: Pointer; p2: PIscQuad); stdcall;

procedure isc_event_counts(p1: PInteger; p2: Shortint; p3: PChar;
  p4: PChar); stdcall;

function isc_modify_dpb(var p1: PChar; p2: PShortint; p3: Word; p4: PChar;
  p5: Shortint): Integer; stdcall;

function isc_free(p1: PChar): Integer; stdcall;

function isc_get_segment(p1: PInteger; var p2: isc_blob_handle; p3: PWord;
  p4: Word; p5: PChar): Integer; stdcall;

function isc_get_slice(p1: PInteger; var p2: isc_db_handle; var p3: isc_tr_handle;
  p4: PIscQuad; p5: Shortint; p6: PChar; p7: Shortint; p8: PInteger;
  p9: Integer; Pntr: Pointer; p11: PInteger): Integer; stdcall;

function isc_interprete(p1: PChar; var p2: PInteger): Integer; stdcall;

function isc_open_blob(p1: PInteger; var p2: isc_db_handle; var p3: isc_tr_handle;
  var p4: isc_blob_handle; p5: PIscQuad): Integer; stdcall;

function isc_open_blob2(p1: PInteger; var p2: isc_db_handle; var p3: isc_tr_handle;
  var p4: isc_blob_handle; p5: PIscQuad; p6: Shortint; p7: PChar): Integer; stdcall;

function isc_prepare_transaction2(p1: PInteger; var p2: isc_tr_handle;
  p3: Shortint; p4: PChar): Integer; stdcall;

procedure isc_print_sqlerror(p1: Shortint; p2: PInteger); stdcall;

function isc_print_status(p1: PInteger): Integer; stdcall;

function isc_put_segment(p1: PInteger; var p2: isc_blob_handle; p3: Word;
  p4: PChar): Integer; stdcall;

function isc_put_slice(p1: PInteger; var p2: isc_db_handle; var p3: isc_tr_handle;
  p4: PIscQuad; p5: Shortint; p6: PChar; p7: Shortint; p8: PInteger;
  p9: Integer; Pntr: Pointer): Integer; stdcall;

function isc_que_events(p1: PInteger; var p2: isc_db_handle; p3: PInteger;
  p4: Shortint; p5: PChar; p6: TIscCallback; Pntr: Pointer): Integer; stdcall;

function isc_rollback_transaction(p1: PInteger;
  var p2: isc_tr_handle): Integer; stdcall;

function isc_start_multiple(p1: PInteger; var p2: isc_tr_handle; p3: Shortint;
  Pntr: Pointer): Integer; stdcall;

function isc_sqlcode(p1: PInteger): Integer; stdcall;

procedure isc_sql_interprete(p1: Shortint; p2: PChar; p3: Shortint); stdcall;

function isc_transaction_info(p1: PInteger; var p2: isc_tr_handle; p3: Shortint;
  p4: PChar; p5: Shortint; p6: PChar): Integer; stdcall;

function isc_transact_request(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: Word; p5: PChar; p6: Word; p7: PChar; p8: Word;
  p9: PChar): Integer; stdcall;

function isc_vax_integer(p1: PChar; p2: Shortint): Integer; stdcall;


{ ******************************** }
{  Other OSRI functions          }
{ ******************************** }

function isc_compile_request(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_req_handle; p4: Shortint; p5: PChar): Integer; stdcall;

function isc_compile_request2(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_req_handle; p4: Shortint; p5: PChar): Integer; stdcall;

function isc_ddl(p1: PInteger; var p2: isc_db_handle; var p3: isc_tr_handle;
  p4: Shortint; p5: PChar): Integer; stdcall;

function isc_prepare_transaction(p1: PInteger; var p2: isc_tr_handle): Integer; stdcall;

function isc_receive(p1: PInteger; var p2: isc_req_handle; p3: Shortint;
  p4: Shortint; Pntr: Pointer; p6: Shortint): Integer; stdcall;

function isc_reconnect_transaction(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: Shortint; p5: PChar): Integer; stdcall;

function isc_release_request(p1: PInteger; var p2: isc_req_handle): Integer; stdcall;

function isc_request_info(p1: PInteger; var p2: isc_req_handle; p3: Shortint;
  p4: Shortint; p5: PChar; p6: Shortint; p7: PChar): Integer; stdcall;

function isc_seek_blob(p1: PInteger; var p2: isc_blob_handle; p3: Shortint;
  p4: Integer; p5: PInteger): Integer; stdcall;

function isc_send(p1: PInteger; var p2: isc_req_handle; p3: Shortint;
  p4: Shortint; Pntr: Pointer; p6: Shortint): Integer; stdcall;

function isc_start_and_send(p1: PInteger; var p2: isc_req_handle;
  var p3: isc_tr_handle; p4: Shortint; p5: Shortint; Pntr: Pointer;
  p7: Shortint): Integer; stdcall;

function isc_start_request(p1: PInteger; var p2: isc_req_handle;
  var p3: isc_tr_handle; p4: Shortint): Integer; stdcall;

function isc_unwind_request(p1: PInteger; var p2: isc_tr_handle;
  p3: Shortint): Integer; stdcall;

function isc_wait_for_event(p1: PInteger; var p2: isc_db_handle; p3: Shortint;
  p4: PChar; p5: PChar): Integer; stdcall;

{ *************************** }
{ Other Sql functions       }
{ *************************** }

function isc_close(p1: PInteger; p2: PChar): Integer; stdcall;

function isc_declare(p1: PInteger; p2: PChar; p3: PChar): Integer; stdcall;

function isc_describe(p1: PInteger; p2: PChar; p3: PSQLDA): Integer; stdcall;

function isc_describe_bind(p1: PInteger; p2: PChar; p3: PSQLDA): Integer; stdcall;

function isc_execute(p1: PInteger; var p2: isc_tr_handle; p3: PChar;
  p4: PSQLDA): Integer; stdcall;

function isc_execute_immediate(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: PShortint; p5: PChar): Integer; stdcall;

function isc_fetch(p1: PInteger; p2: PChar; p3: PSQLDA): Integer; stdcall;

function isc_open(p1: PInteger; var p2: isc_tr_handle; p3: PChar;
  p4: PSQLDA): Integer; stdcall;

function isc_prepare(p1: PInteger; var p2: isc_db_handle; var p3: isc_tr_handle;
  p4: PChar; p5: PShortint; p6: PChar; p7: PSQLDA): Integer; stdcall;

{ *********************************** }
{ Other Dynamic sql functions       }
{ *********************************** }

function isc_dsql_execute_m(p1: PInteger; var p2: isc_tr_handle;
  var p3: isc_stmt_handle; p4: Word; p5: PChar; p6: Word; p7: Word;
  p8: PChar): Integer; stdcall;

function isc_dsql_execute2_m(p1: PInteger; var p2: isc_tr_handle;
  var p3: isc_stmt_handle; p4: Word; p5: PChar; p6: Word; p7: Word; p8: PChar;
  p9: Word; p10: PChar; p11: Word; p12: Word; p13: PChar): Integer; stdcall;

function isc_dsql_execute_immediate_m(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: Word; p5: PChar; p6: Word; p7: Word; p8: PChar;
  p9: Word; p10: Word; p11: PChar): Integer; stdcall;

function isc_dsql_exec_immed3_m(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: Word; p5: PChar; p6: Word; p7: Word; p8: PChar;
  p9: Word; p10: Word; p11: PChar; p12: Word; p13: PChar; p14: Word; p15: Word;
  p16: PChar): Integer; stdcall;

function isc_dsql_fetch_m(p1: PInteger; var p2: isc_stmt_handle; p3: Word;
  p4: PChar; p5: Word; p6: Word; p7: PChar): Integer; stdcall;

function isc_dsql_fetch2_m(p1: PInteger; var p2: isc_stmt_handle; p3: Word;
  p4: PChar; p5: Word; p6: Word; p7: PChar; p8: Word;
  p9: Integer): Integer; stdcall;

function isc_dsql_insert_m(p1: PInteger; var p2: isc_stmt_handle; p3: Word;
  p4: PChar; p5: Word; p6: Word; p7: PChar): Integer; stdcall;

function isc_dsql_prepare_m(p1: PInteger; var p2: isc_tr_handle;
  var p3: isc_stmt_handle; p4: Word; p5: PChar; p6: Word; p7: Word; p8: PChar;
  p9: Word; p10: PChar): Integer; stdcall;

function isc_dsql_release(p1: PInteger; p2: PChar): Integer; stdcall;

function isc_embed_dsql_close(p1: PInteger; p2: PChar): Integer; stdcall;

function isc_embed_dsql_declare(p1: PInteger; p2: PChar;
  p3: PChar): Integer; stdcall;

function isc_embed_dsql_describe(p1: PInteger; p2: PChar; p3: Word;
  p4: PXSQLDA): Integer; stdcall;

function isc_embed_dsql_describe_bind(p1: PInteger; p2: PChar; p3: Word;
  p4: PXSQLDA): Integer; stdcall;

function isc_embed_dsql_execute(p1: PInteger; var p2: isc_tr_handle; p3: PChar;
  p4: Word; p5: PXSQLDA): Integer; stdcall;

function isc_embed_dsql_execute2(p1: PInteger; var p2: isc_tr_handle; p3: PChar;
  p4: Word; p5: PXSQLDA; p6: PXSQLDA): Integer; stdcall;

function isc_embed_dsql_execute_immed(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: Word; p5: PChar; p6: Word; p7: PXSQLDA): Integer; stdcall;

function isc_embed_dsql_fetch(p1: PInteger; p2: PChar; p3: Word;
  p4: PXSQLDA): Integer; stdcall;

function isc_embed_dsql_fetch2(p1: PInteger; p2: PChar; p3: Word;
  p4: PXSQLDA; p5: Word; p6: Integer): Integer; stdcall;

function isc_embed_dsql_open(p1: PInteger; var p2: isc_tr_handle; p3: PChar;
  p4: Word; p5: PXSQLDA): Integer; stdcall;

function isc_embed_dsql_open2(p1: PInteger; var p2: isc_tr_handle; p3: PChar;
  p4: Word; p5: PXSQLDA; p6: PXSQLDA): Integer; stdcall;

function isc_embed_dsql_insert(p1: PInteger; p2: PChar; p3: Word;
  p4: PXSQLDA): Integer; stdcall;

function isc_embed_dsql_prepare(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; p4: PChar; p5: Word; p6: PChar; p7: Word;
  p8: PXSQLDA): Integer; stdcall;

function isc_embed_dsql_release(p1: PInteger; p2: PChar): Integer; stdcall;

{ **************************** }
{ Other Blob functions       }
{ **************************** }

function BLOB_open(p1: isc_blob_handle; p2: PChar; p3: Integer): TBStream; stdcall;

function BLOB_put(p1: Char; p2: PBSTREAM): Integer; stdcall;

function BLOB_close(p1: PBSTREAM): Integer; stdcall;

function BLOB_get(p1: PBSTREAM): Integer; stdcall;

function BLOB_display(p1: PIscQuad; var p2: isc_db_handle; p3: isc_tr_handle;
  p4: PChar): Integer; stdcall;

function BLOB_dump(p1: PIscQuad; var p2: isc_db_handle; p3: isc_tr_handle;
  p4: PChar): Integer; stdcall;

function BLOB_edit(p1: PIscQuad; var p2: isc_db_handle; p3: isc_tr_handle;
  p4: PChar): Integer; stdcall;

function BLOB_load(p1: PIscQuad; var p2: isc_db_handle; p3: isc_tr_handle;
  p4: PChar): Integer; stdcall;

function BLOB_text_dump(p1: PIscQuad; var p2: isc_db_handle; p3: isc_tr_handle;
  p4: PChar): Integer; stdcall;

function BLOB_text_load(p1: PIscQuad; var p2: isc_db_handle; p3: isc_tr_handle;
  p4: PChar): Integer; stdcall;

function Bopen(p1: PIscQuad; var p2: isc_db_handle; p3: isc_tr_handle;
  p4: PChar): TBStream; stdcall;

function Bopen2(p1: PIscQuad; var p2: isc_db_handle; p3: isc_tr_handle; p4: PChar;
  p5: Word): TBStream; stdcall;

{ **************************** }
{ Other Misc functions       }
{ **************************** }

function isc_ftof(p1: PChar; p2: Word; p3: PChar; p4: Word): Integer; stdcall;

function isc_print_blr(p1: PChar; p2: TIscCallback; Pntr: Pointer;
  p4: Shortint): Integer; stdcall;

procedure isc_set_debug(p1: Integer); stdcall;

procedure isc_qtoq(p1: PIscQuad; p2: PIscQuad); stdcall;

procedure isc_vtof(p1: PChar; p2: PChar; p3: Word); stdcall;

procedure isc_vtov(p1: PChar; p2: PChar; p3: Shortint); stdcall;

function isc_version(var p1: isc_db_handle; p2: TIscCallback; Pntr: Pointer): Integer; stdcall;

{ *************************************** }
{ Service manager functions             }
{ *************************************** }

function isc_attach_service(p1: PInteger; p2: Word; p3: PChar;
  var p4: isc_svc_handle; p5: Word; p6: PChar): Integer; stdcall;

function isc_detach_service(p1: PInteger; var p2: isc_svc_handle): Integer; stdcall;

function isc_query_service(p1: PInteger; var p2: isc_svc_handle; p3: Word;
  p4: PChar; p5: Word; p6: PChar; p7: Word; p8: PChar): Integer; stdcall;

{ ***************************** }
{ Forms functions             }
{ ***************************** }

function isc_compile_map(p1: PInteger; var p2: isc_form_handle;
  var p3: isc_req_handle; p4: PShortint; p5: PChar): Integer; stdcall;

function isc_compile_menu(p1: PInteger; var p2: isc_form_handle;
  var p3: isc_req_handle; p4: PShortint; p5: PChar): Integer; stdcall;

function isc_compile_sub_map(p1: PInteger; var p2: isc_win_handle;
  var p3: isc_req_handle; p4: PShortint; p5: PChar): Integer; stdcall;

function isc_create_window(p1: PInteger; var p2: isc_win_handle; p3: PShortint;
  p4: PChar; p5: PShortint; p6: PShortint): Integer; stdcall;

function isc_delete_window(p1: PInteger; var p2: isc_win_handle): Integer; stdcall;

function isc_drive_form(p1: PInteger; var p2: isc_db_handle; var p3: isc_tr_handle;
  var p4: isc_win_handle; var p5: isc_req_handle; p6: PByte; p7: PByte): Integer; stdcall;

function isc_drive_menu(p1: PInteger; var p2: isc_win_handle;
  var p3: isc_req_handle; p4: PShortint; p5: PChar; p6: PShortint; p7: PChar;
  p8: PShortint; p9: PShortint; p10: PChar; p11: PInteger): Integer; stdcall;

function isc_form_delete(p1: PInteger; var p2: isc_form_handle): Integer; stdcall;

function isc_form_fetch(p1: PInteger; var p2: isc_db_handle; var p3: isc_tr_handle;
  var p4: isc_req_handle; p5: PByte): Integer; stdcall;

function isc_form_insert(p1: PInteger; var p2: isc_db_handle;
  var p3: isc_tr_handle; var p4: isc_req_handle; p5: PByte): Integer; stdcall;

function isc_get_entree(p1: PInteger; var p2: isc_req_handle; p3: PShortint;
  p4: PChar; p5: PInteger; p6: PShortint): Integer; stdcall;

function isc_initialize_menu(p1: PInteger; var p2: isc_req_handle): Integer; stdcall;

function isc_menu(p1: PInteger; var p2: isc_win_handle; var p3: isc_req_handle;
  p4: PShortint; p5: PChar): Integer; stdcall;

function isc_load_form(p1: PInteger; var p2: isc_db_handle; var p3: isc_tr_handle;
  var p4: isc_form_handle; p5: PShortint; p6: PChar): Integer; stdcall;

function isc_pop_window(p1: PInteger; var p2: isc_win_handle): Integer; stdcall;

function isc_put_entree(p1: PInteger; var p2: isc_req_handle; p3: PShortint;
  p4: PChar; p5: PInteger): Integer; stdcall;

function isc_reset_form(p1: PInteger; var p2: isc_req_handle): Integer; stdcall;

function isc_suspend_window(p1: PInteger; var p2: isc_win_handle): Integer; stdcall;

{ ************************************************* }
{ Actions to pass to the blob filter (ctl_source) }
{ ************************************************* }

const
  isc_blob_filter_open                 = 0;
  isc_blob_filter_get_segment          = 1;
  isc_blob_filter_close                = 2;
  isc_blob_filter_create               = 3;
  isc_blob_filter_put_segment          = 4;
  isc_blob_filter_alloc                = 5;
  isc_blob_filter_free                 = 6;
  isc_blob_filter_seek                 = 7;

{ ***************** }
{ Blr definitions }
{ ***************** }

  blr_text                               = 14;
  blr_text2                              = 15;
  blr_short                              = 7;
  blr_long                               = 8;
  blr_quad                               = 9;
  blr_float                              = 10;
  blr_double                             = 27;
  blr_d_float                            = 11;
  blr_date                               = 35;
  blr_varying                            = 37; 
  blr_varying2                           = 38;
  blr_blob                               = 261; 
  blr_cstring                            = 40; 
  blr_cstring2                           = 41; 
  blr_blob_id                            = 45; 

  blr_inner                              = 0; 
  blr_left                               = 1; 
  blr_right                              = 2; 
  blr_full                               = 3;

  blr_gds_code                           = 0; 
  blr_sql_code                           = 1; 
  blr_exception                          = 2; 
  blr_trigger_code                       = 3; 
  blr_default_code                       = 4; 

  blr_version4                           = 4;
  blr_eoc                                = 76; 
  blr_end                                = -1; 

  blr_assignment                         = 1; 
  blr_begin                              = 2; 
  blr_dcl_variable                       = 3; 
  blr_message                            = 4; 
  blr_erase                              = 5; 
  blr_fetch                              = 6; 
  blr_for                                = 7; 
  blr_if                                 = 8;
  blr_loop                               = 9; 
  blr_modify                             = 10; 
  blr_handler                            = 11; 
  blr_receive                            = 12; 
  blr_select                             = 13; 
  blr_send                               = 14; 
  blr_store                              = 15; 
  blr_label                              = 17; 
  blr_leave                              = 18; 
  blr_store2                             = 19; 
  blr_post                               = 20; 

  blr_literal                            = 21; 
  blr_dbkey                              = 22; 
  blr_field                              = 23; 
  blr_fid                                = 24; 
  blr_parameter                          = 25; 
  blr_variable                           = 26; 
  blr_average                            = 27;
  blr_count                              = 28; 
  blr_maximum                            = 29; 
  blr_minimum                            = 30; 
  blr_total                              = 31; 
  blr_add                                = 34; 
  blr_subtract                           = 35; 
  blr_multiply                           = 36; 
  blr_divide                             = 37; 
  blr_negate                             = 38; 
  blr_concatenate                        = 39; 
  blr_substring                          = 40; 
  blr_parameter2                         = 41; 
  blr_from                               = 42; 
  blr_via                                = 43; 
  blr_user_name                          = 44; 
  blr_null                               = 45; 

  blr_eql                                = 47; 
  blr_neq                                = 48;
  blr_gtr                                = 49; 
  blr_geq                                = 50; 
  blr_lss                                = 51; 
  blr_leq                                = 52; 
  blr_containing                         = 53; 
  blr_matching                           = 54; 
  blr_starting                           = 55; 
  blr_between                            = 56; 
  blr_or                                 = 57; 
  blr_and                                = 58; 
  blr_not                                = 59; 
  blr_any                                = 60; 
  blr_missing                            = 61; 
  blr_unique                             = 62; 
  blr_like                               = 63; 

  blr_stream                             = 65; 
  blr_set_index                          = 66; 
  blr_rse                                = 67;
  blr_first                              = 68; 
  blr_project                            = 69; 
  blr_sort                               = 70; 
  blr_boolean                            = 71; 
  blr_ascending                          = 72; 
  blr_descending                         = 73; 
  blr_relation                           = 74; 
  blr_rid                                = 75; 
  blr_union                              = 76; 
  blr_map                                = 77; 
  blr_group_by                           = 78; 
  blr_aggregate                          = 79; 
  blr_join_type                          = 80; 

  blr_agg_count                          = 83; 
  blr_agg_max                            = 84; 
  blr_agg_min                            = 85; 
  blr_agg_total                          = 86; 
  blr_agg_average                        = 87;
  blr_parameter3                         = 88; 
  blr_run_count                          = 118; 
  blr_run_max                            = 89; 
  blr_run_min                            = 90; 
  blr_run_total                          = 91; 
  blr_run_average                        = 92; 
  blr_agg_count2                         = 93; 
  blr_agg_count_distinct                 = 94; 
  blr_agg_total_distinct                 = 95; 
  blr_agg_average_distinct               = 96; 

  blr_function                           = 100; 
  blr_gen_id                             = 101; 
  blr_prot_mask                          = 102; 
  blr_upcase                             = 103; 
  blr_lock_state                         = 104; 
  blr_value_if                           = 105; 
  blr_matching2                          = 106; 
  blr_index                              = 107;
  blr_ansi_like                          = 108; 
  blr_bookmark                           = 109; 
  blr_crack                              = 110; 
  blr_force_crack                        = 111; 
  blr_seek                               = 112; 
  blr_find                               = 113; 

  blr_forward                            = 0; 
  blr_backward                           = 1; 
  blr_bof_forward                        = 2; 
  blr_eof_backward                       = 3; 
  blr_forward_starting                   = 4; 
  blr_backward_starting                  = 5; 

  blr_lock_relation                      = 114; 
  blr_lock_record                        = 115; 
  blr_set_bookmark                       = 116; 
  blr_get_bookmark                       = 117; 
  blr_rs_stream                          = 119;
  blr_exec_proc                          = 120; 
  blr_begin_range                        = 121; 
  blr_end_range                          = 122; 
  blr_delete_range                       = 123; 
  blr_procedure                          = 124; 
  blr_pid                                = 125; 
  blr_exec_pid                           = 126; 
  blr_singular                           = 127; 
  blr_abort                              = 128; 
  blr_block                              = 129; 
  blr_error_handler                      = 130; 
  blr_cast                               = 131; 
  blr_release_lock                       = 132; 
  blr_release_locks                      = 133; 
  blr_start_savepoint                    = 134; 
  blr_end_savepoint                      = 135; 
  blr_find_dbkey                         = 136; 
  blr_range_relation                     = 137; 
  blr_delete_ranges                      = 138;

  blr_plan                               = 139; 
  blr_merge                              = 140; 
  blr_join                               = 141; 
  blr_sequential                         = 142; 
  blr_navigational                       = 143; 
  blr_indices                            = 144; 
  blr_retrieve                           = 145; 

  blr_relation2                          = 146; 
  blr_rid2                               = 147; 
  blr_reset_stream                       = 148; 
  blr_release_bookmark                   = 149; 
  blr_set_generator                      = 150; 
  blr_ansi_any                           = 151; 
  blr_exists                             = 152; 
  blr_cardinality                        = 153; 

  blr_record_version                     = 154;         { get tid of record }
  blr_stall                              = 155;         { fake server stall }
  blr_seek_no_warn                       = 156;
  blr_find_dbkey_version                 = 157;
  blr_ansi_all                           = 158;

{ ******************************** }
{ Database parameter block stuff }
{ ******************************** }

  isc_dpb_version1                      = 1; 
  isc_dpb_cdd_pathname                  = 1; 
  isc_dpb_allocation                    = 2; 
  isc_dpb_journal                       = 3; 
  isc_dpb_page_size                     = 4; 
  isc_dpb_num_buffers                   = 5; 
  isc_dpb_buffer_length                 = 6; 
  isc_dpb_debug                         = 7; 
  isc_dpb_garbage_collect               = 8; 
  isc_dpb_verify                        = 9; 
  isc_dpb_sweep                         = 10; 
  isc_dpb_enable_journal                = 11; 
  isc_dpb_disable_journal               = 12; 
  isc_dpb_dbkey_scope                   = 13; 
  isc_dpb_number_of_users               = 14; 
  isc_dpb_trace                         = 15;
  isc_dpb_no_garbage_collect            = 16; 
  isc_dpb_damaged                       = 17; 
  isc_dpb_license                       = 18; 
  isc_dpb_sys_user_name                 = 19; 
  isc_dpb_encrypt_key                   = 20; 
  isc_dpb_activate_shadow               = 21; 
  isc_dpb_sweep_interval                = 22; 
  isc_dpb_delete_shadow                 = 23; 
  isc_dpb_force_write                   = 24; 
  isc_dpb_begin_log                     = 25; 
  isc_dpb_quit_log                      = 26; 
  isc_dpb_no_reserve                    = 27; 
  isc_dpb_user_name                     = 28; 
  isc_dpb_password                      = 29; 
  isc_dpb_password_enc                  = 30; 
  isc_dpb_sys_user_name_enc             = 31; 
  isc_dpb_interp                        = 32; 
  isc_dpb_online_dump                   = 33; 
  isc_dpb_old_file_size                 = 34;
  isc_dpb_old_num_files                 = 35; 
  isc_dpb_old_file                      = 36; 
  isc_dpb_old_start_page                = 37; 
  isc_dpb_old_start_seqno               = 38; 
  isc_dpb_old_start_file                = 39; 
  isc_dpb_drop_walfile                  = 40; 
  isc_dpb_old_dump_id                   = 41; 
  isc_dpb_wal_backup_dir                = 42; 
  isc_dpb_wal_chkptlen                  = 43; 
  isc_dpb_wal_numbufs                   = 44; 
  isc_dpb_wal_bufsize                   = 45; 
  isc_dpb_wal_grp_cmt_wait              = 46; 
  isc_dpb_lc_messages                   = 47; 
  isc_dpb_lc_ctype                      = 48; 
  isc_dpb_cache_manager                 = 49; 
  isc_dpb_shutdown                      = 50; 
  isc_dpb_online                        = 51; 
  isc_dpb_shutdown_delay                = 52; 
  isc_dpb_reserved                      = 53;
  isc_dpb_overwrite                     = 54; 
  isc_dpb_sec_attach                    = 55; 
  isc_dpb_disable_wal                   = 56; 


{ ******************************* }
{ isc_dpb_verify specific flags }
{ ******************************* }

  isc_dpb_pages                         = 1; 
  isc_dpb_records                       = 2; 
  isc_dpb_indices                       = 4; 
  isc_dpb_transactions                  = 8; 
  isc_dpb_no_update                     = 16; 
  isc_dpb_repair                        = 32; 
  isc_dpb_ignore                        = 64; 

{ ********************************* }
{ isc_dpb_shutdown specific flags }
{ ********************************* }

  isc_dpb_shut_cache                   = 1; 
  isc_dpb_shut_attachment              = 2; 
  isc_dpb_shut_transaction             = 4; 
  isc_dpb_shut_force                   = 8; 

{ ************************************ }
{ Bit assignments in RDB$SYSTEM_FLAG }
{ ************************************ }

  RDB_system                             = 1; 
  RDB_id_assigned                        = 2; 

{ *********************************** }
{ Transaction parameter block stuff }
{ *********************************** }

  isc_tpb_version1                      = 1;
  isc_tpb_version3                      = 3; 
  isc_tpb_consistency                   = 1; 
  isc_tpb_concurrency                   = 2; 
  isc_tpb_shared                        = 3; 
  isc_tpb_protected                     = 4; 
  isc_tpb_exclusive                     = 5; 
  isc_tpb_wait                          = 6; 
  isc_tpb_nowait                        = 7; 
  isc_tpb_read                          = 8; 
  isc_tpb_write                         = 9; 
  isc_tpb_lock_read                     = 10; 
  isc_tpb_lock_write                    = 11; 
  isc_tpb_verb_time                     = 12; 
  isc_tpb_commit_time                   = 13; 
  isc_tpb_ignore_limbo                  = 14; 
  isc_tpb_read_committed                = 15; 
  isc_tpb_autocommit                    = 16; 
  isc_tpb_rec_version                   = 17; 
  isc_tpb_no_rec_version                = 18;
  isc_tpb_restart_requests              = 19; 
  isc_tpb_no_auto_undo                  = 20; 

{ ********************** }
{ Blob Parameter Block }
{ ********************** }

  isc_bpb_version1                      = 1; 
  isc_bpb_source_type                   = 1; 
  isc_bpb_target_type                   = 2; 
  isc_bpb_type                          = 3; 
  isc_bpb_source_interp                 = 4; 
  isc_bpb_target_interp                 = 5; 
  isc_bpb_filter_parameter              = 6; 

  isc_bpb_type_segmented                = 0; 
  isc_bpb_type_stream                   = 1; 

{ ******************************* }
{ Service parameter block stuff }
{ ******************************* }

  isc_spb_version1                      = 1; 
  isc_spb_user_name                     = 2; 
  isc_spb_sys_user_name                 = 3; 
  isc_spb_sys_user_name_enc             = 4; 
  isc_spb_password                      = 5; 
  isc_spb_password_enc                  = 6; 
  isc_spb_command_line                  = 7; 

{ ******************************* }
{ Information call declarations }
{ ******************************* }

{ ************************** }
{ Common, structural codes }
{ ************************** }

  isc_info_end                          = 1; 
  isc_info_truncated                    = 2; 
  isc_info_error                        = 3; 

{ **************************** }
{ Database information items }
{ **************************** }

  isc_info_db_id                        = 4; 
  isc_info_reads                        = 5; 
  isc_info_writes                       = 6; 
  isc_info_fetches                      = 7; 
  isc_info_marks                        = 8; 
  isc_info_implementation               = 11; 
  isc_info_version                      = 12; 
  isc_info_base_level                   = 13; 
  isc_info_page_size                    = 14; 
  isc_info_num_buffers                  = 15; 
  isc_info_limbo                        = 16;
  isc_info_current_memory               = 17; 
  isc_info_max_memory                   = 18; 
  isc_info_window_turns                 = 19; 
  isc_info_license                      = 20; 
  isc_info_allocation                   = 21; 
  isc_info_attachment_id                = 22; 
  isc_info_read_seq_count               = 23; 
  isc_info_read_idx_count               = 24; 
  isc_info_insert_count                 = 25; 
  isc_info_update_count                 = 26; 
  isc_info_delete_count                 = 27; 
  isc_info_backout_count                = 28; 
  isc_info_purge_count                  = 29; 
  isc_info_expunge_count                = 30; 
  isc_info_sweep_interval               = 31; 
  isc_info_ods_version                  = 32; 
  isc_info_ods_minor_version            = 33; 
  isc_info_no_reserve                   = 34; 
  isc_info_logfile                      = 35;
  isc_info_cur_logfile_name             = 36; 
  isc_info_cur_log_part_offset          = 37; 
  isc_info_num_wal_buffers              = 38; 
  isc_info_wal_buffer_size              = 39; 
  isc_info_wal_ckpt_length              = 40; 
  isc_info_wal_cur_ckpt_interval        = 41; 
  isc_info_wal_prv_ckpt_fname           = 42; 
  isc_info_wal_prv_ckpt_poffset         = 43; 
  isc_info_wal_recv_ckpt_fname          = 44; 
  isc_info_wal_recv_ckpt_poffset        = 45; 
  isc_info_wal_grpc_wait_usecs          = 47; 
  isc_info_wal_num_io                   = 48; 
  isc_info_wal_avg_io_size              = 49; 
  isc_info_wal_num_commits              = 50; 
  isc_info_wal_avg_grpc_size            = 51; 
  isc_info_forced_writes                = 52; 
  isc_info_user_names                   = 53; 
  isc_info_page_errors                  = 54; 
  isc_info_record_errors                = 55;
  isc_info_bpage_errors                 = 56; 
  isc_info_dpage_errors                 = 57; 
  isc_info_ipage_errors                 = 58; 
  isc_info_ppage_errors                 = 59; 
  isc_info_tpage_errors                 = 60; 

{ ************************************ }
{ Database information return values }
{ ************************************ }

  isc_info_db_impl_rdb_vms              = 1; 
  isc_info_db_impl_rdb_eln              = 2; 
  isc_info_db_impl_rdb_eln_dev          = 3; 
  isc_info_db_impl_rdb_vms_y            = 4; 
  isc_info_db_impl_rdb_eln_y            = 5; 
  isc_info_db_impl_jri                  = 6; 
  isc_info_db_impl_jsv                  = 7; 
  isc_info_db_impl_isc_a                = 25; 
  isc_info_db_impl_isc_u                = 26;
  isc_info_db_impl_isc_v                = 27; 
  isc_info_db_impl_isc_s                = 28; 
  isc_info_db_impl_isc_apl_68K          = 25; 
  isc_info_db_impl_isc_vax_ultr         = 26; 
  isc_info_db_impl_isc_vms              = 27; 
  isc_info_db_impl_isc_sun_68k          = 28; 
  isc_info_db_impl_isc_os2              = 29; 
  isc_info_db_impl_isc_sun4             = 30; 
  isc_info_db_impl_isc_hp_ux            = 31; 
  isc_info_db_impl_isc_sun_386i         = 32; 
  isc_info_db_impl_isc_vms_orcl         = 33; 
  isc_info_db_impl_isc_mac_aux          = 34; 
  isc_info_db_impl_isc_rt_aix           = 35; 
  isc_info_db_impl_isc_mips_ult         = 36; 
  isc_info_db_impl_isc_xenix            = 37; 
  isc_info_db_impl_isc_dg               = 38; 
  isc_info_db_impl_isc_hp_mpexl         = 39; 
  isc_info_db_impl_isc_hp_ux68K         = 40; 
  isc_info_db_impl_isc_sgi              = 41;
  isc_info_db_impl_isc_sco_unix         = 42; 
  isc_info_db_impl_isc_cray             = 43; 
  isc_info_db_impl_isc_imp              = 44; 
  isc_info_db_impl_isc_delta            = 45; 
  isc_info_db_impl_isc_next             = 46; 
  isc_info_db_impl_isc_dos              = 47; 
  isc_info_db_impl_isc_winnt            = 48; 
  isc_info_db_impl_isc_epson            = 49; 

  isc_info_db_class_access              = 1; 
  isc_info_db_class_y_valve             = 2; 
  isc_info_db_class_rem_int             = 3; 
  isc_info_db_class_rem_srvr            = 4; 
  isc_info_db_class_pipe_int            = 7; 
  isc_info_db_class_pipe_srvr           = 8; 
  isc_info_db_class_sam_int             = 9; 
  isc_info_db_class_sam_srvr            = 10; 
  isc_info_db_class_gateway             = 11; 
  isc_info_db_class_cache               = 12;

{ *************************** }
{ Request information items }
{ *************************** }

  isc_info_number_messages              = 4; 
  isc_info_max_message                  = 5; 
  isc_info_max_send                     = 6; 
  isc_info_max_receive                  = 7; 
  isc_info_state                        = 8; 
  isc_info_message_number               = 9; 
  isc_info_message_size                 = 10; 
  isc_info_request_cost                 = 11; 
  isc_info_access_path                  = 12; 
  isc_info_req_select_count             = 13; 
  isc_info_req_insert_count             = 14; 
  isc_info_req_update_count             = 15; 
  isc_info_req_delete_count             = 16; 

{ ******************* }
{ Access path items }
{ ******************* }

  isc_info_rsb_end                       = 0; 
  isc_info_rsb_begin                     = 1; 
  isc_info_rsb_type                      = 2; 
  isc_info_rsb_relation                  = 3; 
  isc_info_rsb_plan                      = 4; 

{ *********** }
{ Rsb types }
{ *********** }

  isc_info_rsb_unknown                   = 1; 
  isc_info_rsb_indexed                   = 2; 
  isc_info_rsb_navigate                  = 3; 
  isc_info_rsb_sequential                = 4; 
  isc_info_rsb_cross                     = 5;
  isc_info_rsb_sort                      = 6; 
  isc_info_rsb_first                     = 7; 
  isc_info_rsb_boolean                   = 8; 
  isc_info_rsb_union                     = 9; 
  isc_info_rsb_aggregate                = 10; 
  isc_info_rsb_merge                    = 11; 
  isc_info_rsb_ext_sequential           = 12; 
  isc_info_rsb_ext_indexed              = 13; 
  isc_info_rsb_ext_dbkey                = 14; 
  isc_info_rsb_left_cross               = 15; 
  isc_info_rsb_select                   = 16; 
  isc_info_rsb_sql_join                 = 17; 
  isc_info_rsb_simulate                 = 18; 
  isc_info_rsb_sim_cross                = 19; 
  isc_info_rsb_once                     = 20; 
  isc_info_rsb_procedure                = 21; 

{ ******************** }
{ Bitmap expressions }
{ ******************** }

  isc_info_rsb_and                    = 1; 
  isc_info_rsb_or                     = 2; 
  isc_info_rsb_dbkey                  = 3; 
  isc_info_rsb_index                  = 4; 

  isc_info_req_active                   = 2; 
  isc_info_req_inactive                 = 3; 
  isc_info_req_send                     = 4; 
  isc_info_req_receive                  = 5; 
  isc_info_req_select                   = 6; 
  isc_info_req_sql_stall                = 7; 

{ ************************ }
{ Blob information items }
{ ************************ }

  isc_info_blob_num_segments            = 4;
  isc_info_blob_max_segment             = 5; 
  isc_info_blob_total_length            = 6; 
  isc_info_blob_type                    = 7; 

{ ******************************* }
{ Transaction information items }
{ ******************************* }

  isc_info_tra_id                       = 4; 

{ *************************** }
{ Service information items }
{ *************************** }

  isc_info_svc_version                  = 4; 
  isc_info_svc_message                  = 5; 
  isc_info_svc_total_length             = 6; 
  isc_info_svc_response                 = 7; 
  isc_info_svc_response_more            = 8;
  isc_info_svc_line                     = 9; 
  isc_info_svc_to_eof                   = 10; 
  isc_info_svc_timeout                  = 11; 
  isc_info_svc_server_version           = 12; 
  isc_info_svc_implementation           = 13; 
  isc_info_svc_capabilities             = 14; 
  isc_info_svc_user_dbpath              = 15; 
  isc_info_svc_svr_db_info              = 16; 
  isc_info_svc_svr_online               = 17; 
  isc_info_svc_svr_offline              = 18; 
  isc_info_svc_get_config               = 19; 
  isc_info_svc_set_config               = 20; 
  isc_info_svc_default_config           = 21; 

{ *********************** }
{ SQL information items }
{ *********************** }

  isc_info_sql_select                   = 4;
  isc_info_sql_bind                     = 5; 
  isc_info_sql_num_variables            = 6; 
  isc_info_sql_describe_vars            = 7; 
  isc_info_sql_describe_end             = 8; 
  isc_info_sql_sqlda_seq                = 9; 
  isc_info_sql_message_seq              = 10; 
  isc_info_sql_type                     = 11; 
  isc_info_sql_sub_type                 = 12; 
  isc_info_sql_scale                    = 13; 
  isc_info_sql_length                   = 14; 
  isc_info_sql_null_ind                 = 15; 
  isc_info_sql_field                    = 16; 
  isc_info_sql_relation                 = 17; 
  isc_info_sql_owner                    = 18; 
  isc_info_sql_alias                    = 19; 
  isc_info_sql_sqlda_start              = 20; 
  isc_info_sql_stmt_type                = 21; 
  isc_info_sql_get_plan                 = 22; 
  isc_info_sql_records                  = 23;
  isc_info_sql_batch_fetch              = 24; 

{ ******************************* }
{ SQL information return values }
{ ******************************* }

  isc_info_sql_stmt_select              = 1; 
  isc_info_sql_stmt_insert              = 2; 
  isc_info_sql_stmt_update              = 3; 
  isc_info_sql_stmt_delete              = 4; 
  isc_info_sql_stmt_ddl                 = 5; 
  isc_info_sql_stmt_get_segment         = 6; 
  isc_info_sql_stmt_put_segment         = 7; 
  isc_info_sql_stmt_exec_procedure      = 8; 
  isc_info_sql_stmt_start_trans         = 9; 
  isc_info_sql_stmt_commit              = 10; 
  isc_info_sql_stmt_rollback            = 11; 
  isc_info_sql_stmt_select_for_upd      = 12; 
  isc_info_sql_stmt_set_generator       = 13;

{ ********************************* }
{ Server configuration key values }
{ ********************************* }

  ISCCFG_LOCKMEM_KEY          = 0; 
  ISCCFG_LOCKSEM_KEY          = 1; 
  ISCCFG_LOCKSIG_KEY          = 2; 
  ISCCFG_EVNTMEM_KEY          = 3; 
  ISCCFG_DBCACHE_KEY          = 4; 
  ISCCFG_PRIORITY_KEY         = 5; 
  ISCCFG_IPCMAP_KEY           = 6; 
  ISCCFG_MEMMIN_KEY           = 7; 
  ISCCFG_MEMMAX_KEY           = 8; 
  ISCCFG_LOCKORDER_KEY        = 9; 
  ISCCFG_ANYLOCKMEM_KEY       = 10; 
  ISCCFG_ANYLOCKSEM_KEY       = 11; 
  ISCCFG_ANYLOCKSIG_KEY       = 12; 
  ISCCFG_ANYEVNTMEM_KEY       = 13;

{ ************* }
{ Error codes }
{ ************* }

  isc_facility                           = 20; 
  isc_err_base                           = 335544320; 
  isc_err_factor                         = 1; 
  isc_arg_end                            = 0; 
  isc_arg_gds                            = 1; 
  isc_arg_string                         = 2; 
  isc_arg_cstring                        = 3; 
  isc_arg_number                         = 4; 
  isc_arg_interpreted                    = 5; 
  isc_arg_vms                            = 6; 
  isc_arg_unix                           = 7; 
  isc_arg_domain                         = 8; 
  isc_arg_dos                            = 9; 
  isc_arg_mpexl                          = 10;
  isc_arg_mpexl_ipc                      = 11; 
  isc_arg_next_mach                      = 15; 
  isc_arg_netware                        = 16; 
  isc_arg_win32                          = 17; 

  isc_arith_except                      = 335544321; 
  isc_bad_dbkey                         = 335544322; 
  isc_bad_db_format                     = 335544323; 
  isc_bad_db_handle                     = 335544324; 
  isc_bad_dpb_content                   = 335544325; 
  isc_bad_dpb_form                      = 335544326; 
  isc_bad_req_handle                    = 335544327; 
  isc_bad_segstr_handle                 = 335544328; 
  isc_bad_segstr_id                     = 335544329; 
  isc_bad_tpb_content                   = 335544330; 
  isc_bad_tpb_form                      = 335544331; 
  isc_bad_trans_handle                  = 335544332; 
  isc_bug_check                         = 335544333; 
  isc_convert_error                     = 335544334;
  isc_db_corrupt                        = 335544335; 
  isc_deadlock                          = 335544336; 
  isc_excess_trans                      = 335544337; 
  isc_from_no_match                     = 335544338; 
  isc_infinap                           = 335544339; 
  isc_infona                            = 335544340; 
  isc_infunk                            = 335544341; 
  isc_integ_fail                        = 335544342; 
  isc_invalid_blr                       = 335544343; 
  isc_io_error                          = 335544344; 
  isc_lock_conflict                     = 335544345; 
  isc_metadata_corrupt                  = 335544346; 
  isc_not_valid                         = 335544347; 
  isc_no_cur_rec                        = 335544348; 
  isc_no_dup                            = 335544349; 
  isc_no_finish                         = 335544350; 
  isc_no_meta_update                    = 335544351; 
  isc_no_priv                           = 335544352; 
  isc_no_recon                          = 335544353;
  isc_no_record                         = 335544354; 
  isc_no_segstr_close                   = 335544355; 
  isc_obsolete_metadata                 = 335544356; 
  isc_open_trans                        = 335544357; 
  isc_port_len                          = 335544358; 
  isc_read_only_field                   = 335544359; 
  isc_read_only_rel                     = 335544360; 
  isc_read_only_trans                   = 335544361; 
  isc_read_only_view                    = 335544362; 
  isc_req_no_trans                      = 335544363; 
  isc_req_sync                          = 335544364; 
  isc_req_wrong_db                      = 335544365; 
  isc_segment                           = 335544366; 
  isc_segstr_eof                        = 335544367; 
  isc_segstr_no_op                      = 335544368; 
  isc_segstr_no_read                    = 335544369; 
  isc_segstr_no_trans                   = 335544370; 
  isc_segstr_no_write                   = 335544371; 
  isc_segstr_wrong_db                   = 335544372;
  isc_sys_request                       = 335544373; 
  isc_stream_eof                        = 335544374; 
  isc_unavailable                       = 335544375; 
  isc_unres_rel                         = 335544376; 
  isc_uns_ext                           = 335544377; 
  isc_wish_list                         = 335544378; 
  isc_wrong_ods                         = 335544379; 
  isc_wronumarg                         = 335544380; 
  isc_imp_exc                           = 335544381; 
  isc_random                            = 335544382; 
  isc_fatal_conflict                    = 335544383; 
  isc_badblk                            = 335544384; 
  isc_invpoolcl                         = 335544385; 
  isc_nopoolids                         = 335544386; 
  isc_relbadblk                         = 335544387; 
  isc_blktoobig                         = 335544388; 
  isc_bufexh                            = 335544389; 
  isc_syntaxerr                         = 335544390; 
  isc_bufinuse                          = 335544391;
  isc_bdbincon                          = 335544392; 
  isc_reqinuse                          = 335544393; 
  isc_badodsver                         = 335544394; 
  isc_relnotdef                         = 335544395; 
  isc_fldnotdef                         = 335544396; 
  isc_dirtypage                         = 335544397; 
  isc_waifortra                         = 335544398; 
  isc_doubleloc                         = 335544399; 
  isc_nodnotfnd                         = 335544400; 
  isc_dupnodfnd                         = 335544401; 
  isc_locnotmar                         = 335544402; 
  isc_badpagtyp                         = 335544403; 
  isc_corrupt                           = 335544404; 
  isc_badpage                           = 335544405; 
  isc_badindex                          = 335544406; 
  isc_dbbnotzer                         = 335544407; 
  isc_tranotzer                         = 335544408; 
  isc_trareqmis                         = 335544409; 
  isc_badhndcnt                         = 335544410;
  isc_wrotpbver                         = 335544411; 
  isc_wroblrver                         = 335544412; 
  isc_wrodpbver                         = 335544413; 
  isc_blobnotsup                        = 335544414; 
  isc_badrelation                       = 335544415; 
  isc_nodetach                          = 335544416; 
  isc_notremote                         = 335544417; 
  isc_trainlim                          = 335544418; 
  isc_notinlim                          = 335544419; 
  isc_traoutsta                         = 335544420; 
  isc_connect_reject                    = 335544421; 
  isc_dbfile                            = 335544422; 
  isc_orphan                            = 335544423; 
  isc_no_lock_mgr                       = 335544424; 
  isc_ctxinuse                          = 335544425; 
  isc_ctxnotdef                         = 335544426; 
  isc_datnotsup                         = 335544427; 
  isc_badmsgnum                         = 335544428; 
  isc_badparnum                         = 335544429;
  isc_virmemexh                         = 335544430; 
  isc_blocking_signal                   = 335544431; 
  isc_lockmanerr                        = 335544432; 
  isc_journerr                          = 335544433; 
  isc_keytoobig                         = 335544434; 
  isc_nullsegkey                        = 335544435; 
  isc_sqlerr                            = 335544436; 
  isc_wrodynver                         = 335544437; 
  isc_funnotdef                         = 335544438; 
  isc_funmismat                         = 335544439; 
  isc_bad_msg_vec                       = 335544440; 
  isc_bad_detach                        = 335544441; 
  isc_noargacc_read                     = 335544442; 
  isc_noargacc_write                    = 335544443; 
  isc_read_only                         = 335544444; 
  isc_ext_err                           = 335544445; 
  isc_non_updatable                     = 335544446; 
  isc_no_rollback                       = 335544447; 
  isc_bad_sec_info                      = 335544448;
  isc_invalid_sec_info                  = 335544449; 
  isc_misc_interpreted                  = 335544450; 
  isc_update_conflict                   = 335544451; 
  isc_unlicensed                        = 335544452; 
  isc_obj_in_use                        = 335544453; 
  isc_nofilter                          = 335544454; 
  isc_shadow_accessed                   = 335544455; 
  isc_invalid_sdl                       = 335544456; 
  isc_out_of_bounds                     = 335544457; 
  isc_invalid_dimension                 = 335544458; 
  isc_rec_in_limbo                      = 335544459; 
  isc_shadow_missing                    = 335544460; 
  isc_cant_validate                     = 335544461; 
  isc_cant_start_journal                = 335544462; 
  isc_gennotdef                         = 335544463; 
  isc_cant_start_logging                = 335544464; 
  isc_bad_segstr_type                   = 335544465; 
  isc_foreign_key                       = 335544466; 
  isc_high_minor                        = 335544467;
  isc_tra_state                         = 335544468; 
  isc_trans_invalid                     = 335544469; 
  isc_buf_invalid                       = 335544470; 
  isc_indexnotdefined                   = 335544471; 
  isc_login                             = 335544472; 
  isc_invalid_bookmark                  = 335544473; 
  isc_bad_lock_level                    = 335544474; 
  isc_relation_lock                     = 335544475; 
  isc_record_lock                       = 335544476; 
  isc_max_idx                           = 335544477; 
  isc_jrn_enable                        = 335544478; 
  isc_old_failure                       = 335544479; 
  isc_old_in_progress                   = 335544480; 
  isc_old_no_space                      = 335544481; 
  isc_no_wal_no_jrn                     = 335544482; 
  isc_num_old_files                     = 335544483; 
  isc_wal_file_open                     = 335544484; 
  isc_bad_stmt_handle                   = 335544485; 
  isc_wal_failure                       = 335544486;
  isc_walw_err                          = 335544487; 
  isc_logh_small                        = 335544488; 
  isc_logh_inv_version                  = 335544489; 
  isc_logh_open_flag                    = 335544490; 
  isc_logh_open_flag2                   = 335544491; 
  isc_logh_diff_dbname                  = 335544492; 
  isc_logf_unexpected_eof               = 335544493; 
  isc_logr_incomplete                   = 335544494; 
  isc_logr_header_small                 = 335544495; 
  isc_logb_small                        = 335544496; 
  isc_wal_illegal_attach                = 335544497; 
  isc_wal_invalid_wpb                   = 335544498; 
  isc_wal_err_rollover                  = 335544499; 
  isc_no_wal                            = 335544500; 
  isc_drop_wal                          = 335544501; 
  isc_stream_not_defined                = 335544502; 
  isc_wal_subsys_error                  = 335544503; 
  isc_wal_subsys_corrupt                = 335544504; 
  isc_no_archive                        = 335544505;
  isc_shutinprog                        = 335544506; 
  isc_range_in_use                      = 335544507; 
  isc_range_not_found                   = 335544508; 
  isc_charset_not_found                 = 335544509; 
  isc_lock_timeout                      = 335544510; 
  isc_prcnotdef                         = 335544511; 
  isc_prcmismat                         = 335544512; 
  isc_wal_bugcheck                      = 335544513; 
  isc_wal_cant_expand                   = 335544514; 
  isc_codnotdef                         = 335544515; 
  isc_xcpnotdef                         = 335544516; 
  isc_except                            = 335544517; 
  isc_cache_restart                     = 335544518; 
  isc_bad_lock_handle                   = 335544519; 
  isc_jrn_present                       = 335544520; 
  isc_wal_err_rollover2                 = 335544521; 
  isc_wal_err_logwrite                  = 335544522; 
  isc_wal_err_jrn_comm                  = 335544523; 
  isc_wal_err_expansion                 = 335544524;
  isc_wal_err_setup                     = 335544525; 
  isc_wal_err_ww_sync                   = 335544526; 
  isc_wal_err_ww_start                  = 335544527; 
  isc_shutdown                          = 335544528; 
  isc_existing_priv_mod                 = 335544529; 
  isc_primary_key_ref                   = 335544530; 
  isc_primary_key_notnull               = 335544531; 
  isc_ref_cnstrnt_notfound              = 335544532; 
  isc_foreign_key_notfound              = 335544533; 
  isc_ref_cnstrnt_update                = 335544534; 
  isc_check_cnstrnt_update              = 335544535; 
  isc_check_cnstrnt_del                 = 335544536; 
  isc_integ_index_seg_del               = 335544537; 
  isc_integ_index_seg_mod               = 335544538; 
  isc_integ_index_del                   = 335544539; 
  isc_integ_index_mod                   = 335544540; 
  isc_check_trig_del                    = 335544541; 
  isc_check_trig_update                 = 335544542; 
  isc_cnstrnt_fld_del                   = 335544543;
  isc_cnstrnt_fld_rename                = 335544544; 
  isc_rel_cnstrnt_update                = 335544545; 
  isc_constaint_on_view                 = 335544546; 
  isc_invld_cnstrnt_type                = 335544547; 
  isc_primary_key_exists                = 335544548; 
  isc_systrig_update                    = 335544549; 
  isc_not_rel_owner                     = 335544550; 
  isc_grant_obj_notfound                = 335544551; 
  isc_grant_fld_notfound                = 335544552; 
  isc_grant_nopriv                      = 335544553; 
  isc_nonsql_security_rel               = 335544554; 
  isc_nonsql_security_fld               = 335544555; 
  isc_wal_cache_err                     = 335544556; 
  isc_shutfail                          = 335544557; 
  isc_check_constraint                  = 335544558; 
  isc_bad_svc_handle                    = 335544559; 
  isc_shutwarn                          = 335544560; 
  isc_wrospbver                         = 335544561; 
  isc_bad_spb_form                      = 335544562;
  isc_svcnotdef                         = 335544563; 
  isc_no_jrn                            = 335544564; 
  isc_transliteration_failed            = 335544565; 
  isc_start_cm_for_wal                  = 335544566; 
  isc_wal_ovflow_log_required           = 335544567; 
  isc_text_subtype                      = 335544568; 
  isc_dsql_error                        = 335544569; 
  isc_dsql_command_err                  = 335544570; 
  isc_dsql_constant_err                 = 335544571; 
  isc_dsql_cursor_err                   = 335544572; 
  isc_dsql_datatype_err                 = 335544573; 
  isc_dsql_decl_err                     = 335544574; 
  isc_dsql_cursor_update_err            = 335544575; 
  isc_dsql_cursor_open_err              = 335544576; 
  isc_dsql_cursor_close_err             = 335544577; 
  isc_dsql_field_err                    = 335544578; 
  isc_dsql_internal_err                 = 335544579; 
  isc_dsql_relation_err                 = 335544580; 
  isc_dsql_procedure_err                = 335544581;
  isc_dsql_request_err                  = 335544582; 
  isc_dsql_sqlda_err                    = 335544583; 
  isc_dsql_var_count_err                = 335544584; 
  isc_dsql_stmt_handle                  = 335544585; 
  isc_dsql_function_err                 = 335544586; 
  isc_dsql_blob_err                     = 335544587; 
  isc_collation_not_found               = 335544588; 
  isc_collation_not_for_charset         = 335544589; 
  isc_dsql_dup_option                   = 335544590; 
  isc_dsql_tran_err                     = 335544591; 
  isc_dsql_invalid_array                = 335544592; 
  isc_dsql_max_arr_dim_exceeded         = 335544593; 
  isc_dsql_arr_range_error              = 335544594; 
  isc_dsql_trigger_err                  = 335544595; 
  isc_dsql_subselect_err                = 335544596; 
  isc_dsql_crdb_prepare_err             = 335544597; 
  isc_specify_field_err                 = 335544598; 
  isc_num_field_err                     = 335544599; 
  isc_col_name_err                      = 335544600;
  isc_where_err                         = 335544601; 
  isc_table_view_err                    = 335544602; 
  isc_distinct_err                      = 335544603; 
  isc_key_field_count_err               = 335544604; 
  isc_subquery_err                      = 335544605; 
  isc_expression_eval_err               = 335544606; 
  isc_node_err                          = 335544607; 
  isc_command_end_err                   = 335544608; 
  isc_index_name                        = 335544609; 
  isc_exception_name                    = 335544610; 
  isc_field_name                        = 335544611; 
  isc_token_err                         = 335544612; 
  isc_union_err                         = 335544613; 
  isc_dsql_construct_err                = 335544614; 
  isc_field_aggregate_err               = 335544615; 
  isc_field_ref_err                     = 335544616; 
  isc_order_by_err                      = 335544617; 
  isc_return_mode_err                   = 335544618; 
  isc_extern_func_err                   = 335544619;
  isc_alias_conflict_err                = 335544620; 
  isc_procedure_conflict_error          = 335544621; 
  isc_relation_conflict_err             = 335544622; 
  isc_dsql_domain_err                   = 335544623; 
  isc_idx_seg_err                       = 335544624; 
  isc_node_name_err                     = 335544625; 
  isc_table_name                        = 335544626; 
  isc_proc_name                         = 335544627; 
  isc_idx_create_err                    = 335544628; 
  isc_wal_shadow_err                    = 335544629; 
  isc_dependency                        = 335544630; 
  isc_idx_key_err                       = 335544631; 
  isc_dsql_file_length_err              = 335544632; 
  isc_dsql_shadow_number_err            = 335544633; 
  isc_dsql_token_unk_err                = 335544634; 
  isc_dsql_no_relation_alias            = 335544635; 
  isc_indexname                         = 335544636; 
  isc_no_stream_plan                    = 335544637; 
  isc_stream_twice                      = 335544638;
  isc_stream_not_found                  = 335544639; 
  isc_collation_requires_text           = 335544640; 
  isc_dsql_domain_not_found             = 335544641; 
  isc_index_unused                      = 335544642; 
  isc_dsql_self_join                    = 335544643; 
  isc_stream_bof                        = 335544644; 
  isc_stream_crack                      = 335544645; 
  isc_db_or_file_exists                 = 335544646; 
  isc_invalid_operator                  = 335544647; 
  isc_conn_lost                         = 335544648; 
  isc_bad_checksum                      = 335544649; 
  isc_page_type_err                     = 335544650; 
  isc_ext_readonly_err                  = 335544651; 
  isc_sing_select_err                   = 335544652; 
  isc_psw_attach                        = 335544653; 
  isc_psw_start_trans                   = 335544654; 
  isc_invalid_direction                 = 335544655; 
  isc_dsql_var_conflict                 = 335544656; 
  isc_dsql_no_blob_array                = 335544657;
  isc_dsql_base_table                   = 335544658; 
  isc_duplicate_base_table              = 335544659; 
  isc_view_alias                        = 335544660; 
  isc_index_root_page_full              = 335544661; 
  isc_dsql_blob_type_unknown            = 335544662; 
  isc_req_max_clones_exceeded           = 335544663; 
  isc_dsql_duplicate_spec               = 335544664; 
  isc_unique_key_violation              = 335544665; 
  isc_srvr_version_too_old              = 335544666; 
  isc_drdb_completed_with_errs          = 335544667; 
  isc_dsql_procedure_use_err            = 335544668; 
  isc_dsql_count_mismatch               = 335544669; 
  isc_blob_idx_err                      = 335544670; 
  isc_array_idx_err                     = 335544671; 
  isc_key_field_err                     = 335544672; 
  isc_no_delete                         = 335544673; 
  isc_del_last_field                    = 335544674; 
  isc_sort_err                          = 335544675; 
  isc_sort_mem_err                      = 335544676;
  isc_version_err                       = 335544677; 
  isc_inval_key_posn                    = 335544678; 
  isc_no_segments_err                   = 335544679; 
  isc_crrp_data_err                     = 335544680; 
  isc_rec_size_err                      = 335544681; 
  isc_dsql_field_ref                    = 335544682; 
  isc_req_depth_exceeded                = 335544683; 
  isc_no_field_access                   = 335544684; 
  isc_no_dbkey                          = 335544685; 
  isc_jrn_format_err                    = 335544686; 
  isc_jrn_file_full                     = 335544687; 
  isc_dsql_open_cursor_request          = 335544688; 
  isc_ib_error                          = 335544689; 
  isc_cache_redef                       = 335544690; 
  isc_cache_too_small                   = 335544691; 
  isc_log_redef                         = 335544692; 
  isc_log_too_small                     = 335544693; 
  isc_partition_too_small               = 335544694; 
  isc_partition_not_supp                = 335544695;
  isc_log_length_spec                   = 335544696; 
  isc_precision_err                     = 335544697; 
  isc_scale_nogt                        = 335544698; 
  isc_expec_short                       = 335544699; 
  isc_expec_long                        = 335544700; 
  isc_expec_ushort                      = 335544701; 
  isc_like_escape_invalid               = 335544702; 
  isc_svcnoexe                          = 335544703; 
  isc_net_lookup_err                    = 335544704; 
  isc_service_unknown                   = 335544705; 
  isc_host_unknown                      = 335544706; 
  isc_grant_nopriv_on_base              = 335544707; 
  isc_dyn_fld_ambiguous                 = 335544708; 
  isc_dsql_agg_ref_err                  = 335544709; 
  isc_complex_view                      = 335544710; 
  isc_unprepared_stmt                   = 335544711; 
  isc_expec_positive                    = 335544712; 
  isc_dsql_sqlda_value_err              = 335544713; 
  isc_invalid_array_id                  = 335544714;
  isc_extfile_uns_op                    = 335544715; 
  isc_svc_in_use                        = 335544716; 
  isc_err_stack_limit                   = 335544717; 
  isc_invalid_key                       = 335544718; 
  isc_net_init_error                    = 335544719; 
  isc_loadlib_failure                   = 335544720; 
  isc_network_error                     = 335544721; 
  isc_net_connect_err                   = 335544722; 
  isc_net_connect_listen_err            = 335544723; 
  isc_net_event_connect_err             = 335544724; 
  isc_net_event_listen_err              = 335544725; 
  isc_net_read_err                      = 335544726; 
  isc_net_write_err                     = 335544727; 
  isc_integ_index_deactivate            = 335544728; 
  isc_integ_deactivate_primary          = 335544729; 
  isc_cse_not_supported                 = 335544730; 
  isc_tra_must_sweep                    = 335544731; 
  isc_unsupported_network_drive         = 335544732; 
  isc_io_create_err                     = 335544733;
  isc_io_open_err                       = 335544734; 
  isc_io_close_err                      = 335544735; 
  isc_io_read_err                       = 335544736; 
  isc_io_write_err                      = 335544737; 
  isc_io_delete_err                     = 335544738; 
  isc_io_access_err                     = 335544739; 
  isc_udf_exception                     = 335544740; 
  isc_lost_db_connection                = 335544741; 
  isc_no_write_user_priv                = 335544742; 
  isc_token_too_long                    = 335544743; 
  isc_max_att_exceeded                  = 335544744; 
  isc_err_max                           = 424; 

{ ******************************************** }
{ Dynamic Data Definition Language operators }
{ ******************************************** }

{ **************** }
{ Version number }
{ **************** }

  isc_dyn_version_1                     = 1; 
  isc_dyn_eoc                           = -1; 

{ **************************** }
{ Operations (may be nested) }
{ **************************** }

  isc_dyn_begin                         = 2; 
  isc_dyn_end                           = 3; 
  isc_dyn_if                            = 4; 
  isc_dyn_def_database                  = 5; 
  isc_dyn_def_global_fld                = 6; 
  isc_dyn_def_local_fld                 = 7; 
  isc_dyn_def_idx                       = 8; 
  isc_dyn_def_rel                       = 9; 
  isc_dyn_def_sql_fld                   = 10; 
  isc_dyn_def_view                      = 12;
  isc_dyn_def_trigger                   = 15; 
  isc_dyn_def_security_class            = 120; 
  isc_dyn_def_dimension                 = 140; 
  isc_dyn_def_generator                 = 24; 
  isc_dyn_def_function                  = 25; 
  isc_dyn_def_filter                    = 26; 
  isc_dyn_def_function_arg              = 27; 
  isc_dyn_def_shadow                    = 34; 
  isc_dyn_def_trigger_msg               = 17; 
  isc_dyn_def_file                      = 36; 
  isc_dyn_mod_database                  = 39; 
  isc_dyn_mod_rel                       = 11; 
  isc_dyn_mod_global_fld                = 13; 
  isc_dyn_mod_idx                       = 102; 
  isc_dyn_mod_local_fld                 = 14; 
  isc_dyn_mod_view                      = 16; 
  isc_dyn_mod_security_class            = 122; 
  isc_dyn_mod_trigger                   = 113; 
  isc_dyn_mod_trigger_msg               = 28;
  isc_dyn_delete_database               = 18; 
  isc_dyn_delete_rel                    = 19; 
  isc_dyn_delete_global_fld             = 20; 
  isc_dyn_delete_local_fld              = 21; 
  isc_dyn_delete_idx                    = 22; 
  isc_dyn_delete_security_class         = 123; 
  isc_dyn_delete_dimensions             = 143; 
  isc_dyn_delete_trigger                = 23; 
  isc_dyn_delete_trigger_msg            = 29; 
  isc_dyn_delete_filter                 = 32; 
  isc_dyn_delete_function               = 33; 
  isc_dyn_delete_shadow                 = 35; 
  isc_dyn_grant                         = 30; 
  isc_dyn_revoke                        = 31; 
  isc_dyn_def_primary_key               = 37; 
  isc_dyn_def_foreign_key               = 38; 
  isc_dyn_def_unique                    = 40; 
  isc_dyn_def_procedure                 = 164; 
  isc_dyn_delete_procedure              = 165;
  isc_dyn_def_parameter                 = 135; 
  isc_dyn_delete_parameter              = 136; 
  isc_dyn_mod_procedure                 = 175; 
  isc_dyn_def_log_file                  = 176; 
  isc_dyn_def_cache_file                = 180; 
  isc_dyn_def_exception                 = 181; 
  isc_dyn_mod_exception                 = 182; 
  isc_dyn_del_exception                 = 183; 
  isc_dyn_drop_log                      = 194; 
  isc_dyn_drop_cache                    = 195; 
  isc_dyn_def_default_log               = 202; 

{ ********************* }
{ View specific stuff }
{ ********************* }

  isc_dyn_view_blr                      = 43; 
  isc_dyn_view_source                   = 44; 
  isc_dyn_view_relation                 = 45;
  isc_dyn_view_context                  = 46; 
  isc_dyn_view_context_name             = 47; 

{ ******************** }
{ Generic attributes }
{ ******************** }

  isc_dyn_rel_name                      = 50; 
  isc_dyn_fld_name                      = 51; 
  isc_dyn_idx_name                      = 52; 
  isc_dyn_description                   = 53; 
  isc_dyn_security_class                = 54; 
  isc_dyn_system_flag                   = 55; 
  isc_dyn_update_flag                   = 56; 
  isc_dyn_prc_name                      = 166; 
  isc_dyn_prm_name                      = 137; 
  isc_dyn_sql_object                    = 196; 
  isc_dyn_fld_character_set_name        = 174; 

{ ****************************** }
{ Relation specific attributes }
{ ****************************** }

  isc_dyn_rel_dbkey_length              = 61; 
  isc_dyn_rel_store_trig                = 62; 
  isc_dyn_rel_modify_trig               = 63; 
  isc_dyn_rel_erase_trig                = 64; 
  isc_dyn_rel_store_trig_source         = 65; 
  isc_dyn_rel_modify_trig_source        = 66; 
  isc_dyn_rel_erase_trig_source         = 67; 
  isc_dyn_rel_ext_file                  = 68; 
  isc_dyn_rel_sql_protection            = 69; 
  isc_dyn_rel_constraint                = 162; 
  isc_dyn_delete_rel_constraint         = 163; 

{ ********************************** }
{ Global field specific attributes }
{ ********************************** }

  isc_dyn_fld_type                      = 70; 
  isc_dyn_fld_length                    = 71; 
  isc_dyn_fld_scale                     = 72; 
  isc_dyn_fld_sub_type                  = 73; 
  isc_dyn_fld_segment_length            = 74; 
  isc_dyn_fld_query_header              = 75; 
  isc_dyn_fld_edit_string               = 76; 
  isc_dyn_fld_validation_blr            = 77; 
  isc_dyn_fld_validation_source         = 78; 
  isc_dyn_fld_computed_blr              = 79; 
  isc_dyn_fld_computed_source           = 80; 
  isc_dyn_fld_missing_value             = 81; 
  isc_dyn_fld_default_value             = 82; 
  isc_dyn_fld_query_name                = 83; 
  isc_dyn_fld_dimensions                = 84; 
  isc_dyn_fld_not_null                  = 85; 
  isc_dyn_fld_char_length               = 172; 
  isc_dyn_fld_collation                 = 173;
  isc_dyn_fld_default_source            = 193; 
  isc_dyn_del_default                   = 197; 
  isc_dyn_del_validation                = 198; 
  isc_dyn_single_validation             = 199; 
  isc_dyn_fld_character_set             = 203; 

{ ********************************* }
{ Local field specific attributes }
{ ********************************* }

  isc_dyn_fld_source                    = 90; 
  isc_dyn_fld_base_fld                  = 91; 
  isc_dyn_fld_position                  = 92; 
  isc_dyn_fld_update_flag               = 93; 

{ *************************** }
{ Index specific attributes }
{ *************************** }

  isc_dyn_idx_unique                    = 100; 
  isc_dyn_idx_inactive                  = 101; 
  isc_dyn_idx_type                      = 103; 
  isc_dyn_idx_foreign_key               = 104; 
  isc_dyn_idx_ref_column                = 105; 
  isc_dyn_idx_statistic                  = 204; 

{ ***************************** }
{ Trigger specific attributes }
{ ***************************** }

  isc_dyn_trg_type                      = 110; 
  isc_dyn_trg_blr                       = 111; 
  isc_dyn_trg_source                    = 112; 
  isc_dyn_trg_name                      = 114; 
  isc_dyn_trg_sequence                  = 115; 
  isc_dyn_trg_inactive                  = 116; 
  isc_dyn_trg_msg_number                = 117; 
  isc_dyn_trg_msg                       = 118;

{ ************************************ }
{ Security Class specific attributes }
{ ************************************ }

  isc_dyn_scl_acl                       = 121; 
  isc_dyn_grant_user                    = 130; 
  isc_dyn_grant_proc                    = 186; 
  isc_dyn_grant_trig                    = 187; 
  isc_dyn_grant_view                    = 188; 
  isc_dyn_grant_options                 = 132; 


{ ******************************** }
{ Dimension specific information }
{ ******************************** }

  isc_dyn_dim_lower                     = 141; 
  isc_dyn_dim_upper                     = 142;

{ ************************** }
{ File specific attributes }
{ ************************** }

  isc_dyn_file_name                     = 125; 
  isc_dyn_file_start                    = 126; 
  isc_dyn_file_length                   = 127; 
  isc_dyn_shadow_number                 = 128; 
  isc_dyn_shadow_man_auto               = 129; 
  isc_dyn_shadow_conditional            = 130; 

{ ****************************** }
{ Log file specific attributes }
{ ****************************** }

  isc_dyn_log_file_sequence             = 177; 
  isc_dyn_log_file_partitions           = 178; 
  isc_dyn_log_file_serial               = 179;
  isc_dyn_log_file_overflow             = 200; 
  isc_dyn_log_file_raw                  = 201; 

{ ************************* }
{ Log specific attributes }
{ ************************* }

  isc_dyn_log_group_commit_wait         = 189; 
  isc_dyn_log_buffer_size               = 190; 
  isc_dyn_log_check_point_length        = 191; 
  isc_dyn_log_num_of_buffers            = 192; 

{ ****************************** }
{ Function specific attributes }
{ ****************************** }

  isc_dyn_function_name                 = 145; 
  isc_dyn_function_type                 = 146; 
  isc_dyn_func_module_name              = 147;
  isc_dyn_func_entry_point              = 148; 
  isc_dyn_func_return_argument          = 149; 
  isc_dyn_func_arg_position             = 150; 
  isc_dyn_func_mechanism                = 151; 
  isc_dyn_filter_in_subtype             = 152; 
  isc_dyn_filter_out_subtype            = 153; 


  isc_dyn_description2                  = 154; 
  isc_dyn_fld_computed_source2          = 155; 
  isc_dyn_fld_edit_string2              = 156; 
  isc_dyn_fld_query_header2             = 157; 
  isc_dyn_fld_validation_source2        = 158; 
  isc_dyn_trg_msg2                      = 159; 
  isc_dyn_trg_source2                   = 160; 
  isc_dyn_view_source2                  = 161; 
  isc_dyn_xcp_msg2                      = 184; 

{ ******************************* }
{ Generator specific attributes }
{ ******************************* }

  isc_dyn_generator_name                = 95; 
  isc_dyn_generator_id                  = 96; 

{ ******************************* }
{ Procedure specific attributes }
{ ******************************* }

  isc_dyn_prc_inputs                    = 167; 
  isc_dyn_prc_outputs                   = 168; 
  isc_dyn_prc_source                    = 169; 
  isc_dyn_prc_blr                       = 170; 
  isc_dyn_prc_source2                   = 171; 

{ ******************************* }
{ Parameter specific attributes }
{ ******************************* }

  isc_dyn_prm_number                    = 138; 
  isc_dyn_prm_type                      = 139; 

{ ****************************** }
{ Relation specific attributes }
{ ****************************** }

  isc_dyn_xcp_msg                       = 185; 

{ ******************************************** }
{ Cascading referential integrity values     }
{ ******************************************** }
  isc_dyn_foreign_key_update            = 205; 
  isc_dyn_foreign_key_delete            = 206; 
  isc_dyn_foreign_key_cascade           = 207; 
  isc_dyn_foreign_key_default           = 208; 
  isc_dyn_foreign_key_null              = 209; 
  isc_dyn_foreign_key_none              = 210;

{ ************************** }
{ Last $dyn value assigned }
{ ************************** }

  isc_dyn_last_dyn_value                = 210; 

{ **************************************** }
{ Array slice description language (SDL) }
{ **************************************** }

  isc_sdl_version1                      = 1; 
  isc_sdl_eoc                           = -1; 
  isc_sdl_relation                      = 2; 
  isc_sdl_rid                           = 3; 
  isc_sdl_field                         = 4; 
  isc_sdl_fid                           = 5; 
  isc_sdl_struct                        = 6; 
  isc_sdl_variable                      = 7;
  isc_sdl_scalar                        = 8; 
  isc_sdl_tiny_integer                  = 9; 
  isc_sdl_short_integer                 = 10; 
  isc_sdl_long_integer                  = 11; 
  isc_sdl_literal                       = 12; 
  isc_sdl_add                           = 13; 
  isc_sdl_subtract                      = 14; 
  isc_sdl_multiply                      = 15; 
  isc_sdl_divide                        = 16; 
  isc_sdl_negate                        = 17; 
  isc_sdl_eql                           = 18; 
  isc_sdl_neq                           = 19; 
  isc_sdl_gtr                           = 20; 
  isc_sdl_geq                           = 21; 
  isc_sdl_lss                           = 22; 
  isc_sdl_leq                           = 23; 
  isc_sdl_and                           = 24; 
  isc_sdl_or                            = 25; 
  isc_sdl_not                           = 26;
  isc_sdl_while                         = 27; 
  isc_sdl_assignment                    = 28; 
  isc_sdl_label                         = 29; 
  isc_sdl_leave                         = 30; 
  isc_sdl_begin                         = 31; 
  isc_sdl_end                           = 32; 
  isc_sdl_do3                           = 33; 
  isc_sdl_do2                           = 34; 
  isc_sdl_do1                           = 35; 
  isc_sdl_element                       = 36; 

{ ****************************************** }
{ International text interpretation values }
{ ****************************************** }

  isc_interp_eng_ascii                  = 0; 
  isc_interp_jpn_sjis                   = 5; 
  isc_interp_jpn_euc                    = 6; 

{ ************************************** }
{ Scroll direction for isc_dsql_fetch2 }
{ ************************************** }

  isc_fetch_next                         = 0; 
  isc_fetch_prior                        = 1; 
  isc_fetch_first                        = 2; 
  isc_fetch_last                         = 3; 
  isc_fetch_absolute                     = 4; 
  isc_fetch_relative                     = 5; 

{ ***************** }
{ SQL definitions }
{ ***************** }

  SQLDA_VERSION1              = 1; 

  SQL_TEXT                               = 452; 
  SQL_VARYING                            = 448;
  SQL_SHORT                              = 500; 
  SQL_LONG                               = 496; 
  SQL_FLOAT                              = 482; 
  SQL_DOUBLE                             = 480; 
  SQL_D_FLOAT                            = 530; 
  SQL_DATE                               = 510; 
  SQL_BLOB                               = 520; 
  SQL_ARRAY                              = 540; 
  SQL_QUAD                               = 550; 

{ *************** }
{ Blob Subtypes }
{ *************** }

{ types less than zero are reserved for customer use }

  isc_blob_untyped                       = 0; 

{ internal subtypes }

  isc_blob_text                          = 1;
  isc_blob_blr                           = 2; 
  isc_blob_acl                           = 3; 
  isc_blob_ranges                        = 4; 
  isc_blob_summary                       = 5; 
  isc_blob_format                        = 6; 
  isc_blob_tra                           = 7; 
  isc_blob_extfile                       = 8; 

{ the range 20-30 is reserved for dBASE and Paradox types }

  isc_blob_formatted_memo                = 20; 
  isc_blob_paradox_ole                   = 21; 
  isc_blob_graphic                       = 22; 
  isc_blob_dbase_ole                     = 23; 
  isc_blob_typed_binary                  = 24; 

implementation

function GetB(var S: TBStream): Integer;
begin
  Dec(S.bstr_cnt);
  if S.bstr_cnt  >= 0 then
  begin
    Inc(S.bstr_ptr);
    Result := Integer(S.bstr_ptr) and 377;
  end
  else
    Result := Blob_get(@S);
end;

function PutB(C: Char; var S: TBStream): Integer;
begin
  Dec(S.bstr_cnt);
  if (C = #13) or (S.bstr_cnt = 0) then
    Result := Blob_put(C, @S)
  else begin
    Inc(S.bstr_ptr);
    Result := Ord(S.bstr_ptr^ = C);
  end;
end;

function PutBX(C: Char; var S: TBStream): Integer;
begin
  Dec(S.bstr_cnt);
  if S.bstr_cnt = 0 then
    Result := Blob_put(C, @S)
  else begin
    Inc(S.bstr_ptr);
    Result := Ord(S.bstr_ptr^ = C);
  end;
end;

function SQLDA_Length(n: Integer): Integer;
begin
  Result := SizeOf(TSQLDA) + (n - 1) * SizeOf(TSQLVar);
end;

function XSQLDA_LENGTH(n: Integer): Integer;
begin
  Result := SizeOf(TXSQLDA) + (n - 1) * SizeOf(TXSQLVar)
end;

const
  ModName = 'GDS32.DLL';

function BLOB_close;                    external ModName name 'BLOB_close';
function BLOB_display;                  external ModName name 'BLOB_display';
function BLOB_dump;                     external ModName name 'BLOB_dump';
function BLOB_edit;                     external ModName name 'BLOB_edit';
function BLOB_get;                      external ModName name 'BLOB_get';
function BLOB_load;                     external ModName name 'BLOB_load';
function BLOB_open;                     external ModName name 'BLOB_open';
function BLOB_put;                      external ModName name 'BLOB_put';
function BLOB_text_dump;                external ModName name 'BLOB_text_dump';
function BLOB_text_load;                external ModName name 'BLOB_text_load';
function Bopen;                         external ModName name 'Bopen';
function Bopen2;                        external ModName name 'Bopen2';
function isc_array_gen_sdl;             external ModName name 'isc_array_gen_sdl';
function isc_array_get_slice;           external ModName name 'isc_array_get_slice';
function isc_array_lookup_bounds;       external ModName name 'isc_array_lookup_bounds';
function isc_array_lookup_desc;         external ModName name 'isc_array_lookup_desc';
function isc_array_put_slice;           external ModName name 'isc_array_put_slice';
function isc_array_set_desc;            external ModName name 'isc_array_set_desc';
function isc_attach_database;           external ModName name 'isc_attach_database';
function isc_attach_service;            external ModName name 'isc_attach_service';
procedure isc_blob_default_desc;         external ModName name 'isc_blob_default_desc';
function isc_blob_gen_bpb;              external ModName name 'isc_blob_gen_bpb';
function isc_blob_info;                 external ModName name 'isc_blob_info';
function isc_blob_lookup_desc;          external ModName name 'isc_blob_lookup_desc';
function isc_blob_set_desc;             external ModName name 'isc_blob_set_desc';
function isc_cancel_blob;               external ModName name 'isc_cancel_blob';
function isc_cancel_events;             external ModName name 'isc_cancel_events';
function isc_close;                     external ModName name 'isc_close';
function isc_close_blob;                external ModName name 'isc_close_blob';
function isc_commit_retaining;          external ModName name 'isc_commit_retaining';
function isc_commit_transaction;        external ModName name 'isc_commit_transaction';
function isc_compile_map;               external ModName name 'isc_compile_map';
function isc_compile_menu;              external ModName name 'isc_compile_menu';
function isc_compile_request;           external ModName name 'isc_compile_request';
function isc_compile_request2;          external ModName name 'isc_compile_request2';
function isc_compile_sub_map;           external ModName name 'isc_compile_sub_map';
function isc_create_blob;               external ModName name 'isc_create_blob';
function isc_create_blob2;              external ModName name 'isc_create_blob2';
function isc_create_database;           external ModName name 'isc_create_database';
function isc_create_window;             external ModName name 'isc_create_window';
function isc_database_info;             external ModName name 'isc_database_info';
function isc_ddl;                       external ModName name 'isc_ddl';
function isc_declare;                   external ModName name 'isc_declare';
procedure isc_decode_date;               external ModName name 'isc_decode_date';
function isc_delete_window;             external ModName name 'isc_delete_window';
function isc_describe;                  external ModName name 'isc_describe';
function isc_describe_bind;             external ModName name 'isc_describe_bind';
function isc_detach_database;           external ModName name 'isc_detach_database';
function isc_detach_service;            external ModName name 'isc_detach_service';
function isc_drive_form;                external ModName name 'isc_drive_form';
function isc_drive_menu;                external ModName name 'isc_drive_menu';
function isc_drop_database;             external ModName name 'isc_drop_database';
function isc_dsql_alloc_statement2;     external ModName name 'isc_dsql_alloc_statement2';
function isc_dsql_allocate_statement;   external ModName name 'isc_dsql_allocate_statement';
function isc_dsql_describe;             external ModName name 'isc_dsql_describe';
function isc_dsql_describe_bind;        external ModName name 'isc_dsql_describe_bind';
function isc_dsql_exec_immed2;          external ModName name 'isc_dsql_exec_immed2';
function isc_dsql_exec_immed3_m;        external ModName name 'isc_dsql_exec_immed3_m';
function isc_dsql_execute;              external ModName name 'isc_dsql_execute';
function isc_dsql_execute_immediate;    external ModName name 'isc_dsql_execute_immediate';
function isc_dsql_execute_immediate_m;  external ModName name 'isc_dsql_execute_immediate_m';
function isc_dsql_execute_m;            external ModName name 'isc_dsql_execute_m';
function isc_dsql_execute2;             external ModName name 'isc_dsql_execute2';
function isc_dsql_execute2_m;           external ModName name 'isc_dsql_execute2_m';
function isc_dsql_fetch;                external ModName name 'isc_dsql_fetch';
function isc_dsql_fetch_m;              external ModName name 'isc_dsql_fetch_m';
function isc_dsql_fetch2;               external ModName name 'isc_dsql_fetch2';
function isc_dsql_fetch2_m;             external ModName name 'isc_dsql_fetch2_m';
function isc_dsql_finish;               external ModName name 'isc_dsql_finish';
function isc_dsql_free_statement;       external ModName name 'isc_dsql_free_statement';
function isc_dsql_insert;               external ModName name 'isc_dsql_insert';
function isc_dsql_insert_m;             external ModName name 'isc_dsql_insert_m';
function isc_dsql_prepare;              external ModName name 'isc_dsql_prepare';
function isc_dsql_prepare_m;            external ModName name 'isc_dsql_prepare_m';
function isc_dsql_release;              external ModName name 'isc_dsql_release';
function isc_dsql_set_cursor_name;      external ModName name 'isc_dsql_set_cursor_name';
function isc_dsql_sql_info;             external ModName name 'isc_dsql_sql_info';
function isc_embed_dsql_close;          external ModName name 'isc_embed_dsql_close';
function isc_embed_dsql_declare;        external ModName name 'isc_embed_dsql_declare';
function isc_embed_dsql_describe;       external ModName name 'isc_embed_dsql_describe';
function isc_embed_dsql_describe_bind;  external ModName name 'isc_embed_dsql_describe_bind';
function isc_embed_dsql_execute;        external ModName name 'isc_embed_dsql_execute';
function isc_embed_dsql_execute_immed;  external ModName name 'isc_embed_dsql_execute_immed';
function isc_embed_dsql_execute2;       external ModName name 'isc_embed_dsql_execute2';
function isc_embed_dsql_fetch;          external ModName name 'isc_embed_dsql_fetch';
function isc_embed_dsql_fetch2;         external ModName name 'isc_embed_dsql_fetch2';
function isc_embed_dsql_insert;         external ModName name 'isc_embed_dsql_insert';
function isc_embed_dsql_open;           external ModName name 'isc_embed_dsql_open';
function isc_embed_dsql_open2;          external ModName name 'isc_embed_dsql_open2';
function isc_embed_dsql_prepare;        external ModName name 'isc_embed_dsql_prepare';
function isc_embed_dsql_release;        external ModName name 'isc_embed_dsql_release';
procedure isc_encode_date;               external ModName name 'isc_encode_date';
procedure isc_event_counts;              external ModName name 'isc_event_counts';
function isc_execute;                   external ModName name 'isc_execute';
function isc_execute_immediate;         external ModName name 'isc_execute_immediate';
function isc_fetch;                     external ModName name 'isc_fetch';
function isc_form_delete;               external ModName name 'isc_form_delete';
function isc_form_fetch;                external ModName name 'isc_form_fetch';
function isc_form_insert;               external ModName name 'isc_form_insert';
function isc_free;                      external ModName name 'isc_free';
function isc_ftof;                      external ModName name 'isc_ftof';
function isc_get_entree;                external ModName name 'isc_get_entree';
function isc_get_segment;               external ModName name 'isc_get_segment';
function isc_get_slice;                 external ModName name 'isc_get_slice';
function isc_initialize_menu;           external ModName name 'isc_initialize_menu';
function isc_interprete;                external ModName name 'isc_interprete';
function isc_load_form;                 external ModName name 'isc_load_form';
function isc_menu;                      external ModName name 'isc_menu';
function isc_modify_dpb;                external ModName name 'isc_modify_dpb';
function isc_open;                      external ModName name 'isc_open';
function isc_open_blob;                 external ModName name 'isc_open_blob';
function isc_open_blob2;                external ModName name 'isc_open_blob2';
function isc_pop_window;                external ModName name 'isc_pop_window';
function isc_prepare;                   external ModName name 'isc_prepare';
function isc_prepare_transaction;       external ModName name 'isc_prepare_transaction';
function isc_prepare_transaction2;      external ModName name 'isc_prepare_transaction2';
function isc_print_blr;                 external ModName name 'isc_print_blr';
procedure isc_print_sqlerror;            external ModName name 'isc_print_sqlerror';
function isc_print_status;              external ModName name 'isc_print_status';
function isc_put_entree;                external ModName name 'isc_put_entree';
function isc_put_segment;               external ModName name 'isc_put_segment';
function isc_put_slice;                 external ModName name 'isc_put_slice';
procedure isc_qtoq;                      external ModName name 'isc_qtoq';
function isc_que_events;                external ModName name 'isc_que_events';
function isc_query_service;             external ModName name 'isc_query_service';
function isc_receive;                   external ModName name 'isc_receive';
function isc_reconnect_transaction;     external ModName name 'isc_reconnect_transaction';
function isc_release_request;           external ModName name 'isc_release_request';
function isc_request_info;              external ModName name 'isc_request_info';
function isc_reset_form;                external ModName name 'isc_reset_form';
function isc_rollback_transaction;      external ModName name 'isc_rollback_transaction';
function isc_seek_blob;                 external ModName name 'isc_seek_blob';
function isc_send;                      external ModName name 'isc_send';
procedure isc_set_debug;                 external ModName name 'isc_set_debug';
procedure isc_sql_interprete;            external ModName name 'isc_sql_interprete';
function isc_sqlcode;                   external ModName name 'isc_sqlcode';
function isc_start_and_send;            external ModName name 'isc_start_and_send';
function isc_start_multiple;            external ModName name 'isc_start_multiple';
function isc_start_request;             external ModName name 'isc_start_request';
function isc_suspend_window;            external ModName name 'isc_suspend_window';
function isc_transact_request;          external ModName name 'isc_transact_request';
function isc_transaction_info;          external ModName name 'isc_transaction_info';
function isc_unwind_request;            external ModName name 'isc_unwind_request';
function isc_vax_integer;               external ModName name 'isc_vax_integer';
function isc_version;                   external ModName name 'isc_version';
procedure isc_vtof;                      external ModName name 'isc_vtof';
procedure isc_vtov;                      external ModName name 'isc_vtov';
function isc_wait_for_event;            external ModName name 'isc_wait_for_event';

end.
