unit Main;
// Main unit for IB Create Database example. This program will create an
// InterBase database and optionally add sample data to a table within
// the Database. An InterBase database (.gdb) file cannot be created
// through BDE calls; it must be created through the InterBase API. This
// program creates a simple wrapper function to create a new IB database.

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, DBGrids, DB, DBTables, StdCtrls, ExtCtrls, IBUtils, BDE;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    CreateBtn: TButton;
    AddDataChk: TCheckBox;
    IBDB: TDatabase;
    DBGrid1: TDBGrid;
    Label1: TLabel;
    UserName: TEdit;
    Label2: TLabel;
    Password: TEdit;
    Label3: TLabel;
    PathDB: TEdit;
    IBTable: TTable;
    DataSource1: TDataSource;
    procedure CreateBtnClick(Sender: TObject);
    procedure PathDBChange(Sender: TObject);
  private
    procedure AddSampleData;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

const
  IBTableName = 'TEST_TABLE';
  MaxRecords = 4;
  SampleData: array[1..4, 1..4] of string = (
    ('1', 'This is a sample', 'of Creating a InterBase Database File', '3284'),
    ('2', 'Using an array', 'To add records', '483'),
    ('3', 'This is a CHAR Field', 'This is a VARCHAR Field', '272'),
    ('4', 'Using Direct BDE Calls', 'to Add this Data to the Table', '9993'));

procedure TMainForm.CreateBtnClick(Sender: TObject);
// Create and optionally add data to a new Database
begin
  Screen.Cursor := crHourGlass;
  try
    IBTable.Close;
    IBDB.Close;
    CreateDatabase(PathDB.Text, UserName.Text, Password.Text);
    if AddDataChk.Checked then AddSampleData;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TMainForm.AddSampleData;
// Add sample data to a Database
const
  SCreateSQL = 'CREATE TABLE %s (Field1 INTEGER NOT NULL UNIQUE,  ' +
    'Field2 CHAR(30), Field3 VARCHAR(45), Field4 SMALLINT NOT NULL);';
  SInsertSQL = 'INSERT INTO %s VALUES (%s, "%s", "%s", %s);';
var
  Buf: string;
  Rows, i: Word;
  x_handle: hDBIXact;
begin
  Buf := Format(SCreateSQL, [IBTableName]);
  { Setup and connect the TDatabase to the new Database }
  with IBDB.Params do
  begin
    Values['SERVER NAME'] := PathDB.Text;
    Values['USER NAME'] := UserName.Text;
    Values['PASSWORD'] := Password.Text;
  end;
  IBDB.Open;
  Check(DbiQExecDirect(IBDB.Handle, qrylangSQL, PChar(Buf), nil));
  Rows := MaxRecords;
  // Start a transaction...
  Check(DbiBeginTran(IBDB.Handle, xilREADCOMMITTED, x_handle));
  try
    for i := 1 to Rows do
    begin
      Buf := Format(SInsertSQL, [IBTableName, SampleData[i, 1],
        SampleData[i, 2], SampleData[i, 3], SampleData[i, 4]]);
      // Insert the data
      Check(DbiQExecDirect(IBDB.Handle, qrylangSQL, PChar(Buf), nil));
    end;
    // If all goes well, commit the inserted records
    Check(DbiEndTran(IBDB.Handle, x_handle, xendCOMMIT));
  except
    on E:EDatabaseError do
    begin
      // Rollback any inserted records if an error occurred
      // and re-raise the exception
      Check(DbiEndTran(IBDB.Handle, x_handle, xendABORT));
      raise;
    end;
  end;
  // Open and display the new table information
  IBTable.TableName := IBTableName;
  IBTable.Open;
end;

procedure TMainForm.PathDBChange(Sender: TObject);
// OnChange handler for edit controls
begin
  // only enable the database create button if all information is entered
  CreateBtn.Enabled := (PathDB.Text <> '') and (UserName.Text <> '') and
    (Password.Text <> '');
end;

end.
