unit CustomerSrchFrm;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, SysUtils;

type
  TCustomerSearchForm = class(TForm)
    lblIDNumber: TLabel;
    edtIDNumber: TEdit;
    lblFirstName: TLabel;
    lblLastName: TLabel;
    lblAltPhone: TLabel;
    lblWorkPhone: TLabel;
    lblWorkAddress: TLabel;
    lblAltAddress: TLabel;
    lblCompany: TLabel;
    edtFirstName: TEdit;
    edtLastName: TEdit;
    edtWorkPhone: TEdit;
    edtAltPhone: TEdit;
    edtWorkAddress: TEdit;
    edtAltAddress: TEdit;
    edtCompany: TEdit;
    btnCancel: TButton;
    btnFind: TButton;
    lblInstruction: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FindCustBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure lblIDNumberClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FindPressed: Boolean;
    procedure ClearEditFields;
    function BuildSQLStatement: string;
  public
    QueryString: String;
  end;

function SearchCustomer: String;

implementation

{$R *.DFM}

uses Dialogs;

function SearchCustomer: String;
var
  CustomerSearchForm: TCustomerSearchForm;
begin
  Result := EmptyStr;
  CustomerSearchForm := TCustomerSearchForm.Create(Application);
  try
    if CustomerSearchForm.ShowModal = mrOk then
      Result := CustomerSearchForm.QueryString;
  finally
    CustomerSearchForm.Free;
  end;

end;


function TCustomerSearchForm.BuildSQLStatement: string;
{ This function builds an SQL query statement based on the search
  fields of a customer record as specified by the user. The search
  fields are indicated by the labels whose color is clRed. The user
  can select these labels by clicking on them. The user must enter a
  value into the edit field to which the labels refer. }
var
  Sep: String[3]; // Used as a seperator.
begin
  Sep := '';
  Result := '';

  if lblIDNumber.Font.Color = clRed then
  begin
    Result := Format('(CUSTOMER_ID = %s)', [edtIDNumber.Text]);
    Sep := 'AND';
  end;

  if lblLastName.Font.Color = clRed then
  begin
    Result := Format('%s %s (UPPER(LNAME) = "%s")',
                    [Result, Sep, UpperCase(edtLastName.Text)]);
    Sep := 'AND';
  end;

  if lblFirstName.Font.Color = clRed then
  begin
    Result := Format('%s %s (UPPER(FNAME) = "%s")',
                    [Result, Sep, UpperCase(edtFirstName.Text)]);
    Sep := 'AND';
  end;

  if lblWorkPhone.Font.Color = clRed then
  begin
    Result := Format('%s %s (UPPER(WORK_PHONE) = "%s")',
                    [Result, Sep, UpperCase(edtWorkPhone.Text)]);
    Sep := 'AND';
  end;

  if lblAltPhone.Font.Color = clRed then
  begin
    Result := Format('%s %s (UPPER(ALT_PHONE) = "%s")',
                    [Result, Sep, UpperCase(edtAltPhone.Text)]);
    Sep := 'AND';
  end;

  if lblWorkAddress.Font.Color = clRed then
  begin
    Result := Format('%s %s (UPPER(WORK_ADDRESS) = "%s")',
                    [Result, Sep, UpperCase(edtWorkAddress.Text)]);
    Sep := 'AND';
  end;

  if lblAltAddress.Font.Color = clRed then
  begin
    Result := Format('%s %s (UPPER(ALT_ADDRESS) = "%s")',
                    [Result, Sep, UpperCase(edtAltAddress.Text)]);
    Sep := 'AND';
  end;

  if lblCompany.Font.Color = clRed then
  begin
    Result := Format('%s %s (UPPER(COMPANY) = "%s")',
                    [Result, Sep, UpperCase(edtCompany.Text)]);
  end;

  if Length(Result) > 0 then
    Result := Format('SELECT CUSTOMER_ID FROM CUSTOMER WHERE (%s)',
      [Result]);

end;

procedure TCustomerSearchForm.ClearEditFields;
{ This method clears all of the edit fields and sets their labels
  to clNavy in color. }
var
  i: word;
begin
  for i := 0 to ComponentCount - 1 do
  begin
    if Components[i] is TEdit then
      TEdit(Components[i]).Text := '';

    if Components[i] is TLabel then
      TLabel(Components[i]).Font.Color := clNavy;
  end;

end;

procedure TCustomerSearchForm.FormCreate(Sender: TObject);
begin
  FindPressed := False;
  // Clear the edit fields.
  ClearEditFields;
end;

procedure TCustomerSearchForm.FindCustBtnClick(Sender: TObject);
begin
  FindPressed := True;
  // Make the QueryString available to the caller of this dialog.
  QueryString := BuildSQLStatement;
end;

procedure TCustomerSearchForm.CancelBtnClick(Sender: TObject);
begin
  ClearEditFields;
end;

procedure TCustomerSearchForm.lblIDNumberClick(Sender: TObject);
{ All labels are hooked to this OnClick event handler which changes
  the color of the labels. The clRed color is used to specify a label on
  which to perform a search operation. }
begin
  with (Sender as TLabel) do
    if Font.Color = clNavy then
      Font.Color := clRed
    else
      Font.Color := clNavy;
end;

procedure TCustomerSearchForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  { Before closing the form to perform a search operation, make sure
    the user has specified on which fields to perform the search. }
  if (QueryString = '') and FindPressed then
  begin
    MessageDlg('You must highlight a search field by'+
               ' clicking on a label.', mtInformation, [mbOk], 0);
    Action := caNone;
  end
  else begin
    Action := caHide;
    ClearEditFields;
  end;
end;

end.
