unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ComCtrls, ExtCtrls, ChildFrm;

type

  { There are four types of child forms that can be displayed in this
    application. The TActiveScreenType is declared to allow us to know
    which of the four types of forms are active. }

  TActiveScreenType = (acCustomer, acParts, acSales, acNewSales);

  TMainForm = class(TForm)
    mmSales: TMainMenu;
    mmiScreen: TMenuItem;
    mmiCustomer: TMenuItem;
    mmiParts: TMenuItem;
    mmiNewSale: TMenuItem;
    mmiSales: TMenuItem;
    mmiFile: TMenuItem;
    mmiExit: TMenuItem;
    mmiHelp: TMenuItem;
    tcMain: TTabControl;
    mmiUser: TMenuItem;
    mmiLogon: TMenuItem;
    mmiLogoff: TMenuItem;
    imgCar: TImage;
    procedure ScreenClick(Sender: TObject);
    procedure mmiExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tcMainChange(Sender: TObject);
    procedure tcMainChanging(Sender: TObject; var AllowChange: Boolean);
    procedure mmiLogonClick(Sender: TObject);
    procedure mmiLogoffClick(Sender: TObject);
  private
    // ActiveScreenType stores the type of form that is active.
    ActiveScreenType: TActiveScreenType;
    // ActiveScreen is a reference to the active child form. 
    ActiveScreen: TChildForm;
    procedure SetActiveScreen;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses CustomerFrm, PartsFrm, NewSalesFrm, SalesFrm, SalesDM;

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  // Set the alignment for the main tab control.
  tcMain.Align := alClient;
end;

procedure TMainForm.mmiExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.ScreenClick(Sender: TObject);
begin
{ This method is invoked when the user has chosen to change the screen via
  the main menu.
  This method determines if it is possible to change to another child form. It
  does this by making sure that each child form's CanChange() method returns
  True. If so, it changes the global ActiveScreenType value and invokes the
  SetActiveScreen() method to actually perform the change logic. }
  if Sender is TMenuItem then
  begin
    if ActiveScreen <> nil then
    begin
      if ActiveScreen.CanChange then
      begin

        TMenuItem(Sender).Checked := True;
        if Sender = mmiCustomer then
          ActiveScreenType := acCustomer
        else if Sender = mmiParts then
          ActiveScreenType := acParts
        else if Sender = mmiSales then
          ActiveScreenType := acSales
        else if Sender = mmiNewSale then
          ActiveScreenType := acNewSales;

        // Ensure the TTabControl is in-sync with the clicked item on the menu.
        tcMain.TabIndex := ord(ActiveScreenType);
        SetActiveScreen;
      end
    end;
  end;
end;

procedure TMainForm.tcMainChange(Sender: TObject);
begin
{ This method changes the screen when the user has switched tabs. It
  synchronizes the settings for the main menu and the tab control. This method
  also calls the SetActiveScreen() method to actually change the active screen.}
  if ActiveScreen <> nil then
  begin
    case tcMain.TabIndex of
      0: mmiCustomer.Checked := True;
      1: mmiParts.Checked    := True;
      2: mmiSales.Checked    := True;
      3: mmiNewSale.Checked  := True;
    end;
    ActiveScreenType := TActiveScreenType(tcMain.TabIndex);
    SetActiveScreen;
  end;
end;

procedure TMainForm.SetActiveScreen;
{ This method changes the active screen to one of the four child forms. Each
  child form becomes a child of the TTabControl tcMain. }
var
  i: integer;
  TempScreen: TChildForm;
begin
  { Determine if we have an instantiated child form yet. If so, unmerge its
    menu and free the child form. }

  TempScreen := ActiveScreen;

  // Unmerge the menu.
  if Assigned(ActiveScreen) then
    if ActiveScreen.GetFormMenu <> nil then
      mmSales.UnMerge(ActiveScreen.GetFormMenu);

   { Determine which active screen (child form) to create and set its toolbar
     to have the main form as the parent if appropriate. }
  case ActiveScreenType of
    acCustomer:
      begin
        ActiveScreen := TCustomerForm.Create(Application, tcMain);
        TCustomerForm(ActiveScreen).SetToolBarParent(self);
      end;
    acParts:
      begin
        ActiveScreen := TPartsForm.Create(Application, tcMain);
        TPartsForm(ActiveScreen).SetToolBarParent(self);
      end;
    acSales:
        ActiveScreen := TSalesForm.Create(Application, tcMain);
    acNewSales:
      begin
        ActiveScreen := TNewSalesForm.Create(Application, tcMain);
        TNewSalesForm(ActiveScreen).SetToolBarParent(self);
      end;
  end;
  // Merge the menu of the child form with the menu of the main form.
  if ActiveScreen <> nil then
  begin
    if ActiveScreen.GetFormMenu <> nil then
      mmSales.Merge(ActiveScreen.GetFormMenu);
    ActiveScreen.Show;
  end;

  if Assigned(TempScreen) then
    TempScreen.Free;

end;

procedure TMainForm.tcMainChanging(Sender: TObject;
  var AllowChange: Boolean);
begin
  // Change only if the child form is in the mode that allows changing.
  AllowChange := ActiveScreen.CanChange;
end;

procedure TMainForm.mmiLogonClick(Sender: TObject);
begin
  // Log the user onto the system
  if DDGSalesDataModule.Login then
  begin
    tcMain.Align := alClient;
    tcMain.Visible := True;
    ActiveScreenType := acCustomer;
    SetActiveScreen;
    mmiScreen.Enabled := True;
    mmiLogon.Enabled  := False;
    mmiLogoff.Enabled := True;
  end;
end;

procedure TMainForm.mmiLogoffClick(Sender: TObject);
begin
  // Log the user off the system.
  if Assigned(ActiveScreen) then
  begin
    if ActiveScreen.GetFormMenu <> nil then
      mmSales.UnMerge(ActiveScreen.GetFormMenu);
    ActiveScreen.Free;
    ActiveScreen := nil;
  end;

  tcMain.Visible := False;
  DDGSalesDataModule.Logout;

  mmiScreen.Enabled := False;
  mmiLogon.Enabled  := True;
  mmiLogoff.Enabled := False;

end;

end.
