unit NewSalesFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  CHILDFRM, Grids, DBGrids, Buttons, StdCtrls, Db, Menus, ToolWin, ComCtrls,
  ImgList;

type
  TNewSalesForm = class(TChildForm)
    dsParts: TDataSource;
    dsTempItems: TDataSource;
    lblCustomer1: TLabel;
    lblCustomerName: TLabel;
    lblTotCost: TLabel;
    lblTotalCost: TLabel;
    lblSelectParts: TLabel;
    sbAddPart: TSpeedButton;
    sbRemovePart: TSpeedButton;
    lblSaleItems: TLabel;
    dbgParts: TDBGrid;
    dbgSaleItems: TDBGrid;
    mmFormMenu: TMainMenu;
    mmiSales: TMenuItem;
    mmiNew: TMenuItem;
    mmiCancel: TMenuItem;
    mmiSave: TMenuItem;
    tbSales: TToolBar;
    sbAccept: TToolButton;
    sbCancel: TToolButton;
    tb1: TToolButton;
    sbInsert: TToolButton;
    ilNavigationBar: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure sbAddPartClick(Sender: TObject);
    procedure mmiNewClick(Sender: TObject);
    procedure mmiCancelClick(Sender: TObject);
    procedure mmiSaveClick(Sender: TObject);
  private
    AddingSale: Boolean;
    function CanChange: Boolean; override;
    procedure SetSaleMenus;
    procedure TempItemsAfterChange(DataSet: TDataSet);
  public
    function GetFormMenu: TMainMenu; override;
    procedure SetToolBarParent(AParent: TWinControl);
  end;

var
  NewSalesForm: TNewSalesForm;

implementation

uses SalesDM;

{$R *.DFM}

function TNewSalesForm.CanChange: Boolean;
begin
  Result := not AddingSale;
end;

procedure TNewSalesForm.FormCreate(Sender: TObject);
begin
  inherited;
  // The tblTempItems table on DDGSalesDataModule is required for this form.
  DDGSalesDataModule.OpenTempItems;
  AddingSale := False; // Initially we're not adding a sale.

  { Assign the TempItemsAfterChange event handler to the event handlers
    surfaced by DDGSalesDataModule. } 
  DDGSalesDataModule.AfterTempItemsChange := TempItemsAfterChange;
  SetSaleMenus;
end;

procedure TNewSalesForm.FormDestroy(Sender: TObject);
begin
  // Close the DDGSalesDataModule.tblTempItems table. 
  inherited;
  DDGSalesDataModule.CloseTempItems;
end;

procedure TNewSalesForm.FormShow(Sender: TObject);
begin
  // Retrieve the customer name for the current customer. 
  inherited;
  lblCustomerName.Caption := DDGSalesDataModule.GetCustomerName;
  { The total should show a balance of zero since the form has just been
    invoked. }
  lblTotalCost.Caption    := '$   0.00';
end;

procedure TNewSalesForm.TempItemsAfterChange(DataSet: TDataSet);
begin
  // This is required in the AfterPost event of the tblTempItems on the datamodule
  // because we must recalculate this everytime the user makes a change.
  lblTotalCost.Caption := FormatFloat('$#,##0.00	', DDGSalesDataModule.SaleItemsTotalPrice);
end;

procedure TNewSalesForm.sbAddPartClick(Sender: TObject);
begin
  // Add the selected item to the sale.
  inherited;
  DDGSalesDataModule.AddItemToSale;
end;

procedure TNewSalesForm.mmiNewClick(Sender: TObject);
begin
  // Set the form into a mode to repesent adding a sale. 
  inherited;
  AddingSale := True;
  SetSaleMenus;
end;

procedure TNewSalesForm.mmiCancelClick(Sender: TObject);
begin
  // Cancel the current sale.
  inherited;
  AddingSale := False;
  DDGSalesDataModule.CancelSale;
  SetSaleMenus;
end;

procedure TNewSalesForm.mmiSaveClick(Sender: TObject);
begin
  // Save the current sale.
  inherited;
  DDGSalesDataModule.SaveSale;
  AddingSale := False;
  SetSaleMenus;
  { Invoke the TempItemsAfterChange event handler to ensure that the form
    updates is controls accordingly. }
  TempItemsAfterChange(nil);
end;

procedure TNewSalesForm.SetSaleMenus;
begin
  // Set menu items and speed buttons to reflect the form's mode. 
  mmiNew.Enabled       := not AddingSale;
  mmiCancel.Enabled    := AddingSale;
  mmiSave.Enabled      := AddingSale;
  sbAddPart.Enabled    := AddingSale;
  sbRemovePart.Enabled := AddingSale;

  sbAccept.Enabled     := mmiSave.Enabled;
  sbCancel.Enabled     := mmiCancel.Enabled;
  sbInsert.Enabled     := mmiNew.Enabled;

end;

function TNewSalesForm.GetFormMenu: TMainMenu;
begin
  // Return the main menu to be used by the main form for menu merging. 
  Result := mmFormMenu;
end;

procedure TNewSalesForm.SetToolBarParent(AParent: TWinControl);
begin
  { This form uses a toolbar, return its parent. We were required to
    create this method for this form as it is a descendent of TChildForm,
    not TDBNavStatForm which already contains this method. }
  tbSales.Parent := AParent;
end;

end.
