unit SalesDM;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBTables, Db;

type
  TDDGSalesDataModule = class(TDataModule)
    qryCustomer: TQuery;
    dbSales: TDatabase;
    usqlCustomer: TUpdateSQL;
    qryCustomerCUSTOMER_ID: TIntegerField;
    qryCustomerFNAME: TStringField;
    qryCustomerLNAME: TStringField;
    qryCustomerCREDIT_LINE: TSmallintField;
    qryCustomerWORK_ADDRESS: TStringField;
    qryCustomerALT_ADDRESS: TStringField;
    qryCustomerCITY: TStringField;
    qryCustomerSTATE: TStringField;
    qryCustomerZIP: TStringField;
    qryCustomerWORK_PHONE: TStringField;
    qryCustomerALT_PHONE: TStringField;
    qryCustomerCOMMENTS: TMemoField;
    qryCustomerCOMPANY: TStringField;
    qryParts: TQuery;
    usqlParts: TUpdateSQL;
    qryPartsPART_NUMBER: TStringField;
    qryPartsDESCRIPTION: TStringField;
    qryPartsQUANTITY: TSmallintField;
    qryPartsLIST_PRICE: TFloatField;
    qryPartsRETAIL_PRICE: TFloatField;
    qryPartsDEALER_PRICE: TFloatField;
    qryPartsJOBBER_PRICE: TFloatField;
    spInsertSaleItem: TStoredProc;
    spInsertSale: TStoredProc;
    qryTotalPrice: TQuery;
    tblTempItems: TTable;
    tblTempItemsPART_NUMBER: TStringField;
    tblTempItemsDESCRIPTION: TStringField;
    tblTempItemsQUANTITY: TSmallintField;
    tblTempItemsRETAIL_PRICE: TFloatField;
    tblTempItemsTOTAL_PRICE: TFloatField;
    qryTotalPriceSUMOFTOTAL_PRICE: TFloatField;
    qrySale: TQuery;
    dsCustomer: TDataSource;
    qryItems: TQuery;
    dsSale: TDataSource;
    qrySaleSALE_NUMBER: TIntegerField;
    qrySaleSALE_DATE: TDateTimeField;
    qrySaleTOTAL_PRICE: TFloatField;
    qryItemsDESCRIPTION: TStringField;
    qryItemsQTY: TSmallintField;
    qryCustomerSearch: TQuery;
    procedure tblTempItemsBeforePost(DataSet: TDataSet);
    procedure dbSalesLogin(Database: TDatabase; LoginParams: TStrings);
  protected
    procedure SetAfterTempItemsChange(Value: TDataSetNotifyEvent);
    function GetAfterTempItemsChange: TDataSetNotifyEvent;
  public

    // Connection methods

    procedure Logout;
    function Login: Boolean;
    function Connect: Boolean;
    procedure Disconnect;

    // Customer methods
    procedure FirstCustomer;
    procedure LastCustomer;
    procedure NextCustomer;
    procedure PrevCustomer;
    procedure EditCustomer;
    procedure NewCustomer;
    procedure AcceptCustomer;
    procedure CancelCustomer;
    procedure DeleteCustomer;
    function IsFirstCustomer: Boolean;
    function IsLastCustomer: Boolean;
    function GetCustomerName: String;
    function SearchForCustomer: Boolean;

    // Parts methods
    procedure FirstPart;
    procedure LastPart;
    procedure NextPart;
    procedure PrevPart;
    procedure EditPart;
    procedure NewPart;
    procedure AcceptPart;
    procedure CancelPart;
    procedure DeletePart;
    function IsFirstPart: Boolean;
    function IsLastPart: Boolean;
    function SearchForPart: Boolean;

    // Sales methods

    procedure AddItemToSale;
    procedure SaveSale;
    procedure CancelSale;
    function SaleItemsTotalPrice: double;
    procedure OpenTempItems;
    procedure CloseTempItems;

    // Surfaced properties
    property AfterTempItemsChange: TDataSetNotifyEvent read GetAfterTempItemsChange
      write SetAfterTempItemsChange;

  end;

var
  DDGSalesDataModule: TDDGSalesDataModule;

implementation

uses CustomerSrchFrm, LoginFrm;

{$R *.DFM}


procedure TDDGSalesDataModule.SetAfterTempItemsChange(Value: TDataSetNotifyEvent);
begin
  { This writer method adds the Value parameter to both the AfterPost and
    AfterDelete events of the temporary items table. This ensures that whenever
    the data changes, the event handler will get called. } 
  tblTempItems.AfterPost   := Value;
  tblTempItems.AfterDelete := Value;
end;

function TDDGSalesDataModule.GetAfterTempItemsChange: TDataSetNotifyEvent;
begin
  Result := tblTempItems.AfterPost;
end;

// Login methods.

procedure TDDGSalesDataModule.dbSalesLogin(Database: TDatabase;
  LoginParams: TStrings);
begin
  { Calls method below to populate the LoginParams strings list
    with the user's login information. GetLoginParams is defined in
    LoginFrm.pas. }
  GetLoginParams(LoginParams);
end;

procedure TDDGSalesDataModule.Logout;
begin
  Disconnect;
end;

function TDDGSalesDataModule.Login: Boolean;
begin
  Result := Connect;
end;

function  TDDGSalesDataModule.Connect: Boolean;
begin
  { Connects the user to the database. When dbSales is set to True, its OnLogon
    event handler will be invoked which will invoke our customer login dialog
    defined in LoginFrm.pas. }
  try
    dbSales.Connected  := True;
    qryCustomer.Active := True;
    qryParts.Active    := True;
    qrySale.Active     := True;
    qryItems.Active    := True;
    Result := True;
  except
    MessageDlg('Invalid Password or login information, cannot login.', mtError, [mbok], 0);
    dbSales.Connected := False;
    Result := False;
  end;
end;

procedure TDDGSalesDataModule.Disconnect;
begin
  // Disconnect from the database.
  dbSales.Connected := False;
end;

// Customer methods

procedure TDDGSalesDataModule.AcceptCustomer;
begin
  dbSales.ApplyUpdates([qryCustomer]);
end;

procedure TDDGSalesDataModule.CancelCustomer;
begin
  qryCustomer.CancelUpdates;
end;

procedure TDDGSalesDataModule.DeleteCustomer;
begin
  qryCustomer.Delete;
end;

procedure TDDGSalesDataModule.EditCustomer;
begin
  qryCustomer.Edit;
end;

procedure TDDGSalesDataModule.FirstCustomer;
begin
  qryCustomer.First;
end;

procedure TDDGSalesDataModule.LastCustomer;
begin
  qryCustomer.Last;
end;

procedure TDDGSalesDataModule.NewCustomer;
begin
  qryCustomer.Insert;
end;

procedure TDDGSalesDataModule.NextCustomer;
begin
  qryCustomer.Next;
end;

procedure TDDGSalesDataModule.PrevCustomer;
begin
  qryCustomer.Prior;
end;

function TDDGSalesDataModule.IsFirstCustomer: Boolean;
begin
  Result := qryCustomer.Bof;
end;

function TDDGSalesDataModule.IsLastCustomer: Boolean;
begin
  Result := qryCustomer.Eof;
end;

function TDDGSalesDataModule.GetCustomerName: String;
begin
  { Normally, return the company name. If there is not a company name, return
    the customer's name. }
  if qryCustomerCOMPANY.AsString <> EmptyStr then
    Result := qryCustomerCOMPANY.AsString
  else
    Result := Format('%s %s', [qryCustomerFNAME.AsString, qryCustomerLNAME.AsString]);
end;

function TDDGSalesDataModule.SearchForCustomer: Boolean;
var
  CustID: Integer;
  SearchQry: String;
begin
  // Assume failure.
  Result := False;
  { Invoke the SearchCustomer function which is defined in CustomerSrchFrm.pas.
    this function returns the query string that is added to the qryCustomerSearch
    TQuery component }
  SearchQry := SearchCustomer;
  if SearchQry <> EmptyStr then
  begin
    Screen.Cursor := crSQLWait;
    try
        qryCustomerSearch.Close;
        qryCustomerSearch.SQL.Clear;
        qryCustomerSearch.SQL.Add(SearchQry);
        qryCustomerSearch.Open;
        try

          // If a record was not found, exit this method.
          if qryCustomerSearch.FieldByName('CUSTOMER_ID').IsNull then
          begin
            Screen.Cursor := crDefault;
            Exit;
          end;

          { If a record is found, get the customer's id that is used to
            locate the record in the actual qryCustomer, TQuery component. This
            will position the cursor to the location of the record. }
          CustID := qryCustomerSearch.FieldByName('CUSTOMER_ID').AsInteger;

          { If the record is not found in qryCustomer, there is an inconsistency
            in the database, raise an error. }
          if not qryCustomer.Locate('CUSTOMER_ID', CustID, []) then
            raise Exception.Create('Inconsistency in database.')
          else
            Result := True;
        finally
          qryCustomerSearch.Close;
        end;
    finally
      Screen.Cursor := crDefault;
    end;
  end
  else
    Result := False;;
end;

// Parts Methods

function TDDGSalesDataModule.IsFirstPart: Boolean;
begin
  Result := qryParts.Bof;
end;

function TDDGSalesDataModule.IsLastPart: Boolean;
begin
  Result := qryParts.Eof;
end;

procedure TDDGSalesDataModule.AcceptPart;
begin
  dbSales.ApplyUpdates([qryParts]);
end;

procedure TDDGSalesDataModule.CancelPart;
begin
  qryParts.CancelUpdates;
end;

procedure TDDGSalesDataModule.DeletePart;
begin
  qryParts.Delete;
end;

procedure TDDGSalesDataModule.EditPart;
begin
  qryParts.Edit;
end;

procedure TDDGSalesDataModule.FirstPart;
begin
  qryParts.First;
end;

procedure TDDGSalesDataModule.LastPart;
begin
  qryParts.Last;
end;

procedure TDDGSalesDataModule.NewPart;
begin
  qryParts.Insert;
end;

procedure TDDGSalesDataModule.NextPart;
begin
  qryParts.Next;
end;

procedure TDDGSalesDataModule.PrevPart;
begin
  qryParts.Prior;
end;

function TDDGSalesDataModule.SearchForPart: Boolean;
{ This method searches for a part based on the part id specified by the
  user. }
var
  PartNumber: string;
begin
  Result := False;
  PartNumber := '';
  if InputQuery('Part Search', 'Enter a Part Number', PartNumber) then
    if not qryParts.Locate('PART_NUMBER', PartNumber, []) then
      Exit
    else
      Result := True;
end;

// Sales methods

procedure TDDGSalesDataModule.AddItemToSale;
begin
  { The tblTempItems is a temporary table used to hold the
    items that are being added to a sale. If the user saves
    the sale, these records will be used in the stored procedure
    calls that actually store the sale on the database. }
  if not tblTempItems.Locate('PART_NUMBER',
    qryParts.FieldByName('PART_NUMBER').AsString, []) then
  begin
    tblTempItems.Insert;
    try
      tblTempItems['PART_NUMBER']  := qryParts['PART_NUMBER'];
      tblTempItems['DESCRIPTION']  := qryParts['DESCRIPTION'];
      tblTempItems['QUANTITY']     := 1;
      tblTempItems['RETAIL_PRICE'] := qryParts['RETAIL_PRICE'];
      tblTempItems.Post;
    except
      tblTempItems.Cancel;
    end;
  end
  else
    MessageDlg('Item already in list', mtWarning, [mbok], 0);
end;

procedure TDDGSalesDataModule.CancelSale;
begin
  { If the user cancels the sale, the items that were added to the tblTempItems
    table will have to be cleared. }
  tblTempItems.Close;
  tblTempItems.EmptyTable;
  tblTempItems.Open;
end;

procedure TDDGSalesDataModule.SaveSale;
var
  SaleNo: Integer;
begin
  { If the user saves the sale, first create a sale record which will return
    a sale key to SaleNo. This is used as the link for sale items which are
    added next. The sale items are gotten from the temporary table
    tblTempItems. }
  dbSales.StartTransaction;
  try
    { First create the sale record. }
    with spInsertSale do
    begin
      ParamByName('iCUSTOMER_ID').AsInteger := qryCustomer['CUSTOMER_ID'];
      ParamByName('iSALE_DATE').AsDateTime  := Now;
      ParamByName('iTOTAL_PRICE').AsFloat   := SaleItemsTotalPrice;
      ExecProc;
      // Get the key value in SaleNo. 
      SaleNo := ParamByName('rSALE_NUMBER').AsInteger;
    end;

    // Now add all records in tblTempItems to the sale specified by SaleNo. 
    tblTempItems.First;
    while not tblTempItems.Eof do
    begin
      with spInsertSaleItem do
      begin
        ParamByName('IPART_NO').AsString      := tblTempItems['PART_NUMBER'];
        ParamByName('IQTY').AsInteger         := tblTempItems['QUANTITY'];
        ParamByName('ISALE_NUMBER').AsInteger := SaleNo;
        ExecProc;
      end;
      tblTempItems.Next;
    end;

    dbSales.Commit;

    // Refresh modified tables.
    qryParts.Close;
    qryParts.Open;

    tblTempItems.Close;
    tblTempItems.EmptyTable;
    tblTempItems.Open;

  except
    dbSales.Rollback;
  end;
end;

function TDDGSalesDataModule.SaleItemsTotalPrice: double;
begin
  { qryTotalPrice retrieves the total price for all records added to the
    tblTempItems table. This method may be called the any form using this
    datamodule. }
  qryTotalPrice.Close;
  qryTotalPrice.Open;
  try
    Result := qryTotalPrice.FieldByName('SUM OF TOTAL_PRICE').AsFloat;
  finally
    qryTotalPrice.Close;
  end;
end;

procedure TDDGSalesDataModule.tblTempItemsBeforePost(DataSet: TDataSet);
begin
  { Before posting a record to the temporary table, calculate the total price
    for the TOTAL_PRICE field based on the number of items that the user is
    adding. }
  tblTempItemsTOTAL_PRICE.ReadOnly := False;
  try
    tblTempItems['TOTAL_PRICE'] := tblTempItems['RETAIL_PRICE'] *
      tblTempItems['QUANTITY'];
  finally
    tblTempItemsTOTAL_PRICE.ReadOnly := True;
  end;
end;

procedure TDDGSalesDataModule.OpenTempItems;
begin
  tblTempItems.Close;
  tblTempItems.EmptyTable;
  tblTempItems.Open;
end;

procedure TDDGSalesDataModule.CloseTempItems;
begin
  tblTempItems.Active := False;
end;

end.
