unit CustDM;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBClient, MConnect, Db;

const
  cFileName        = 'CustData.cds';

  cRegIniFile      = 'Software\DDG Client App';
  cRegSection      = 'Startup Config';
  cRegOnlineIdent  = 'Run Online';

type

  TFilterType = (ftNone, ftByCity, ftByState);

  TAddErrorToClientEvent = procedure(const AFieldName, OldStr, NewStr,
    CurStr, ErrMsg: String) of Object;


  TCustomerDataModule = class(TDataModule)
    cdsCust: TClientDataSet;
    dcomCust: TDCOMConnection;
    cdsCustCUSTOMER_ID: TIntegerField;
    cdsCustFNAME: TStringField;
    cdsCustLNAME: TStringField;
    cdsCustCREDIT_LINE: TSmallintField;
    cdsCustWORK_ADDRESS: TStringField;
    cdsCustALT_ADDRESS: TStringField;
    cdsCustCITY: TStringField;
    cdsCustSTATE: TStringField;
    cdsCustZIP: TStringField;
    cdsCustWORK_PHONE: TStringField;
    cdsCustALT_PHONE: TStringField;
    cdsCustCOMMENTS: TMemoField;
    cdsCustCOMPANY: TStringField;
    procedure CustomerDataModuleCreate(Sender: TObject);
    procedure cdsCustReconcileError(DataSet: TClientDataSet;
      E: EReconcileError; UpdateKind: TUpdateKind;
      var Action: TReconcileAction);
    procedure CustomerDataModuleDestroy(Sender: TObject);
    procedure cdsCustFilterRecord(DataSet: TDataSet; var Accept: Boolean);
  private
    FFilterType: TFilterType;
    FFilterStr: String;
    FOnAddErrorToClient: TAddErrorToClientEvent;

    function GetOnline: Boolean;
    procedure SetOnline(const Value: Boolean);
    { Private declarations }
  protected
    function GetChangeCount: Integer;

  public
    procedure EditClient;
    procedure AddClient;
    procedure SaveClient;
    procedure CancelClient;
    procedure DeleteClient;
    procedure ApplyUpdates;
    procedure CancelUpdates;
    procedure First;
    procedure Previous;
    procedure Next;
    procedure Last;
    function IsBOF: Boolean;
    function IsEOF: Boolean;

    procedure FilterByState;
    procedure FilterByCity;
    procedure NoFilter;

    property ChangeCount: Integer read GetChangeCount;
    property Online: Boolean read GetOnline write SetOnline;

    property OnAddErrorToClient: TAddErrorToClientEvent read FOnAddErrorToClient
      write FOnAddErrorToClient;

  end;

var
  CustomerDataModule: TCustomerDataModule;

implementation
uses MainCustFrm, Registry;

{$R *.DFM}

procedure TCustomerDataModule.AddClient;
begin
  cdsCust.Insert;
end;

procedure TCustomerDataModule.ApplyUpdates;
begin
  cdsCust.ApplyUpdates(-1);
end;

procedure TCustomerDataModule.CancelClient;
begin
  cdsCust.Cancel;
end;

procedure TCustomerDataModule.CancelUpdates;
begin
  cdsCust.CancelUpdates;
end;

procedure TCustomerDataModule.DeleteClient;
begin
  if MessageDlg('Are you sure you want to delete the current record?',
  mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      cdsCust.Delete;
end;

procedure TCustomerDataModule.EditClient;
begin
  cdsCust.Edit;
end;

function TCustomerDataModule.IsBOF: Boolean;
begin
  Result := cdsCust.Bof;
end;

function TCustomerDataModule.IsEOF: Boolean;
begin
  Result := cdsCust.Eof;
end;

procedure TCustomerDataModule.First;
begin
  cdsCust.First;
end;

procedure TCustomerDataModule.Last;
begin
  cdsCust.Last;
end;

procedure TCustomerDataModule.Next;
begin
  cdsCust.Next;
end;

procedure TCustomerDataModule.Previous;
begin
  cdsCust.Prior;
end;

procedure TCustomerDataModule.SaveClient;
begin
  cdsCust.Post;
end;

procedure TCustomerDataModule.cdsCustReconcileError(
  DataSet: TClientDataSet; E: EReconcileError; UpdateKind: TUpdateKind;
  var Action: TReconcileAction);
{ If an error occurs, update the appropriate TListview on the main form with
  the error data.  }
var
  CurStr, NewStr, OldStr: String;
  i: integer;
  V: Variant;

procedure SetString(V: Variant; var S: String);
{ We must test for a null value on V which would be returned if the
  table field was null. This is necessary because we can't typcast null
  as a string. }
begin

  if VarIsNull(V) then
    S := EmptyStr
  else
    S := String(V);
end;

begin
  for i := 0 to DataSet.FieldCount - 1 do
  begin

    V := DataSet.Fields[i].NewValue;
    SetString(V, NewStr);

    V := DataSet.Fields[i].CurValue;
    SetString(V, CurStr);

    V := DataSet.Fields[i].OldValue;
    SetString(V, OldStr);

    if NewStr <> CurStr then
      if Assigned(FOnAddErrorToClient) then
        FOnAddErrorToClient(DataSet.Fields[i].FieldName, OldStr, NewStr,
        CurStr, E.Message)
  end;
  //Update record and removes changes from the change log.  
  Action := raRefresh;
end;

function TCustomerDataModule.GetChangeCount: Integer;
begin
  Result := cdsCust.ChangeCount;
end;

function TCustomerDataModule.GetOnline: Boolean;
begin
  Result := dcomCust.Connected;
end;

procedure TCustomerDataModule.SetOnline(const Value: Boolean);
{ ** JOSH,  It seems to me that the result set from the server
  should take presidence over what's in file referred to by
  FileName. }
begin

 if Value = True then
 begin
   dcomCust.Connected := True;

   if cdsCust.ChangeCount > 0 then begin
     ShowMessage('You''re changes must be applied before going online');
     cdsCust.ApplyUpdates(-1);
   end;
   cdsCust.Refresh;

 end
 else begin
   cdsCust.FileName := cFileName;
   dcomCust.Connected := False;
 end;
end;

procedure TCustomerDataModule.CustomerDataModuleCreate(Sender: TObject);
{ Determine if the user last left the application online or offline and
  re-launch the application in that same mode. }
var
  RegIniFile: TRegIniFile;
  IsOnline: Boolean;
begin
  RegIniFile := TRegIniFile.Create(cRegIniFile);
  try
    IsOnline := RegIniFile.ReadBool(cRegSection, cRegOnlineIdent, True);
  finally
    RegIniFile.Free;
  end;

  if IsOnline then
  begin
    dcomCust.Connected := True;
    cdsCust.Open;
  end
  else begin
    cdsCust.FileName := cFileName;
    cdsCust.Open;
  end;

end;

procedure TCustomerDataModule.CustomerDataModuleDestroy(Sender: TObject);
{ Save the online/offline status of the application to the registry. When
  the user runs the application again, it will launch as it was last
  brought down. }
var
  RegIniFile: TRegIniFile;
begin
  RegIniFile := TRegIniFile.Create(cRegIniFile);
  try
    RegIniFile.WriteBool(cRegSection, cRegOnlineIdent, Online);
  finally
    RegIniFile.Free;
  end;
end;

procedure TCustomerDataModule.FilterByCity;
{ If we're online, let the server apply the filter so that we only retrieve
  the records we want. Otherwise, apply the filter to the in-memory result
  set of cdsCust. }
var
  CityStr: String;
  Data: OleVariant;
begin
  InputQuery('Filter on City', 'Enter City: ', CityStr);
  FFilterStr := CityStr;

  if Online then
  begin
    dcomCust.AppServer.FilterByCity(CityStr, Data);
(* Note to myself --- doesn't seem that I need to make this
   data assignment below. *)

//    cdsCust.Data := Data;
    cdsCust.Refresh;
  end
  else begin
    FFilterType      := ftByCity;
    cdsCust.Filtered := True;
    cdsCust.Refresh;
  end;
end;

procedure TCustomerDataModule.FilterByState;
{ If we're online, let the server apply the filter so that we only retrieve
  the records we want. Otherwise, apply the filter to the in-memory result
  set of cdsCust. }
var
  StateStr: String;
  Data: OleVariant;
begin
  InputQuery('Filter on State', 'Enter State: ', StateStr);
  FFilterStr := StateStr;

  if Online then
  begin
    dcomCust.AppServer.FilterByState(StateStr, Data);

(* Note to myself --- doesn't seem that I need to make this data
   assignment below. *)

//    cdsCust.Data := Data;
    cdsCust.Refresh;
  end
  else begin
    FFilterType      := ftByState;
    cdsCust.Filtered := True;
    cdsCust.Refresh;
  end;
end;

procedure TCustomerDataModule.NoFilter;
{ If we're online, let the server apply the filter so that we only retrieve
  the records we want. Otherwise, apply the filter to the in-memory result
  set of cdsCust. }
var
  Data: OleVariant;
begin

  if Online then
  begin
    dcomCust.AppServer.NoFilter(Data);

(* Note to myself --- doesn't seem that I need to make this data
   assignment below. *)

//    cdsCust.Data := Data;
    cdsCust.Refresh;
  end
  else begin
    FFilterType      := ftNone;
    cdsCust.Filtered := False;
    cdsCust.Refresh;
  end;
end;

procedure TCustomerDataModule.cdsCustFilterRecord(DataSet: TDataSet;
  var Accept: Boolean);
begin
  case FFilterType of
    ftByCity:  Accept := DataSet.FieldByName('CITY').AsString = FFilterStr;
    ftByState: Accept := DataSet.FieldByName('STATE').AsString = FFilterStr;
    ftNone:  Accept := True;
  end;
end;

end.
