unit MainCustFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBNAVSTATFRM, Db, StdCtrls, DBCtrls, Mask, ComCtrls, Menus, ImgList,
  ToolWin, DBMODEFRM, Grids, DBGrids;

type

  TMainCustForm = class(TDBNavStatForm)
    pcClients: TPageControl;
    dsClientDetail: TTabSheet;
    lblFirstName: TLabel;
    lblLastName: TLabel;
    lblCreditLine: TLabel;
    lblWorkAddress: TLabel;
    lblAltAddress: TLabel;
    lblCity: TLabel;
    lblState: TLabel;
    lblZipCode: TLabel;
    lblWorkPhone: TLabel;
    lblAltPhone: TLabel;
    lblCompany: TLabel;
    dbeFirstName: TDBEdit;
    dbeLastName: TDBEdit;
    dbeCreditLine: TDBEdit;
    dbeWorkAddress: TDBEdit;
    dbeAltAddress: TDBEdit;
    dbeCity: TDBEdit;
    dbeState: TDBEdit;
    dbeZipCode: TDBEdit;
    dbeWorkPhone: TDBEdit;
    dbeAltPhone: TDBEdit;
    dbeCompany: TDBEdit;
    tsComments: TTabSheet;
    dbmComments: TDBMemo;
    dsClients: TDataSource;
    SaveDialog1: TSaveDialog;
    OpenDialog1: TOpenDialog;
    mmiSave: TMenuItem;
    N3: TMenuItem;
    mmiApplyUpdates: TMenuItem;
    mmiCancelUpdates: TMenuItem;
    mmiMode: TMenuItem;
    mmiOffline: TMenuItem;
    mmiOnline: TMenuItem;
    tsErrors: TTabSheet;
    lvClient: TListView;
    mmiExit: TMenuItem;
    mmiFilter: TMenuItem;
    mmiByState: TMenuItem;
    mmiByCity: TMenuItem;
    mmiNoFilter: TMenuItem;
    tsClientList: TTabSheet;
    DBGrid1: TDBGrid;
    procedure sbAcceptClick(Sender: TObject);
    procedure sbCancelClick(Sender: TObject);
    procedure sbInsertClick(Sender: TObject);
    procedure sbEditClick(Sender: TObject);
    procedure sbDeleteClick(Sender: TObject);
    procedure sbFirstClick(Sender: TObject);
    procedure sbPrevClick(Sender: TObject);
    procedure sbNextClick(Sender: TObject);
    procedure sbLastClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure mmiOnlineClick(Sender: TObject);
    procedure mmiApplyUpdatesClick(Sender: TObject);
    procedure mmiCancelUpdatesClick(Sender: TObject);
    procedure dsClientsDataChange(Sender: TObject; Field: TField);
    procedure Exit1Click(Sender: TObject);
    procedure mmiExitClick(Sender: TObject);
    procedure mmiByStateClick(Sender: TObject);
    procedure mmiByCityClick(Sender: TObject);
    procedure mmiNoFilterClick(Sender: TObject);
  private
    procedure SetControls;
    procedure GoToOnlineMode;
    procedure GoToOfflineMode;

  public
    procedure AddErrorToClient(const aFieldName, aOldValue, aNewValue,
      aCurValue, aErrorStr: String);
  end;

var
  MainCustForm: TMainCustForm;

implementation

uses CustDM;

{$R *.DFM}

procedure TMainCustForm.AddErrorToClient(const aFieldName, aOldValue, aNewValue,
  aCurValue, aErrorStr: String);
{ This method is used to add a TListItem to the TListView, aLV. The items
  added here give an indication of the errors that occur when performing
  updates to the server data. }
var
  NewItem: TListItem;
begin
  NewItem := lvClient.Items.Add;
  NewItem.Caption := aFieldName;
  NewItem.SubItems.Add(aOldValue);
  NewItem.SubItems.Add(aNewValue);
  NewItem.SubItems.Add(aCurValue);
  NewItem.SubItems.Add(aErrorStr);
end;

procedure TMainCustForm.SetControls;
begin
  // Ensure that the navigational buttons are set according to the form's mode.
  sbFirst.Enabled := not CustomerDataModule.IsBof;
  sbLast.Enabled  := not CustomerDataModule.IsEof;
  sbPrev.Enabled  := not CustomerDataModule.IsBof;
  sbNext.Enabled  := not CustomerDataModule.IsEof;

  // syncronize the navigational menu items with the speedbuttons.
  mmiFirst.Enabled    := sbFirst.Enabled;
  mmiLast.Enabled     := sbLast.Enabled;
  mmiPrevious.Enabled := sbPrev.Enabled;
  mmiNext.Enabled     := sbNext.Enabled;

  // Set other menus accordingly

  mmiApplyUpdates.Enabled  := mmiOnline.Checked and (FormMode = fmBrowse) and
    (CustomerDataModule.ChangeCount > 0);
  mmiCancelUpdates.Enabled := mmiOnline.Checked and (FormMode = fmBrowse) and
    (CustomerDataModule.ChangeCount > 0);


  mmiOnline.Checked  := CustomerDataModule.Online;
  mmiOffline.Checked := not mmiOnline.Checked;

  stbStatusBar.Panels[0].Text := Format('Changed Records: %d',
    [CustomerDataModule.ChangeCount]);

  if CustomerDataModule.Online then
    stbStatusBar.Panels[2].Text := 'Working Online'
  else
    stbStatusBar.Panels[2].Text := 'Working Offline'


end;

procedure TMainCustForm.sbAcceptClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.SaveClient;
  SetControls;
end;

procedure TMainCustForm.sbCancelClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.CancelClient;
  SetControls;
end;

procedure TMainCustForm.sbInsertClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.AddClient;
  SetControls;
end;

procedure TMainCustForm.sbEditClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.EditClient;
  SetControls;
end;

procedure TMainCustForm.sbDeleteClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.DeleteClient;
  SetControls;
end;

procedure TMainCustForm.sbFirstClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.First;
  SetControls;
end;

procedure TMainCustForm.sbPrevClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.Previous;
  SetControls;
end;

procedure TMainCustForm.sbNextClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.Next;
  SetControls;
end;

procedure TMainCustForm.sbLastClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.Last;
  SetControls;
end;

procedure TMainCustForm.FormCreate(Sender: TObject);
begin
  inherited;
  CustomerDataModule.OnAddErrorToClient := AddErrorToClient;
  SetControls;

  // Make these guys refer to each other so that they reset the other
  mmiOnline.Tag := Longint(mmiOffline);
  mmiOffline.Tag := Longint(mmiOnline);
end;

procedure TMainCustForm.GoToOnlineMode;
begin
  CustomerDataModule.Online := True;
  SetControls;
end;

procedure TMainCustForm.GoToOfflineMode;
begin
  CustomerDataModule.Online := False;
  SetControls;
end;

procedure TMainCustForm.mmiOnlineClick(Sender: TObject);
var
  mi: TMenuItem;
begin
  inherited;
  mi := Sender as TMenuItem;

  if not mi.Checked then
  begin

    mi.Checked := not mi.Checked;
    TMenuItem(mi.Tag).Checked := not mi.Checked;

    if mi = mmiOnline then
    begin
      if mi.Checked then
        GoToOnlineMode
      else
        GoToOffLineMode
    end

    else begin
      if mi.Checked then
        GoToOfflineMode
      else
        GoToOnlineMode
    end;
  end;
end;

procedure TMainCustForm.mmiApplyUpdatesClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.ApplyUpdates;
  SetControls;
end;

procedure TMainCustForm.mmiCancelUpdatesClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.CancelUpdates;
  SetControls;
end;

procedure TMainCustForm.dsClientsDataChange(Sender: TObject;
  Field: TField);
begin
  inherited;
  SetControls;
end;

procedure TMainCustForm.Exit1Click(Sender: TObject);
begin
  inherited;
  Close;
end;

procedure TMainCustForm.mmiExitClick(Sender: TObject);
begin
  inherited;
  Close;
end;

procedure TMainCustForm.mmiByStateClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.FilterByState;
end;

procedure TMainCustForm.mmiByCityClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.FilterByCity;
end;

procedure TMainCustForm.mmiNoFilterClick(Sender: TObject);
begin
  inherited;
  CustomerDataModule.NoFilter;
end;

end.
 