unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBNAVSTATFRM, Menus, ImgList, ComCtrls, ToolWin, StdCtrls, DBCtrls, Db,
  Mask, dbModeFrm, ActnList, Grids, DBGrids, ExtCtrls;

type

  TMainForm = class(TDBNavStatForm)
    pcMain: TPageControl;
    tsBugInformation: TTabSheet;
    tsActions: TTabSheet;
    lblBugID: TLabel;
    dbeBugID: TDBEdit;
    dsBugs: TDataSource;
    lblDateReported: TLabel;
    lblSummary: TLabel;
    lblDetails: TLabel;
    lblAffectedSource: TLabel;
    lblReportedBy: TLabel;
    lblAssignedTo: TLabel;
    lblStatus: TLabel;
    dbmSummary: TDBMemo;
    dbmDetails: TDBMemo;
    dbmAffectedSource: TDBMemo;
    tsBrowseBugs: TTabSheet;
    rgWhoseBugs: TRadioGroup;
    dbgBugs: TDBGrid;
    dbmSummary2: TDBMemo;
    memAction: TMemo;
    dblcAssignedTo: TDBLookupComboBox;
    dblcStatus: TDBLookupComboBox;
    dbeDateReported: TDBEdit;
    mmiUsers: TMenuItem;
    mmiAddUser: TMenuItem;
    mmiActions: TMenuItem;
    mmiAddActionToBug: TMenuItem;
    dblcReportedBy: TDBLookupComboBox;
    Exit1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure sbFirstClick(Sender: TObject);
    procedure sbPreviousClick(Sender: TObject);
    procedure sbNextClick(Sender: TObject);
    procedure sbLastClick(Sender: TObject);
    procedure sbSearchClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure sbAcceptClick(Sender: TObject);
    procedure sbCancelClick(Sender: TObject);
    procedure sbInsertClick(Sender: TObject);
    procedure sbEditClick(Sender: TObject);
    procedure sbDeleteClick(Sender: TObject);
    procedure sbBrowseClick(Sender: TObject);
    procedure rgWhoseBugsClick(Sender: TObject);
    procedure mmiExitClick(Sender: TObject);
    procedure mmiAddUserClick(Sender: TObject);
    procedure mmiAddActionToBugClick(Sender: TObject);
    procedure dsBugsDataChange(Sender: TObject; Field: TField);
  private
    procedure SetActionStatus;

  protected
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses DDGBugsDM;

{$R *.DFM}

{ TMainForm }

procedure TMainForm.SetActionStatus;
begin
  mmiFirst.Enabled := not DDGBugsDataModule.IsFirstBug;
  mmiLast.Enabled  := not DDGBugsDataModule.IsLastBug;
  mmiNext.Enabled  := not DDGBugsDataModule.IsLastBug;
  mmiPrevious.Enabled := not DDGBugsDataModule.IsFirstBug;
  mmiDelete.Enabled := not DDGBugsDataModule.IsBugsTblEmpty;

  sbFirst.Enabled := mmiFirst.Enabled;
  sbLast.Enabled  := mmiLast.Enabled;
  sbNext.Enabled  := mmiNext.Enabled;
  sbPrev.Enabled  := mmiPrevious.Enabled;
  sbDelete.Enabled := mmiDelete.Enabled;

  // User cannot add users or actions when adding/editing a bug.
  mmiUsers.Enabled := FormMode = fmBrowse;
  mmiActions.Enabled := (FormMode = fmBrowse) and (DDGBugsDataModule.NumBugs <> 0);

  { disable the browsing of bug records when the user is editing or adding
    a new bug. }
  dbgBugs.Enabled     := FormMode = fmBrowse;
  rgWhoseBugs.Enabled := FormMOde = fmBrowse;

end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  inherited;
  SetActionStatus;
end;

procedure TMainForm.sbFirstClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.FirstBug;
  SetActionStatus;
end;

procedure TMainForm.sbPreviousClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.PreviousBug;
  SetActionStatus;
end;

procedure TMainForm.sbNextClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.NextBug;
  SetActionStatus;
end;

procedure TMainForm.sbLastClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.LastBug;
  SetActionStatus;
end;

procedure TMainForm.sbSearchClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.SearchForBug;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  Rslt: word;
begin
  inherited;
  if not (FormMode = fmBrowse) then
  begin
    rslt := MessageDlg('Save changes?', mtConfirmation, mbYesNoCancel, 0);
    case rslt of
      mrYes:
        begin
          DDGBugsDataModule.SaveBug;
          FormMode := fmBrowse;
          CanClose := True;
        end;
      mrNo:
        begin
          DDGBugsDataModule.CancelBug;
          FormMode := fmBrowse;
          CanClose := True;
        end;
      mrCancel:
        CanClose := False;
    end;
  end;
end;

procedure TMainForm.sbAcceptClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.SaveBug;
  SetActionStatus;
end;

procedure TMainForm.sbCancelClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.CancelBug;
  SetActionStatus;
end;

procedure TMainForm.sbInsertClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.InsertBug;
  SetActionStatus;
end;

procedure TMainForm.sbEditClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.EditBug;
  SetActionStatus;
end;

procedure TMainForm.sbDeleteClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.DeleteBug;
  SetActionStatus;
end;

procedure TMainForm.sbBrowseClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.CancelBug;
  SetActionStatus;
end;

procedure TMainForm.rgWhoseBugsClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.FilterOnUser := rgWhoseBugs.ItemIndex = 0; 
end;

procedure TMainForm.mmiExitClick(Sender: TObject);
begin
  inherited;
  Close;
end;

procedure TMainForm.mmiAddUserClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.AddUser;
end;

procedure TMainForm.mmiAddActionToBugClick(Sender: TObject);
begin
  inherited;
  DDGBugsDataModule.AddAction;
  dsBugsDataChange(nil, nil);
end;

procedure TMainForm.dsBugsDataChange(Sender: TObject; Field: TField);
begin
  inherited;
  // A new bug is being displayed so clear the action list and
  // retrieve the actions for the newly displayed bug. 
  memAction.Lines.Clear;
  DDGBugsDataModule.GetActions(memAction.Lines);
end;

end.
 