{
Copyright  1998 by Delphi 4 Developer's Guide - Xavier Pacheco and Steve Teixeira
}
unit Cards;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls;

type
  TCardSuit = (csClub, csDiamond, csHeart, csSpade);
  TCardValue = (cvAce, cvTwo, cvThree, cvFour, cvFive, cvSix, cvSeven,
    cvEight, cvNine, cvTen, cvJack, cvQueen, cvKing);

  TCard = class(TCustomControl)
  private
    FBackBrush: HBrush;
    FBackColor: TColor;
    FFaceUp: Boolean;
    FSuit: TCardSuit;
    FValue: TCardValue;
    procedure SetBackColor(Value: TColor);
    procedure SetFaceUp(Value: Boolean);
    procedure SetSuit(Value: TCardSuit);
    procedure SetValue(Value: TCardValue);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property BackColor: TColor read FBackColor write SetBackColor default clBlue;
    property Color default clWhite;
    property DragCursor;
    property DragMode;
    property FaceUp: Boolean read FFaceUp write SetFaceUp default True;
    property Height default 120;
    property Hint;
    property ParentColor default False;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Suit: TCardSuit read FSuit write SetSuit;
    property Value: TCardValue read FValue write SetValue;
    property Width default 90;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

const
  SValueArray: array[TCardValue] of PChar = (
    'A', '2', '3', '4', '5', '6', '7', '8', '9', '10', 'J', 'Q', 'K');

constructor TCard.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Height := 120;
  Width := 90;
  ParentColor := False;
  Color := clWhite;
  FFaceUp := True;
  SetBackColor(clBlue);
end;

destructor TCard.Destroy;
begin
  if FBackBrush <> 0 then DeleteObject(FBackBrush);
  inherited Destroy;
end;

procedure TCard.SetBackColor(Value: TColor);
begin
  if Value <> FBackColor then
  begin
    FBackColor := Value;
    // delete old HBrush, if there is one
    if FBackBrush <> 0 then DeleteObject(FBackBrush);
    // create an HBrush to use to fill the background.
    FBackBrush := CreateSolidBrush(ColorToRGB(FBackColor));
    Invalidate;
  end;
end;

procedure TCard.SetFaceUp(Value: Boolean);
begin
  if Value <> FFaceUp then
  begin
    FFaceUp := Value;
    Invalidate;
  end;
end;

procedure TCard.SetSuit(Value: TCardSuit);
begin
  if Value <> FSuit then
  begin
    FSuit := Value;
    Invalidate;
  end;
end;

procedure TCard.SetValue(Value: TCardValue);
begin
  if Value <> FValue then
  begin
    FValue := Value;
    Invalidate;
  end;
end;

procedure TCard.Paint;
const
  // ordinal values of "suit" characters in Symbol font:
  SSuits: PChar = #167#168#169#170;
  BorderWidth = 4;
var
  ARect: TRect;
begin
  ARect := ClientRect;
  with ARect do
  begin
    // draw bounding rectangle
    Canvas.Rectangle(Left, Top, Right, Bottom);
    // tighten drawing rect to allow for a border
    Inc(Top, BorderWidth);
    Inc(Left, BorderWidth);
    Dec(Right, BorderWidth);
    Dec(Bottom, BorderWidth);
  end;
  if FFaceUp then
  begin
    // draw card face if it is face-up
    with Canvas do
    begin
      with Font do
      begin
        Name := 'Symbol';  // font containing suit symbols
        Size := 36;
        // set color of font to match that of suit
        if FSuit in [csDiamond, csHeart] then Color := clRed
        else Color := clBlack;
      end;
      Brush.Style := bsClear;
      // draw suit symbol in center of card
      DrawText(Handle, SSuits + Ord(FSuit), 1, ARect, DT_SINGLELINE or
        DT_CENTER or DT_VCENTER);
      // use new typeface and size for value
      Font.Name := 'MS Sans Serif';
      Font.Size := 24;
      // draw value in top,left and bottom,right of card
      DrawText(Handle, SValueArray[FValue], -1, ARect, DT_SINGLELINE or
        DT_TOP or DT_LEFT);
      DrawText(Handle, SValueArray[FValue], -1, ARect, DT_SINGLELINE or
        DT_BOTTOM or DT_RIGHT);
    end;
  end
  else
    // if card is face-down, then draw a big rectangle
    // Using the FillRect() API with an HBrush is slighly more efficient
    // than using the TCanvas methods. 
    Windows.FillRect(Canvas.Handle, ARect, FBackBrush);
end;

procedure Register;
begin
  RegisterComponents('DDG', [TCard]);
end;

end.
