{
Copyright  1998 by Delphi 4 Developer's Guide - Xavier Pacheco and Steve Teixeira
}
unit MemView;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TddgMemView = class(TCustomMemo)
  private
    { Private declarations }
    FMemory: Pointer;
    FMemSize: integer;
    FDisplayBytes: integer;
    procedure ReadMem;
    function ShowChar(B: Byte): Char;
    procedure SetDisplayBytes(Value: integer);
  protected
    { Protected declarations }
    function GetLineFormat: String; virtual;
    function FormatNewLine(CurPos: PByte; var BytesLeft: integer): String; virtual;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    property Memory: Pointer read FMemory default Nil;
    property MemSize: integer read FMemSize default 0;
    procedure SetMemory(Value: Pointer; Size: integer);
  published
    property DisplayBytes: integer read FDisplayBytes write SetDisplayBytes;
    property Align;
    property Alignment;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ScrollBars;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property WordWrap;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

implementation

type
  PByteArray = ^ByteArray;
  ByteArray = array[1..MaxInt] of byte;

const
  SLineFormat = '[%p]: %s[%s]';
  MaxSize = $7FFE;

constructor TddgMemView.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Font.Name := 'Courier New';
  ReadOnly := True;
  FDisplayBytes := 8;
  FMemory := Nil;
  FMemSize := 0;
end;

function TddgMemView.GetLineFormat: String;
begin
  Result := SLineFormat;
end;

function TddgMemView.ShowChar(B: Byte): Char;
begin
  if B in [32..127] then Result := Chr(B)
  else Result := '.';
end;

function TddgMemView.FormatNewLine(CurPos: PByte; var BytesLeft: integer): String;
var
  PB: PByteArray;
  BS, CS: String;
  i: integer;
begin
  PByte(PB) := CurPos;
  BS := '';
  CS := '';
  if BytesLeft > FDisplayBytes then
    BytesLeft := FDisplayBytes;
  for i := 1 to BytesLeft do
    BS := BS + ' ' + IntToHex(PB^[i], 2);
  for i := 1 to BytesLeft do
    CS := CS + ShowChar(PB^[i]);
  for i := BytesLeft to FDisplayBytes do begin
    BS := BS + '   ';
    CS := CS + ' ';
  end;
  Result := Format(GetLineFormat, [CurPos, BS, CS]);
end;

procedure TddgMemView.ReadMem;
var
  CurPos: PByte;
  BytesLeft: integer;
begin
  Lines.Clear;
  CurPos := FMemory;
  repeat
    BytesLeft := FMemSize - (Integer(CurPos) - Integer(FMemory));
    Lines.Add(FormatNewLine(CurPos, BytesLeft));
    inc(CurPos, BytesLeft);
  until Integer(CurPos) - Integer(FMemory) >= FMemSize;
end;

procedure TddgMemView.SetDisplayBytes(Value: integer);
begin
  if FDisplayBytes <> Value then begin
    FDisplayBytes := Value;
    ReadMem;
  end;
end;

procedure TddgMemView.SetMemory(Value: Pointer; Size: integer);
begin
  if (FMemory <> Value) or (FMemSize <> Size) then begin
    FMemory := Value;
    if Size > MaxSize then Size := MaxSize;
    FMemSize := Size;
    ReadMem;
  end;
end;

end.
