{
Copyright  1998 by Delphi 4 Developer's Guide - Xavier Pacheco and Steve Teixeira
}
unit ddgFrmXprts;

interface

uses Windows, ExptIntf, ToolIntf, Classes, Dialogs;

type

  { TChildFormExpert }
  TChildFormExpert = class(TIExpert)
  private
    function GetFormStream(const AName: string): TMemoryStream;
    function GetSourceStream(const AUnitName, AFormName: string): TMemoryStream;
  public
    function GetName: string; override; stdcall;
    function GetAuthor: string; override; stdcall;
    function GetComment: string; override; stdcall;
    function GetPage: string; override; stdcall;
    function GetGlyph: HICON; override; stdcall;
    function GetStyle: TExpertStyle; override; stdcall;
    function GetState: TExpertState; override; stdcall;
    function GetIDString: string; override; stdcall;
    procedure Execute; override; stdcall;
  end;

  { TDBModeFormExpert }
  TDBModeFormExpert = class(TIExpert)
  private
    function GetFormStream(const AName: string): TMemoryStream;
    function GetSourceStream(const AUnitName, AFormName: string): TMemoryStream;
  public
    function GetName: string; override; stdcall;
    function GetAuthor: string; override; stdcall;
    function GetComment: string; override; stdcall;
    function GetPage: string; override; stdcall;
    function GetGlyph: HICON; override; stdcall;
    function GetStyle: TExpertStyle; override; stdcall;
    function GetState: TExpertState; override; stdcall;
    function GetIDString: string; override; stdcall;
    procedure Execute; override; stdcall;
  end;

  { TDBNavStatFormExpert }
  TDBNavStatFormExpert = class(TIExpert)
  private
    function GetFormStream(const AName: string; AAncestor: TComponent): TMemoryStream;
    function GetSourceStream(const AUnitName, AFormName: string): TMemoryStream;
  public
    function GetName: string; override; stdcall;
    function GetAuthor: string; override; stdcall;
    function GetComment: string; override; stdcall;
    function GetPage: string; override; stdcall;
    function GetGlyph: HICON; override; stdcall;
    function GetStyle: TExpertStyle; override; stdcall;
    function GetState: TExpertState; override; stdcall;
    function GetIDString: string; override; stdcall;
    procedure Execute; override; stdcall;
  end;


implementation
uses ChildFrm, DBModeFrm, DBNavStatFrm, EditIntF, IStreams, Proxies, SysUtils,
     TypInfo;

{ TDBModeFormExpert }

function TChildFormExpert.GetGlyph: HICON;
begin
  Result := 0;
end;

function TChildFormExpert.GetIDString: string;
begin
  Result := 'DDG.ChildFormExpert';
end;

function TChildFormExpert.GetName: string;
begin
  Result := 'DDG Child Form Expert';
end;

function TChildFormExpert.GetPage: string;
begin
  Result := 'DDG';
end;

function TChildFormExpert.GetState: TExpertState;
begin
  Result := [esEnabled];
end;

function TChildFormExpert.GetStyle: TExpertStyle;
begin
  Result := esForm;
end;

function TChildFormExpert.GetAuthor: string;
begin
  Result := 'Delphi 4 Developer''s Guide';
end;

function TChildFormExpert.GetComment: string;
begin
  Result := 'Creates a specialized Child form for D4DG''s projects.'
end;


function TChildFormExpert.GetFormStream(const AName: string): TMemoryStream;
var
  F: TChildForm;
begin
  F := TChildForm.CreateNew(nil);
  try
    Proxies.CreateSubClass(F, 'T' + AName, TChildForm);
    F.Name := AName;
    F.BoundsRect := ToolServices.GetFormBounds(btForm);
    F.Caption := F.Name;


    Result := TMemoryStream.Create;
//  Result.WriteComponentRes(AName, F);
    Result.WriteDescendentRes(F.ClassName, F, nil);

    Result.Position := 0;
  finally
    F.Free;
  end;
end;

function TChildFormExpert.GetSourceStream(const AUnitName,
   AFormName: string): TMemoryStream;
var
  Text: string;
  ResInstance: THandle;
begin
  ResInstance := FindResourceHInstance(HInstance);
  Text := PChar(LockResource(LoadResource(ResInstance,
    FindResource(ResInstance, 'CHILDFORM_DEC', RT_RCDATA))));
  Text := Format(Text, [AUnitName, AFormName]);
  Result := TMemoryStream.Create;
  Result.Write(Pointer(Text)^, Length(Text));
  Result.Position := 0;
end;

procedure TChildFormExpert.Execute;
var
  UnitName, FileName, FormName: string;
  SourceStream, FormStream: TIMemoryStream;

  AncUnitName, AncFileName: String;

  cfTypeData: PTypeData;

begin
  if (ToolServices <> nil) and ToolServices.GetNewModuleName(UnitName, FileName) then
  begin
    UnitName := LowerCase(UnitName);
    UnitName[1] := UpCase(UnitName[1]);
    FormName := 'ChildForm' + Copy(UnitName, 5, Length(UnitName));
    FormStream := TIMemoryStream.Create(GetFormStream(FormName), soOwned);
    SourceStream := TIMemoryStream.Create(GetSourceStream(UnitName, FormName),
      soOwned);

    // Add the descendant to the project

    cfTypeData := GetTypeData(TChildForm.ClassInfo);
    AncUnitName := 'F:\X\Projects\D4DG\Code\Comps\'+cfTypeData.UnitName+'.pas';
    ToolServices.CreateModuleEx(AncUnitName, EmptyStr, EmptyStr, EmptyStr,
      nil, nil, [cmExisting, cmAddToProject]);

(*
    ToolServices.CreateModule(FileName, SourceStream, FormStream,
      [cmAddToProject, cmShowSource, cmShowForm, cmUnNamed, cmMarkModified]);
*)
    ToolServices.CreateModuleEx(FileName, FormName, 'ChildForm',
      EmptyStr, SourceStream, FormStream,
      [cmAddToProject, cmShowSource, cmShowForm, cmUnNamed, cmMarkModified]);

  end;
end;


{ TDBModeForm }

function TDBModeFormExpert.GetGlyph: HICON;
begin
  Result := 0;
end;

function TDBModeFormExpert.GetIDString: string;
begin
  Result := 'DDG.DBModeFormExpert';
end;

function TDBModeFormExpert.GetName: string;
begin
  Result := 'DDG Database Mode Form Expert';
end;

function TDBModeFormExpert.GetPage: string;
begin
  Result := 'DDG';
end;

function TDBModeFormExpert.GetState: TExpertState;
begin
  Result := [esEnabled];
end;

function TDBModeFormExpert.GetStyle: TExpertStyle;
begin
  Result := esForm;
end;

function TDBModeFormExpert.GetAuthor: string;
begin
  Result := 'Delphi 4 Developer''s Guide';
end;

function TDBModeFormExpert.GetComment: string;
begin
  Result := 'Creates a specialized database mode form for D4DG''s projects.'
end;


function TDBModeFormExpert.GetFormStream(const AName: string): TMemoryStream;
var
  F: TDBModeForm;
begin
  F := TDBModeForm.CreateNew(nil);
  try
    Proxies.CreateSubClass(F, 'T' + AName, TDBModeForm);
    F.Name := AName;
    F.BoundsRect := ToolServices.GetFormBounds(btForm);
    F.Caption := F.Name;

    Result := TMemoryStream.Create;

//    Result.WriteComponentRes(AName, F);
    Result.WriteDescendentRes(F.ClassName, F, nil);
    Result.Position := 0;
  finally
    F.Free;
  end;
end;

function TDBModeFormExpert.GetSourceStream(const AUnitName,
   AFormName: string): TMemoryStream;
var
  Text: string;
  ResInstance: THandle;
begin
  ResInstance := FindResourceHInstance(HInstance);
  Text := PChar(LockResource(LoadResource(ResInstance,
    FindResource(ResInstance, 'DBMODEFORM_DEC', RT_RCDATA))));
  Text := Format(Text, [AUnitName, AFormName]);
  Result := TMemoryStream.Create;
  Result.Write(Pointer(Text)^, Length(Text));
  Result.Position := 0;
end;

procedure TDBModeFormExpert.Execute;
var
  UnitName, FileName, FormName: string;
  SourceStream, FormStream: TIMemoryStream;

  cfTypeData: PTypeData;
  AncUnitName: String;

begin
  if (ToolServices <> nil) and ToolServices.GetNewModuleName(UnitName, FileName) then
  begin
    UnitName := LowerCase(UnitName);
    UnitName[1] := UpCase(UnitName[1]);
    FormName := 'DBModeForm' + Copy(UnitName, 5, Length(UnitName));
    FormStream := TIMemoryStream.Create(GetFormStream(FormName), soOwned);
    SourceStream := TIMemoryStream.Create(GetSourceStream(UnitName, FormName),
      soOwned);

    cfTypeData := GetTypeData(TChildForm.ClassInfo);
    AncUnitName := 'F:\X\Projects\D4DG\Code\Comps\'+cfTypeData.UnitName+'.pas';
    ToolServices.CreateModuleEx(AncUnitName, EmptyStr, EmptyStr, EmptyStr,
      nil, nil, [cmExisting, cmAddToProject]);

    cfTypeData := GetTypeData(TDBModeForm.ClassInfo);
    AncUnitName := 'F:\X\Projects\D4DG\Code\Comps\'+cfTypeData.UnitName+'.pas';
    ToolServices.CreateModuleEx(AncUnitName, EmptyStr, EmptyStr, EmptyStr,
      nil, nil, [cmExisting, cmAddToProject]);

    ToolServices.CreateModuleEx(FileName, FormName, 'DBModeForm',
      EmptyStr, SourceStream, FormStream,
      [cmAddToProject, cmShowSource, cmShowForm, cmUnNamed, cmMarkModified]);

  end;
end;

{ TDBNavStatFormExpert }

function TDBNavStatFormExpert.GetGlyph: HICON;
begin
  Result := 0;
end;

function TDBNavStatFormExpert.GetIDString: string;
begin
  Result := 'DDG.DBNavStatFormExpert';
end;

function TDBNavStatFormExpert.GetName: string;
begin
  Result := 'DDG Database NavBar/StatBar Form Expert';
end;

function TDBNavStatFormExpert.GetPage: string;
begin
  Result := 'DDG';
end;

function TDBNavStatFormExpert.GetState: TExpertState;
begin
  Result := [esEnabled];
end;

function TDBNavStatFormExpert.GetStyle: TExpertStyle;
begin
  Result := esForm;
end;

function TDBNavStatFormExpert.GetAuthor: string;
begin
  Result := 'Delphi 4 Developer''s Guide';
end;

function TDBNavStatFormExpert.GetComment: string;
begin
  Result := 'Creates a specialized db navbar/statbar form for D4DG''s projects.'
end;


function TDBNavStatFormExpert.GetFormStream(const AName: string;
  AAncestor: TComponent): TMemoryStream;
var
  F: TDBNavStatForm;
begin
  F := TDBNavStatForm.CreateNew(nil);
  try
    Proxies.CreateSubClass(F, 'T' + AName, TDBNavStatForm);
    F.Name := AName;
    F.BoundsRect := ToolServices.GetFormBounds(btForm);
    F.Caption := F.Name;



    Result := TMemoryStream.Create;

    Result.WriteDescendentRes(F.ClassName, F, AAncestor);
    Result.Position := 0;

    Result.SaveToFile('d:\work\blah.dfm');

  finally
    F.Free;
  end;
end;

{
function TDBNavStatFormExpert.GetFormStream(const AName: string): TMemoryStream;
var
  Text: string;
  ResInstance: THandle;
  ObjTextStream: TMemoryStream;
begin
  ResInstance := FindResourceHInstance(HInstance);
  Text := PChar(LockResource(LoadResource(ResInstance,
    FindResource(ResInstance, 'DBNAVSTATFORMBIN', RT_RCDATA))));
  Text := Format(Text, [AName]);

  Result := TMemoryStream.Create;
  ObjTextStream := TMemoryStream.Create;
  try
    ObjTextStream.Write(Pointer(Text)^, Length(Text));
    ObjTextStream.Position := 0;
    ObjectTextToResource(ObjTextStream, Result);
    Result.Position := 0;
  finally
    ObjTextStream.Free;
  end;
end;
}

function TDBNavStatFormExpert.GetSourceStream(const AUnitName,
   AFormName: string): TMemoryStream;
var
  Text: string;
  ResInstance: THandle;
begin
  ResInstance := FindResourceHInstance(HInstance);
  Text := PChar(LockResource(LoadResource(ResInstance,
    FindResource(ResInstance, 'DBNAVSTATFORM_DEC', RT_RCDATA))));
  Text := Format(Text, [AUnitName, AFormName]);
  Result := TMemoryStream.Create;
  Result.Write(Pointer(Text)^, Length(Text));
  Result.Position := 0;
end;

procedure TDBNavStatFormExpert.Execute;
var
  UnitName, FileName, FormName: string;
  SourceStream, FormStream: TIMemoryStream;

  cfTypeData: PTypeData;
  AncUnitName: String;

  ModIntF: TIModuleInterface;
  FormIntF: TIFormInterface;
  CompIntF: TIComponentInterface;
  FormComponent: TComponent;
  i: integer;
begin
  if (ToolServices <> nil) and ToolServices.GetNewModuleName(UnitName, FileName) then
  begin
    UnitName := LowerCase(UnitName);
    UnitName[1] := UpCase(UnitName[1]);

    cfTypeData := GetTypeData(TChildForm.ClassInfo);
    AncUnitName := 'F:\X\Projects\D4DG\Code\Comps\'+cfTypeData.UnitName+'.pas';
    ToolServices.CreateModuleEx(AncUnitName, EmptyStr, EmptyStr, EmptyStr,
      nil, nil, [cmExisting, cmAddToProject]);

    cfTypeData := GetTypeData(TDBModeForm.ClassInfo);
    AncUnitName := 'F:\X\Projects\D4DG\Code\Comps\'+cfTypeData.UnitName+'.pas';
    ToolServices.CreateModuleEx(AncUnitName, EmptyStr, 'ChildForm', EmptyStr,
      nil, nil, [cmExisting, cmAddToProject]);

    cfTypeData := GetTypeData(TDBNavStatForm.ClassInfo);
    AncUnitName := 'F:\X\Projects\D4DG\Code\Comps\'+cfTypeData.UnitName+'.pas';
    ModIntF := ToolServices.CreateModuleEx(AncUnitName, EmptyStr, 'DBModeForm', EmptyStr,
      nil, nil, [cmExisting, cmAddToProject]);

    FormIntF := ModIntF.GetFormInterface;
    CompIntF := FormIntF.GetFormComponent;
    FormComponent := CompIntF.GetComponentHandle;

    FormName := 'DBNavStatForm' + Copy(UnitName, 5, Length(UnitName));

    FormStream := TIMemoryStream.Create(GetFormStream(FormName,
      FormComponent), soOwned);

    SourceStream := TIMemoryStream.Create(GetSourceStream(UnitName, FormName),
      soOwned);

    ToolServices.CreateModuleEx(FileName, FormName, 'DBNavStatForm',
      EmptyStr, nil, nil, {SourceStream, FormStream,}
       [cmNewForm, cmAddToProject, cmShowSource, cmShowForm, cmMarkModified]);
  end;
end;

end.
