unit DDGBugsDM;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBTables, HTTPApp, DBWeb;

type

  EUnableToObtainID = class(Exception);

  TDDGBugsDataModule = class(TDataModule)
    dbDDGBugs: TDatabase;
    tblBugs: TTable;
    tblUsers: TTable;
    tblStatus: TTable;
    tblActions: TTable;
    tblBugsBugID: TIntegerField;
    tblBugsWhenReported: TDateField;
    tblBugsSummaryDescription: TStringField;
    tblBugsDetails: TStringField;
    tblBugsAffectedSource: TStringField;
    tblBugsUserID: TIntegerField;
    tblBugsStatusID: TIntegerField;
    dsUsers: TDataSource;
    dsStatus: TDataSource;
    tblIDs: TTable;
    tblBugsUserNameLookup: TStringField;
    tblBugsAssignedToLookup: TStringField;
    tblUsersUserID: TIntegerField;
    tblUsersUserName: TStringField;
    tblUsersUserFirstName: TStringField;
    tblUsersUserLastName: TStringField;
    tblBugsAssignedToUserID: TIntegerField;
    dsBugs: TDataSource;
    wbdpBugs: TWebDispatcher;
    dstpBugs: TDataSetTableProducer;
    pprdBugs: TPageProducer;
    ssnBugs: TSession;
    tblBugsStatusTitle: TStringField;
    procedure DDGBugsDataModuleCreate(Sender: TObject);
    procedure tblBugsBeforePost(DataSet: TDataSet);
    procedure tblBugsFilterRecord(DataSet: TDataSet; var Accept: Boolean);
    procedure tblUsersBeforePost(DataSet: TDataSet);
    procedure tblBugsAfterInsert(DataSet: TDataSet);
    procedure wbdpBugswaIntroAction(Sender: TObject; Request: TWebRequest;
      Response: TWebResponse; var Handled: Boolean);
    procedure wbdpBugswaUserNameAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure wbdpBugswaVerifyUserNameAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure wbdpBugswaBrowseBugsAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure wbdpBugswaBrowseAllBugsAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure pprdBugsHTMLTag(Sender: TObject; Tag: TTag;
      const TagString: String; TagParams: TStrings;
      var ReplaceText: String);
    procedure wbdpBugswaBrowseYourBugsAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure dstpBugsFormatCell(Sender: TObject; CellRow,
      CellColumn: Integer; var BgColor: THTMLBgColor;
      var Align: THTMLAlign; var VAlign: THTMLVAlign; var CustomAttrs,
      CellData: String);
    procedure wbdpBugswaRetrieveBugAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure wbdpBugswaGetBugInfoAction(Sender: TObject; Request: TWebRequest;
      Response: TWebResponse; var Handled: Boolean);
    procedure wbdpBugswaAddBugAction(Sender: TObject; Request: TWebRequest;
      Response: TWebResponse; var Handled: Boolean);
  private
    FLoginUserID: Integer;
    FLoginUserName: String;

    function GetFilterOnUser: Boolean;
    procedure SetFilterOnUser(const Value: Boolean);
    function GetNumBugs: Integer;
  protected
    procedure PostAction(Sender: TObject; Action: TStrings);
  public
  
    // Bugs Methods
    procedure FirstBug;
    procedure LastBug;
    procedure NextBug;
    procedure PreviousBug;
    function IsLastBug: Boolean;
    function IsFirstBug: Boolean;
    function IsBugsTblEmpty: Boolean;
    procedure InsertBug;
    procedure DeleteBug;
    procedure EditBug;
    procedure SaveBug;
    procedure CancelBug;
    procedure SearchForBug;

    // User Functions
   {$IFNDEF DDGWEBBUGS}
    procedure AddUser;
   {$ENDIF}

    procedure PostUser(Sender: TObject);
    function GetUserFLName(AUserID: Integer): String;

    // Action Methods

    {$IFNDEF DDGWEBBUGS}
    procedure AddAction;
    {$ENDIF}

    procedure GetActions(AActions: TStrings);

    // Id Generation
    function GetDataSetID(const AFieldName: String): Integer;
    function GetNewBugID: Integer;
    function GetNewUserID: Integer;
    function GetNewActionID: Integer;

    // Login Function
    function Login: Boolean;

    // Exposed properties

    property LoginUserID: Integer read FLoginUserID;
    property FilterOnUser: Boolean read GetFilterOnUser write SetFilterOnUser;
    property NumBugs: Integer read GetNumBugs;
  end;

var
  DDGBugsDataModule: TDDGBugsDataModule;

implementation

{$IFNDEF DDGWEBBUGS}
uses UserFrm, ActionFrm;
{$ENDIF}

{$R *.DFM}

// Helper functions.

function IsInteger(IntVal: string): Boolean;
var
  v, code: Integer;
begin
  val(IntVal, v, code);
  Result := code = 0;
end;

procedure MemoFromStrings(AMemoField: TMemoField; AStrings: TStrings);
var
  Stream: TMemoryStream;
begin
  Stream := TMemoryStream.Create;
  try
    AStrings.SaveToStream(Stream);
    Stream.Seek(0, soFromBeginning);
    AMemoField.LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure StringsFromMemo(AStrings: TStrings; AMemoField: TMemoField);
var
  Stream: TMemoryStream;
begin
  Stream := TMemoryStream.Create;
  try
    AMemoField.SaveToStream(Stream);
    Stream.Seek(0, soFromBeginning);
    AStrings.LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

// Internal methods

function TDDGBugsDataModule.GetFilterOnUser: Boolean;
begin
  Result := tblBugs.Filtered;
end;

procedure TDDGBugsDataModule.SetFilterOnUser(const Value: Boolean);
begin
 tblBugs.Filtered := Value;
end;

function TDDGBugsDataModule.GetNumBugs: Integer;
begin
  Result := tblBugs.RecordCount;
end;

// Identifier methods

function TDDGBugsDataModule.GetDataSetID(const AFieldName: String): Integer;
const
  MaxAttempts = 50;
var
  Attempts: Integer;
  NextID: Integer;
begin
  tblIDs.Active := True;
  // Try fifty times if until this works or raise an exception
  Attempts := 0;
  while Attempts <= MaxAttempts do
  begin
    try
      Inc(Attempts);
      // If another user has the table in edit mode, an error occurs here.
      tblIDs.Edit;
      // If we reach the Break statement, we are successful. Break out of loop.
      Break;
    except
      on EDBEngineError do
      begin
        // Do some delay
        Continue;
      end;
    end;
  end;

  if tblIDs.State = dsEdit then
  begin
    // Increment the value obtained from the table and restore the new value
    // to the table for the next record. 
    NextID := tblIDs.FieldByName(AFieldName).AsInteger;
    tblIDs.FieldByName(AFieldName).AsInteger := NextID + 1;
    TblIDs.Post;
    Result := NextID;
  end
  else
    Raise EUnableToObtainID.Create('Cannot create unique ID');
end;

function TDDGBugsDataModule.GetNewActionID: Integer;
begin
  Result := GetDataSetID('ActionsID');
end;

function TDDGBugsDataModule.GetNewBugID: Integer;
begin
  Result := GetDataSetID('BugsID');
end;

function TDDGBugsDataModule.GetNewUserID: Integer;
begin
  Result := GetDataSetID('UsersID');
end;

// Initialization/Login methods.

procedure TDDGBugsDataModule.DDGBugsDataModuleCreate(Sender: TObject);
begin
  { These tables are opened in the proper order so the master-detail
    relationship does not fail.}

  ssnBugs.Active := True;
  dbDDGBugs.Connected := True;
  tblUsers.Active  := True;
  tblStatus.Active := True;
  tblBugs.Active   := True;
  tblActions.Active := True;

end;


function TDDGBugsDataModule.Login: Boolean;
var
  UserName: String;
begin
  InputQuery('Login', 'Enter User Name: ', UserName);
  Result := tblUsers.Locate('UserName', UserName, []);
  if Result then
  begin
    FLoginUserID   := tblUsers.FieldByName('UserID').AsInteger;
    FLoginUserName := tblUsers.FieldByName('UserName').AsString;
  end;
end;

// Bug methods.

procedure TDDGBugsDataModule.FirstBug;
begin
  tblBugs.First;
end;

procedure TDDGBugsDataModule.LastBug;
begin
  tblBugs.Last;
end;

procedure TDDGBugsDataModule.NextBug;
begin
  tblBugs.Next;
end;

procedure TDDGBugsDataModule.PreviousBug;
begin
  tblBugs.Prior;
end;

function TDDGBugsDataModule.IsLastBug: Boolean;
begin
  Result := tblBugs.Eof;
end;

function TDDGBugsDataModule.IsFirstBug: Boolean;
begin
  Result := tblBugs.Bof;
end;

function TDDGBugsDataModule.IsBugsTblEmpty: Boolean;
begin
  // If RecordCount is zero, there are not bugs in the table.
  Result := tblBugs.RecordCount = 0;
end;

procedure TDDGBugsDataModule.InsertBug;
begin
  tblBugs.Insert;
end;

procedure TDDGBugsDataModule.DeleteBug;
var
  Qry: TQuery;
  BugID: Integer;
begin
  if MessageDlg('Delete Action?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
  begin
    BugID := tblBugs.FieldByName('BugID').AsInteger;
    // Use a dynamically created TQuery component to perform these operations.
    Qry := TQuery.Create(self);
    try
      dbDDGBugs.StartTransaction;
      try
        // First delete any action belonging to this bug.
        Qry.DatabaseName := dbDDGBugs.DataBaseName;
        Qry.SQL.Add(Format('DELETE FROM ACTIONS WHERE BugID = %d', [BugID]));
        Qry.ExecSQL;

        // Now delete bug from the bugs table.
        Qry.SQL.Clear;
        Qry.SQL.Add(Format('DELETE FROM BUGS WHERE BugID = %d', [BugID]));
        Qry.ExecSQL;

        tblBugs.Refresh;
        tblActions.Refresh;

        dbDDGBugs.Commit;
      except
        dbDDGBugs.Rollback;
        raise;
      end;
    finally
      Qry.Free;
    end;
  end;
end;

procedure TDDGBugsDataModule.EditBug;
begin
  tblBugs.Edit;
end;

procedure TDDGBugsDataModule.SaveBug;
begin
  tblBugs.Post;
end;

procedure TDDGBugsDataModule.CancelBug;
begin
  tblBugs.Cancel;
end;

procedure TDDGBugsDataModule.SearchForBug;
var
  BugStr: String;
begin
  InputQuery('Search for bug', 'Enter bug ID: ', BugStr);
  if IsInteger(BugStr) then
    if not tblBugs.Locate('BugID', StrToInt(BugStr), []) then
      MessageDlg('Bug not found.', mtInformation, [mbOK], 0);
end;

// User methods.

{$IFNDEF DDGWEBBUGS}
procedure TDDGBugsDataModule.AddUser;
begin
  tblUsers.Insert;
  try
    if NewUserForm(PostUser) = mrCancel then
      tblUsers.Cancel;
  except
    // An error occurred. Put the table to browse mode and reraise the exception
    tblUsers.Cancel;
    raise;
  end;
end;
{$ENDIF}

procedure TDDGBugsDataModule.PostUser(Sender: TObject);
begin
  if tblUsers.State = dsInsert then
    tblUsers.FieldByName('UserID').AsInteger := GetNewUserID;
  tblUsers.Post;
end;

function TDDGBugsDataModule.GetUserFLName(AUserID: Integer): String;
begin
  // Returns the first and last name concatonated.
  if tblUsers.Locate('UserID', AUserID, []) then
    Result := Format('%s %s', [tblUsers.FieldByName('UserFirstName').AsString,
      tblUsers.FieldByName('UserLastName').AsString])
  else
    Result := EmptyStr;
end;

{$IFNDEF DDGWEBBUGS}
procedure TDDGBugsDataModule.AddAction;
begin
  NewActionForm(PostAction);
end;
{$ENDIF}

procedure TDDGBugsDataModule.GetActions(AActions: TStrings);

const
{$IFDEF DDGWEBBUGS}
  cPrefix = '<BR>';
{$ELSE}
  cPrefix = '';
{$ENDIF}


var
  Action: TStringList;
  ActionUserId: Integer;


begin
  Action := TStringList.Create;
  try
    with tblActions do
    begin
      tblActions.First;
      while not Eof do
      begin
        Action.Clear;
        ActionUserID := FieldByName('UserID').AsInteger;
        StringsFromMemo(Action, TMemoField(FieldByName('ActionDetail')));
        AActions.Add(Format('%sAction Added on: %s', [cPrefix, FormatDateTime('mmm dd, yyyy',
          FieldByName('ActionDate').AsDateTime)]));
        AActions.Add(Format('%sAction Added by: %s', [cPrefix, GetUserFLName(ActionUserID)]));
        AActions.Add(cPrefix+EmptyStr);
        AActions.AddStrings(Action);
        AActions.Add(cPrefix+'==============================');
        AActions.Add(cPrefix+EmptyStr);
        tblActions.Next;
      end; // while
    end; // with
  finally
    Action.Free;
  end;
end;

procedure TDDGBugsDataModule.PostAction(Sender: TObject; Action: TStrings);
var
  BugID: Integer;
begin
  tblActions.Insert;
  try
    BugID := tblBugs.FieldByName('BugID').AsInteger;
    tblActions.FieldByName('ActionID').AsInteger      := GetNewActionID;
    tblActions.FieldByName('BugID').AsInteger         := BugID;
    tblActions.FieldByName('UserID').AsInteger        := LoginUserID;
    tblActions.FieldByName('ActionDate').AsDateTime   := Date;
    MemoFromStrings(TMemoField(tblActions.FieldByName('ActionDetail')), Action);
    tblActions.Post;
  except
    tblActions.Cancel;
    raise;
  end;
end;

// Event Handlers

procedure TDDGBugsDataModule.tblBugsBeforePost(DataSet: TDataSet);
begin
  if tblBugs.State = dsInsert then
    tblBugs.FieldByName('BugID').AsInteger := GetNewBugID;
end;

procedure TDDGBugsDataModule.tblBugsFilterRecord(DataSet: TDataSet;
  var Accept: Boolean);
begin
  Accept := tblBugs.FieldByName('UserID').AsInteger = FLoginUserID;
end;

procedure TDDGBugsDataModule.tblUsersBeforePost(DataSet: TDataSet);
begin
  if tblUsers.State = dsInsert then
    tblUsers.FieldByName('UserID').AsInteger := GetNewUserID;
end;

procedure TDDGBugsDataModule.tblBugsAfterInsert(DataSet: TDataSet);
begin
  tblBugs.FieldByName('UserID').AsInteger := FLoginUserID;
  tblBugs.FieldByName('UserNameLookup').AsString := FLoginUserName;
end;

// Web Methods

procedure AddHeader(AWebPage: TStringList);
// Adds a standard header to each web page.
begin
  with AWebPage do
  begin
    Add('<HTML>');
    Add('<HEAD>');
    Add('<BODY BACKGROUND="/samples/images/backgrnd.gif">');
    Add('<TITLE>Delphi 4 Developer''s Guide Bug Demo</Title>');
    Add('<CENTER>');
    Add('<P>');
    Add('<FONT SIZE=6>Delphi 4 Developer''s Guide Bug Demo</font>');
    Add('</CENTER>');
    Add('</HEAD>');
  end;
end;

procedure AddFooter(AWebPage: TStringList);
// Adds the standard footer information to each web page. 
begin
  with AWebPage do
  begin
    Add('<BR><BR>Copyright (c) 1998, Delphi 4 Developer''s Guide.');
    Add('</BODY>');
    Add('</HTML>');
  end;
end;

procedure TDDGBugsDataModule.wbdpBugswaIntroAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
// Introductory page for the web demo. 
var
  WebPage: TStringList;
begin
  WebPage := TStringList.Create;
  try
    AddHeader(WebPage);
    with WebPage do
    begin
      Add('<BODY>');
      Add('<H1>Introduction</H1>');
      Add('<P>Welcome to the Delphi 4 Developer''s Guide Bug Demonstration.');
      Add('<BR>This demo, illustrates how to web enable an existing application.');
      Add('<BR>To test the demo, just click on the logon link and follow the pages');
      Add('<BR>to add bugs, or just to browse existing bugs.');
      Add('</P>');
      Add('<A href="../DDGWebBugs.dll/UserName">Login to DDG Bug Demo</A>');
      AddFooter(WebPage);
      Response.Content :=  WebPage.Text;
      Handled := True;
    end;
  finally
    WebPage.Free;
  end;
end;

procedure TDDGBugsDataModule.wbdpBugswaUserNameAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
// This page prompts the user for the user name. 
var
  WebPage: TStringList;
begin
  WebPage := TStringList.Create;
  try
    AddHeader(WebPage);
    with WebPage do
    begin
      Add('<BODY>');
      Add('<H1>Enter your user name</H1>');
      Add('<FORM action="../DDGWebBugs.dll/VerifyUserName" method="GET">');
      Add('<p>UserName: <INPUT type="text" name="UserName" maxlength="30" size="50"></P>');
      Add('<p><INPUT type="SUBMIT"><INPUT type="RESET"></p>');
      Add('</FORM>');
      AddFooter(WebPage);
      Response.Content :=  WebPage.Text;
      Handled := True;
    end;
  finally
    WebPage.Free;
  end;
end;

procedure TDDGBugsDataModule.wbdpBugswaVerifyUserNameAction(
  Sender: TObject; Request: TWebRequest; Response: TWebResponse;
  var Handled: Boolean);
{ This page takes the name entered by the user. The information is saved
  and passed back to the client as a cookie. Additional information is also
  passed back as a cookie which will be used later for adding bugs from
  the web. }

var
  WebPage: TStringList;
  CookieList: TStringList;
  UserName: String;
  UserFName,
  UserLName: String;
  UserID: Integer;
  ValidLogin: Boolean;

procedure BuildValidLoginPage;
begin
  AddHeader(WebPage);
  with WebPage do
  begin
    Add('<BODY>');
    Add(Format('<H1>User name, %s verified. User ID is: %d</H1>',
      [Request.QueryFields.Values['UserName'], UserID]));
    Add('<BR><BR><A href="../DDGWebBugs.dll/BrowseBugs">Browse Bug List</A>');
    Add('<BR><A href="../DDGWebBugs.dll/GetBugInfo">Add a New Bug</A>');
    AddFooter(WebPage);
  end;
end;

procedure BuildInValidLoginPage;
begin
  AddHeader(WebPage);
  with WebPage do
  begin
    Add('<BODY>');
    Add(Format('<H1>User name, %s is not a valid user.</H1>',
      [Request.QueryFields.Values['UserName']]));
    AddFooter(WebPage);
  end;
end;

begin

  UserName := Request.QueryFields.Values['UserName'];

  // The login will be valid if the user name exists in the Users.db.
  ValidLogin := tblUsers.Locate('UserName', UserName, []);

  WebPage := TStringList.Create;
  try

    if ValidLogin then
    begin

      // Retrieve the UserID and the user's first and last name
      UserID := tblUsers.FieldByName('UserID').AsInteger;
      UserFName := tblUsers.FieldByName('UserFirstName').AsString;
      UserLName := tblUsers.FieldByName('UserLastName').AsString;

      CookieList := TSTringList.Create;
      try

        // Store the user's infomration as cookies.
        CookieList.Add('UserID='+IntToStr(UserID));
        CookieList.Add('UserName='+UserName);
        CookieList.Add('UserFirstName='+UserFName);
        CookieList.Add('UserLastName='+UserLName);

        Response.SetCookieField(CookieList, '', '', Now + 1, False);
      finally
        CookieList.Free;
      end;
      BuildValidLoginPage;
    end
    else begin
      UserID := -1;
      BuildInvalidLoginPage;
    end;

    Response.Content :=  WebPage.Text;
    Handled := True;

  finally
    WebPage.Free;
  end;

end;

procedure TDDGBugsDataModule.wbdpBugswaBrowseBugsAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
{ This page gives the user the option of browing all bugs or just bugs
  entered by him/her. }
var
  WebPage: TStringList;
begin
  WebPage := TStringList.Create;
  try
    AddHeader(WebPage);
    with WebPage do
    begin
      Add('<BODY>');
      Add('<H1>Browse Option</H1>');
      Add('<BR><BR><A href="../DDGWebBugs.dll/BrowseAllBugs">Browse All Bugs</A>');
      Add('<BR><A href="../DDGWebBugs.dll/BrowseYourBugs">Browse Your Bugs</A>');
      AddFooter(WebPage);
      Response.Content :=  WebPage.Text;
      Handled := True;
    end;
  finally
    WebPage.Free;
  end;

end;

procedure TDDGBugsDataModule.wbdpBugswaBrowseAllBugsAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
{ This page prepares the TPageProducer component for browing all bugs.
  The standard header and footer is applied to this page, but a tag is
  used to add the table to the page.   }
var
  WebPage: TStringList;
begin
  WebPage := TStringList.Create;
  try
    AddHeader(WebPage);
    WebPage.Add('<BODY>');
    WebPage.Add('<H1>Browsing all Bugs</H1>');
    WebPage.Add('<#TABLE>');
    AddFooter(WebPage);

    pprdBugs.HTMLDoc.Clear;
    pprdBugs.HTMLDoc.AddStrings(WebPage);

    { As a result of the line below, the OnHTMLTag event handle for
      pprdBugs will be invoked. }
    Response.Content := pprdBugs.Content;

    Handled := True;
  finally
    WebPage.Free;
  end;
end;

procedure TDDGBugsDataModule.pprdBugsHTMLTag(Sender: TObject; Tag: TTag;
  const TagString: String; TagParams: TStrings; var ReplaceText: String);
begin
  if Tag = tgTable then begin
    with dstpBugs do
    begin
      DataSet.Open;
      ReplaceText := dstpBugs.Content;
      DataSet.Close;
    end;
  end;
end;

procedure TDDGBugsDataModule.wbdpBugswaBrowseYourBugsAction(
  Sender: TObject; Request: TWebRequest; Response: TWebResponse;
  var Handled: Boolean);
{ This page prepares the TPageProducer component for browing bugs which
  belong to the user. The standard header and footer is applied to this page,
  but a tag is used to add the table to the page.   }
var
  WebPage: TStringList;
  UserID: Integer;
  UserFName,
  UserLName: String;
begin
  WebPage := TStringList.Create;
  try
    AddHeader(WebPage);
    WebPage.Add('<BODY>');

    // Retrieve the user ID which is stored in the cookie.
    UserID    := StrToInt(Request.CookieFields.Values['UserID']);
    UserFName := Request.CookieFields.Values['UserFirstName'];
    UserLName := Request.CookieFields.Values['UserLastName'];

    WebPage.Add(Format('<H1>Browsing Bugs Entered by %s %s</H1>',
      [UserFName, UserLName]));
    WebPage.Add('<#TABLE>');
    pprdBugs.HTMLDoc.Clear;
    pprdBugs.HTMLDoc.AddStrings(WebPage);

    AddFooter(WebPage);

    // Make sure the table is now filtered by the UserID
    FLoginUserID := UserID;
    FilterOnUser := True;

    Response.Content := pprdBugs.Content;

    Handled := True;
  finally
    WebPage.Free;
  end;

end;

procedure TDDGBugsDataModule.dstpBugsFormatCell(Sender: TObject; CellRow,
  CellColumn: Integer; var BgColor: THTMLBgColor; var Align: THTMLAlign;
  var VAlign: THTMLVAlign; var CustomAttrs, CellData: String);
{ Convert the BugID cell of the table to a link which invokes the page to
  display the bug detail. }
begin
  if (CellColumn = 0) and not (CellRow = 0) then
    CellData := Format('<A href="../DDGWebBugs.dll/RetrieveBug?BugID=%s">%s</A>',
      [CellData, CellData]);
end;

procedure TDDGBugsDataModule.wbdpBugswaRetrieveBugAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
{ View the bug detail information. }
var
  BugID: Integer;
  WebPage: TStringList;

procedure GetBug;
begin
  if tblBugs.Locate('BugID', BugID, []) then
    with tblBugs do
    begin
      WebPage.Add(Format('Bug ID:        %d', [BugID]));
      WebPage.Add(Format('<BR>Reported By:   %s', [FieldByName('UserNameLookup').AsString]));
      WebPage.Add(FormatDateTime('"<BR>Reported On:"   mmm dd, yyyy',
        FieldByName('WhenReported').AsDateTime));
      WebPage.Add(Format('<BR>Assigned To:   %s', [FieldByName('AssignedToLookup').AsString]));
      WebPage.Add(Format('<BR>Status:        %s', [FieldByName('StatusTitle').AsString]));
      WebPage.Add(Format('<BR>Summary:       %s', [FieldByName('SummaryDescription').AsString]));
      WebPage.Add(Format('<BR>Details:       %s', [FieldByName('Details').AsString]));
      WebPage.Add('<BR>');
      WebPage.Add('<BR>');

      GetActions(WebPage);
    end;
end;

begin
  BugID := StrToInt(Request.QueryFields.Values['BugID']);

  WebPage := TStringList.Create;
  try
    AddHeader(WebPage);
    with WebPage do
    begin
      Add('<BODY>');
      Add('<H1>Bug Detail</H1>');
      GetBug;
      AddFooter(WebPage);
      Response.Content :=  WebPage.Text;
      Handled := True;
    end;
  finally
    WebPage.Free;
  end;

end;

procedure TDDGBugsDataModule.wbdpBugswaGetBugInfoAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
{ Prepares the page to retrieve new bug information from the user. }
var
  WebPage: TStringList;

procedure AddAssignToNames;
{ Adds a drop down list to the HTML Page of Assign to users }
begin

  WebPage.Add('<BR>Assign To:');
  WebPage.Add('<BR><SELECT name="AssignTo"><BR>');

  with tblUsers do
  begin
    First;
    while not Eof do
    begin
      WebPage.Add(Format('<OPTION>%s %s - %s', [FieldByName('UserFirstName').AsString,
        FieldByName('UserLastName').AsString, FieldByName('UserName').AsString]));
      tblUsers.Next;
    end;
    WebPage.Add('</SELECT>');
  end;
end;

procedure AddStatusTitles;
{ Adds a drop down list to the HTML Page of bug status items }
begin
  WebPage.Add('<BR>Status:');
  WebPage.Add('<BR><SELECT name="Status"><BR>');

  with tblStatus do
  begin
    First;
    while not Eof do
    begin
      WebPage.Add(Format('<OPTION>%s', [FieldByName('StatusTitle').AsString]));
      tblStatus.Next;
    end;
    WebPage.Add('</SELECT>');
  end;
end;

begin
  WebPage := TStringList.Create;
  try
    AddHeader(WebPage);
    with WebPage do
    begin
      Add('<BODY>');
      Add('<H1>Add New Bug</H1>');
      Add('<FORM action="../DDGWebBugs.dll/AddBug" method="GET">');
      Add('<BR>Summary Description:<BR><INPUT type="text" name="Summary" maxlength="100" size="50">');
      Add('<BR>Details:<BR><TEXTAREA name="Details" rows=5 cols=50> </TEXTAREA>');

      AddAssignToNames;
      AddStatusTitles;

      Add('<p><INPUT type="SUBMIT"><INPUT type="RESET"></p>');
      Add('</FORM>');
      AddFooter(WebPage);
      Response.Content :=  WebPage.Text;
      Handled := True;
    end;
  finally
    WebPage.Free;
  end;
end;

procedure TDDGBugsDataModule.wbdpBugswaAddBugAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
{ Adds the Bug to the database. Uses the cookies returned by the client
  to diaplsy information about the user. }
var
  SummaryStr,
  DetailsStr,
  AssignToStr,
  StatusStr: String;
  WebPage: TStringList;
  UserID: Integer;
  UserName: String;
  UserFName,
  UserLName: String;
  AssignedToUserName: String;
  PostSucceeded: boolean;

function GetAssignedToID: Integer;
var
  PosIdx: Integer;
begin
  PosIdx := Pos('-', AssignToStr);
  AssignedToUserName := Copy(AssignToStr, PosIdx+2, 100);
  tblUsers.Locate('UserName', AssignedToUserName, []);
  Result := tblUsers.FieldByName('UserID').AsInteger;
end;

function GetStatusID: Integer;
begin
  tblStatus.Locate('StatusTitle', StatusStr, []);
  Result := tblStatus.FieldByName('StatusID').AsInteger;
end;

procedure DoPostSuccessPage;
begin
  with WebPage do
  begin
    Add(Format('<H1>Thank you %s %s, your bug has been added.</H1>', [UserFName, UserLName]));
    Add(FormatDateTime('"<BR><BR>Bug Entered on:"  mmm dd, yyyy', Date));
    Add(Format('<BR>Bug Assigned to: %s', [AssignedToUserName]));
    Add(Format('<BR>Details: %s', [DetailsStr]));
    Add(Format('<BR>Status: %s', [StatusStr]));
  end;
end;

procedure DoPostFailPage;
begin
  WebPage.Add('<BR>Bug Entry failed.');
end;

begin

  // Retrieve the fields inserted.
  SummaryStr  := Request.QueryFields.Values['Summary'];
  DetailsStr  := Request.QueryFields.Values['Details'];
  AssignToStr := Request.QueryFields.Values['AssignTo'];
  StatusStr   := Request.QueryFields.Values['Status'];


  // Retrieve the cookie fields.
  UserID   := StrToInt(Request.CookieFields.Values['UserID']);
  UserName := Request.CookieFields.Values['UserName'];
  UserFName := Request.CookieFields.Values['UserFirstName'];
  UserLName := Request.CookieFields.Values['UserLastName'];

  // Necessary for the AfterInsert event handler.
  FLoginUserID   := UserID;
  FLoginUserName := UserName;

  InsertBug;
  try
    tblBugs.FieldByName('SummaryDescription').AsString := SummaryStr;
    tblBugs.FieldByName('WhenReported').AsDateTime := Date;
    tblBugs.FieldByName('Details').AsString := DetailsStr;
    tblBugs.FieldByName('AssignedToUserID').AsInteger := GetAssignedToID;
    tblBugs.FieldByName('StatusID').AsInteger := GetStatusID;
    tblBugs.Post;
    PostSucceeded := True;

  except
    tblBugs.Cancel;
    PostSucceeded := False;
  end;

  WebPage := TStringList.Create;
  try
    AddHeader(WebPage);
    with WebPage do
    begin
      Add('<BODY>');

      if PostSucceeded then
        DoPostSuccessPage
      else
        DoPostFailPage;

      AddFooter(WebPage);
      Response.Content :=  WebPage.Text;
      Handled := True;
    end;
  finally
    WebPage.Free;
  end;

end;

end.


